#include <kos.h>
#include "defines.h"
#include "filereq.h"
#include "amigafont.h"


static void strupper ( char * str )
	{
		int i;
		
		for (i=0;i<strlen(str);i++)
			str[i] = toupper ( str[i] );
	}

void amiga_charout ( u8 * buffer, char d, unsigned int x, unsigned int y, u8 fgcolor, u8 bgcolor, BOOL reverse )
	{
		unsigned int i,j;
		unsigned int fontoffs;
		u8 c = d;
		u8 * b = &buffer[(vid_mode->width*y)+x];
		
		if (reverse)
			{
				u8 temp = fgcolor;
				fgcolor = bgcolor;
				bgcolor = temp;
			}
		
		fontoffs = ((unsigned int)c)<<4;
		
		for (i=j=0;i<16;i++,j+=vid_mode->width)
			{
				int k;
				u8 cl = amigafont[fontoffs+i];
        b[j+0] = cl & 0x80 ? fgcolor:bgcolor;
        b[j+1] = cl & 0x40 ? fgcolor:bgcolor;
        b[j+2] = cl & 0x20 ? fgcolor:bgcolor;
        b[j+3] = cl & 0x10 ? fgcolor:bgcolor;
        b[j+4] = cl & 0x08 ? fgcolor:bgcolor;
        b[j+5] = cl & 0x04 ? fgcolor:bgcolor;
        b[j+6] = cl & 0x02 ? fgcolor:bgcolor;
        b[j+7] = cl & 0x01 ? fgcolor:bgcolor;
			}
	}

void amiga_textout ( u8 * buffer, char * t, unsigned int x, unsigned int y, u8 fgcolor, u8 bgcolor, BOOL reverse )
	{
		int i,l;
		
		l=strlen(t);
		for (i=0;i<l;i++)
			amiga_charout ( buffer, t[i], x+(i<<3), y, fgcolor, bgcolor, reverse );
	}

static int getNumDirEntries ( char * dir )
	{
		file_t fh;
		char dname[256];
		int numdir;
		
		strcpy (dname, "/cd");
		strcat (dname, dir);
		
		if ((fh=fs_open(dname, O_DIR|O_RDONLY))==0)
			return -1;

		for (numdir=0;fs_readdir(fh)!=NULL;numdir++);

		fs_close ( fh );
		
		return numdir;
	}

static int readDirectory ( char * dir, dirent_t ** directory )
	{
		int i;
		int numentries = getNumDirEntries ( dir );
		file_t fh;
		char dname[256];
		
		strcpy (dname, "/cd");
		if (dir[0] != '/')
			strcat (dname, "/");
		strcat (dname, dir);

		if (numentries==-1)
			return -1;
		
		if ((fh=fs_open(dname, O_DIR|O_RDONLY))==0)
			return -1;

		*directory = calloc ( sizeof(dirent_t), numentries );
		
		for (i=0;i<numentries;i++)
			{
				dirent_t * entry = fs_readdir ( fh );
				if (entry==NULL)
					break;
/*				printf("%s\n", entry->name);*/
				memcpy ( &((*directory)[i]), entry, sizeof(dirent_t) );
			}
		
		fs_close ( fh );
		
		return i;
	}

static int showDir ( u8 * buf, FSEL * fsel, dirent_t ** pdirectory, int numentries, int index, char * dirname )
	{
		int i;
		int screen_start;
		int index_start;
		char str[129];
		int middle = vid_mode->height/(16*2);

		memset ( buf, fsel->bgcolor, vid_mode->width*vid_mode->height );
		if (index>=numentries)
			index=0;
			
		if (index>middle)
			{
				index_start = index-middle;
				screen_start = 0;
			} else
			{
				index_start = 0;
				screen_start = middle-index;
			}

		amiga_textout ( buf, dirname, 0, 0, fsel->curdircolor, fsel->bgcolor, FALSE );
		for (i=0;(i+screen_start)<((vid_mode->height/16)-1) && (i+index_start)<numentries;i++)
			{
				int res;
				char ts[64];
				int j;

				memset ( str, 0x20, 128 );
				str[vid_mode->width/8] = '\0';

				if (pdirectory[i+index_start]->size>=0)
					{
						sprintf(ts, "%d", pdirectory[i+index_start]->size);
						strcpy ( &str[(vid_mode->width/8)-strlen(ts)], ts );
					}
				memcpy ( str, pdirectory[i+index_start]->name, strlen(pdirectory[i+index_start]->name) );
				strupper ( str );
				amiga_textout ( buf, str, 0, (1+i+screen_start)*16, pdirectory[i+index_start]->size<0 ? fsel->dircolor:fsel->filecolor, fsel->bgcolor, (i+index_start)==index );
			}
		
    for (i=0;i<vid_mode->width*vid_mode->height;i++)
	   	vram_l[i] = fsel->palette[buf[i]];
	   
	  return 0;
	}

static int selectFile ( u8 * buf, FSEL * fsel, dirent_t ** pdirectory, int numentries, char * dirname )
	{
		int index = 0;

		while (1)
			{
				uint8 cont;
        cont_cond_t ctrl;
        
	      cont = maple_first_controller();
        if (cont)
          {
        		showDir ( buf, fsel, pdirectory, numentries, index, dirname );

            do
            	{
	            	while (cont_get_cond ( cont, &ctrl ) < 0);
            	} while ((~ctrl.buttons)&(CONT_DPAD_UP|CONT_DPAD_DOWN|CONT_DPAD_LEFT|CONT_DPAD_RIGHT|CONT_A|CONT_X));

	          do
	          	{
            		while (cont_get_cond ( cont, &ctrl ) < 0);
            	} while (!((~ctrl.buttons)&(CONT_DPAD_UP|CONT_DPAD_DOWN|CONT_DPAD_LEFT|CONT_DPAD_RIGHT|CONT_A|CONT_X)));

            if ( (~ctrl.buttons)&CONT_DPAD_UP )
           		index--;

            if ( (~ctrl.buttons)&CONT_DPAD_DOWN )
            	index++;

            if ( (~ctrl.buttons)&CONT_DPAD_LEFT )
            	index -= vid_mode->height/(16*2);
            
            if ( (~ctrl.buttons)&CONT_DPAD_RIGHT )
            	index += vid_mode->height/(16*2);

            if ( (~ctrl.buttons)&CONT_A )
           		return index;
           	
           	if ( (~ctrl.buttons)&CONT_X )
           		return -1;

            if (index>=numentries)
            	index = numentries-1;
            if (index<0)
            	index = 0;
					}
			}
	}

static void parentDir ( char * dir )
	{
    int len = strlen ( dir )-1;
    int i;

    if (dir[len]=='/')
      len--;
    if (len<=0)
      return;

    for (i=len;i>=0;i--)
      {
        if (dir[i]=='/')
          break;
      }

    if (i==0)
      dir[i++] = '/';

    dir[i] = '\0';
	}

static void sortDirEntries ( dirent_t ** pdirlist, int num )
  {
    int i,j;

    if (num<2)
      return;

    for (i=0;i<num;i++)
      {
        for (j=0;j<num-1;j++)
          {
            if (strcmp(pdirlist[j]->name, pdirlist[j+1]->name)>0)
              {
                dirent_t * temp;
                temp = pdirlist[j];
                pdirlist[j] = pdirlist[j+1];
                pdirlist[j+1] = temp;
              }
          }
      }
  }

int FileRequest ( FSEL * fsel, char * startdir )
	{
    dirent_t * directory;
    dirent_t ** pdirectory;		
    u8 * buf;
    int i,j,num;
    char dir[256];
    
    dir[0] = '\0';
    if (startdir[0]!='/')
    	strcat ( dir, "/" );
		strcat ( dir, startdir );
		
		buf = calloc ( vid_mode->width, vid_mode->height );

		while (1)
			{
				strupper ( dir );
    		if ((num=readDirectory(dir, &directory))!=-1)
    			{
	    			int index;
	    			int numdirs;

	    			pdirectory = calloc ( sizeof(dirent_t*), num );
	    			for (i=0,numdirs=0;i<num;i++)
	    				{
	    					pdirectory[i] = &directory[i];
	    					if (directory[i].size<0)
	    						numdirs++;
    					}
	    			/* Separate dirs from files */
	    			j=0;
	    			while (pdirectory[j]->size<0 && j<num) j++;
	    			for (i=j;i<num;i++)
	    				{
		    				if (pdirectory[i]->size<0)
		    					{
			    					dirent_t * tmp = pdirectory[i];
			    					pdirectory[i] = pdirectory[j];
			    					pdirectory[j] = tmp;
			    					j++;
		    					}
	    				}
	    			sortDirEntries(pdirectory, numdirs);
	    			sortDirEntries(pdirectory, num-numdirs);

	    			index = selectFile ( buf, fsel, pdirectory, num, dir );

	    			if (index==-1)
	    				{
		    				parentDir ( dir );
	    				} else
	    			if (directory[index].size<0)
	    				{
		    				if (dir[strlen(dir)-1]!='/')
		    					strcat ( dir, "/" );
	    					strcat ( dir, pdirectory[index]->name );
	    					index = 0;
    					} else
    					{
	    					strcpy (fsel->fname, "/cd");
	    					if (dir[0]!='/')
	    						strcat (fsel->fname, "/");
	    					strcat (fsel->fname,dir);
	    					if (fsel->fname[strlen(fsel->fname)-1]!='/')
	    						strcat(fsel->fname, "/");
	    					strcat (fsel->fname, pdirectory[index]->name);
	    					strupper ( fsel->fname );
	    					free (directory);
	    					free (pdirectory);
	    					free (buf);
	    					return 0;
    					}
	    			free (directory);
	    			free (pdirectory);
    			} else
    			{
	    			free (buf);
    				return -1;
    			}
    	}
	}
