/* zfrequest.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#include "zfrequest.h"

#define GP_RGB24(r,g,b) (((((r>>3))&0x1f)<<11)|((((g>>3))&0x1f)<<6)|((((b>>3))&0x1f)<<1))

#include "amigafont.h"


static void amiga_charout ( GPDRAWSURFACE * gpDraw, char c, u8 color, int xpos, int ypos, int ystart, int yheight )
  {
    u8 * pixels;
    u8 * fontdata;
    int dest;
    int y;

    if (gpDraw==NULL)
      return;

    pixels = gpDraw->ptbuffer;
    fontdata = &amigafont[(c*16)+ystart];
    dest = (240*xpos)+(240-ypos);

    for (y=0;y<yheight;y++, dest-=1)
      {
        u8 d = fontdata[y];

        if (d&0x80) pixels[dest+(0*240)] = color;
        if (d&0x40) pixels[dest+(1*240)] = color;
        if (d&0x20) pixels[dest+(2*240)] = color;
        if (d&0x10) pixels[dest+(3*240)] = color;
        if (d&0x08) pixels[dest+(4*240)] = color;
        if (d&0x04) pixels[dest+(5*240)] = color;
        if (d&0x02) pixels[dest+(6*240)] = color;
        if (d&0x01) pixels[dest+(7*240)] = color;
      }
  }

int amiga_textout ( GPDRAWSURFACE * gpDraw, const char * text, u8 color, int xpos, int ypos, int yb1, int yb2 )
  {
    int x,i;
    char c;
    int ystart, yheight;

    if (gpDraw==NULL || text==NULL)
      return 0;

    if (yb1<0) yb1=0;
    if (yb1>=240) yb1=0;
    if (yb2<0) yb2=239;
    if (yb2>=240) yb2=239;

    if (yb1>yb2 || ypos<yb1-16 || ypos>=yb2 || xpos<0 || xpos>=320)
      return 0;

    ystart = 0;
    yheight = 16;

    if (ypos+16>yb2)
      {
        yheight -= 16-(yb2-ypos);
      }

    if (ypos<yb1)
      {
        yheight -= yb1-ypos;
        ystart = yb1-ypos;
        ypos+= yb1-ypos;
      }

    for (i=0,x=xpos;(c=text[i])!='\0';i++,x+=8)
      amiga_charout ( gpDraw, c, color, x, ypos, ystart, yheight );

    return i;
  }

void amiga_printf ( GPDRAWSURFACE * gpDraw, int xpos, int ypos, u8 color, int yb1, int yb2, const char * format, ... )
  {
    va_list va;
    char message[1024];

    if (gpDraw==NULL || format==NULL)
      return;

    va_start ( va, format );
    vsprintf ( message, format, va );
    va_end ( va );

    amiga_textout ( gpDraw, message, color, xpos, ypos, yb1, yb2 );
  }



u8 amptab[256][32] =
	{
		/* 00 */ { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
		/* 01 */ { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x02 },
		/* 02 */ { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x04, 0x04 },
		/* 03 */ { 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x02, 0x02, 0x02, 0x02, 0x03, 0x03, 0x03, 0x03, 0x03, 0x04, 0x04, 0x04, 0x04, 0x04, 0x05, 0x05, 0x05, 0x05, 0x05, 0x06, 0x06 },
		/* 04 */ { 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x02, 0x02, 0x02, 0x02, 0x03, 0x03, 0x03, 0x04, 0x04, 0x04, 0x04, 0x05, 0x05, 0x05, 0x05, 0x06, 0x06, 0x06, 0x06, 0x07, 0x07, 0x07, 0x08, 0x08 },
		/* 05 */ { 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x02, 0x02, 0x02, 0x03, 0x03, 0x03, 0x04, 0x04, 0x04, 0x05, 0x05, 0x05, 0x06, 0x06, 0x06, 0x07, 0x07, 0x07, 0x08, 0x08, 0x08, 0x09, 0x09, 0x09, 0x0a, 0x0a },
		/* 06 */ { 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x02, 0x02, 0x03, 0x03, 0x04, 0x04, 0x04, 0x05, 0x05, 0x06, 0x06, 0x06, 0x07, 0x07, 0x08, 0x08, 0x08, 0x09, 0x09, 0x0a, 0x0a, 0x0a, 0x0b, 0x0b, 0x0c, 0x0c },
		/* 07 */ { 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x02, 0x03, 0x03, 0x04, 0x04, 0x05, 0x05, 0x06, 0x06, 0x07, 0x07, 0x07, 0x08, 0x08, 0x09, 0x09, 0x0a, 0x0a, 0x0b, 0x0b, 0x0c, 0x0c, 0x0d, 0x0d, 0x0e, 0x0e },
		/* 08 */ { 0x00, 0x00, 0x01, 0x01, 0x02, 0x02, 0x03, 0x03, 0x04, 0x04, 0x05, 0x05, 0x06, 0x06, 0x07, 0x08, 0x08, 0x09, 0x09, 0x0a, 0x0a, 0x0b, 0x0b, 0x0c, 0x0c, 0x0d, 0x0d, 0x0e, 0x0e, 0x0f, 0x10, 0x10 },
		/* 09 */ { 0x00, 0x00, 0x01, 0x01, 0x02, 0x03, 0x03, 0x04, 0x04, 0x05, 0x06, 0x06, 0x07, 0x07, 0x08, 0x09, 0x09, 0x0a, 0x0a, 0x0b, 0x0c, 0x0c, 0x0d, 0x0d, 0x0e, 0x0f, 0x0f, 0x10, 0x10, 0x11, 0x12, 0x12 },
		/* 0a */ { 0x00, 0x00, 0x01, 0x02, 0x02, 0x03, 0x04, 0x04, 0x05, 0x06, 0x06, 0x07, 0x08, 0x08, 0x09, 0x0a, 0x0a, 0x0b, 0x0c, 0x0c, 0x0d, 0x0e, 0x0e, 0x0f, 0x10, 0x10, 0x11, 0x12, 0x12, 0x13, 0x14, 0x14 },
		/* 0b */ { 0x00, 0x00, 0x01, 0x02, 0x02, 0x03, 0x04, 0x05, 0x05, 0x06, 0x07, 0x08, 0x08, 0x09, 0x0a, 0x0b, 0x0b, 0x0c, 0x0d, 0x0d, 0x0e, 0x0f, 0x10, 0x10, 0x11, 0x12, 0x13, 0x13, 0x14, 0x15, 0x16, 0x16 },
		/* 0c */ { 0x00, 0x00, 0x01, 0x02, 0x03, 0x04, 0x04, 0x05, 0x06, 0x07, 0x08, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x10, 0x11, 0x12, 0x13, 0x14, 0x14, 0x15, 0x16, 0x17, 0x18, 0x18 },
		/* 0d */ { 0x00, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1a },
		/* 0e */ { 0x00, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1c },
		/* 0f */ { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f },
		/* 10 */ { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x20, 0x21 },
		/* 11 */ { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x22, 0x23 },
		/* 12 */ { 0x00, 0x01, 0x02, 0x03, 0x04, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x12, 0x13, 0x14, 0x15, 0x16, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1e, 0x1f, 0x20, 0x21, 0x22, 0x24, 0x25 },
		/* 13 */ { 0x00, 0x01, 0x02, 0x03, 0x05, 0x06, 0x07, 0x08, 0x0a, 0x0b, 0x0c, 0x0d, 0x0f, 0x10, 0x11, 0x13, 0x14, 0x15, 0x16, 0x18, 0x19, 0x1a, 0x1b, 0x1d, 0x1e, 0x1f, 0x20, 0x22, 0x23, 0x24, 0x26, 0x27 },
		/* 14 */ { 0x00, 0x01, 0x02, 0x04, 0x05, 0x06, 0x08, 0x09, 0x0a, 0x0c, 0x0d, 0x0e, 0x10, 0x11, 0x12, 0x14, 0x15, 0x16, 0x18, 0x19, 0x1a, 0x1c, 0x1d, 0x1e, 0x20, 0x21, 0x22, 0x24, 0x25, 0x26, 0x28, 0x29 },
		/* 15 */ { 0x00, 0x01, 0x02, 0x04, 0x05, 0x07, 0x08, 0x09, 0x0b, 0x0c, 0x0e, 0x0f, 0x10, 0x12, 0x13, 0x15, 0x16, 0x17, 0x19, 0x1a, 0x1c, 0x1d, 0x1e, 0x20, 0x21, 0x23, 0x24, 0x25, 0x27, 0x28, 0x2a, 0x2b },
		/* 16 */ { 0x00, 0x01, 0x02, 0x04, 0x05, 0x07, 0x08, 0x0a, 0x0b, 0x0d, 0x0e, 0x10, 0x11, 0x13, 0x14, 0x16, 0x17, 0x18, 0x1a, 0x1b, 0x1d, 0x1e, 0x20, 0x21, 0x23, 0x24, 0x26, 0x27, 0x29, 0x2a, 0x2c, 0x2d },
		/* 17 */ { 0x00, 0x01, 0x03, 0x04, 0x06, 0x07, 0x09, 0x0a, 0x0c, 0x0d, 0x0f, 0x10, 0x12, 0x13, 0x15, 0x17, 0x18, 0x1a, 0x1b, 0x1d, 0x1e, 0x20, 0x21, 0x23, 0x24, 0x26, 0x27, 0x29, 0x2a, 0x2c, 0x2e, 0x2f },
		/* 18 */ { 0x00, 0x01, 0x03, 0x04, 0x06, 0x08, 0x09, 0x0b, 0x0c, 0x0e, 0x10, 0x11, 0x13, 0x14, 0x16, 0x18, 0x19, 0x1b, 0x1c, 0x1e, 0x20, 0x21, 0x23, 0x24, 0x26, 0x28, 0x29, 0x2b, 0x2c, 0x2e, 0x30, 0x31 },
		/* 19 */ { 0x00, 0x01, 0x03, 0x05, 0x06, 0x08, 0x0a, 0x0b, 0x0d, 0x0f, 0x10, 0x12, 0x14, 0x15, 0x17, 0x19, 0x1a, 0x1c, 0x1e, 0x1f, 0x21, 0x23, 0x24, 0x26, 0x28, 0x29, 0x2b, 0x2d, 0x2e, 0x30, 0x32, 0x33 },
		/* 1a */ { 0x00, 0x01, 0x03, 0x05, 0x06, 0x08, 0x0a, 0x0c, 0x0d, 0x0f, 0x11, 0x13, 0x14, 0x16, 0x18, 0x1a, 0x1b, 0x1d, 0x1f, 0x20, 0x22, 0x24, 0x26, 0x27, 0x29, 0x2b, 0x2d, 0x2e, 0x30, 0x32, 0x34, 0x35 },
		/* 1b */ { 0x00, 0x01, 0x03, 0x05, 0x07, 0x09, 0x0a, 0x0c, 0x0e, 0x10, 0x12, 0x13, 0x15, 0x17, 0x19, 0x1b, 0x1c, 0x1e, 0x20, 0x22, 0x24, 0x25, 0x27, 0x29, 0x2b, 0x2d, 0x2e, 0x30, 0x32, 0x34, 0x36, 0x37 },
		/* 1c */ { 0x00, 0x01, 0x03, 0x05, 0x07, 0x09, 0x0b, 0x0d, 0x0e, 0x10, 0x12, 0x14, 0x16, 0x18, 0x1a, 0x1c, 0x1d, 0x1f, 0x21, 0x23, 0x25, 0x27, 0x29, 0x2a, 0x2c, 0x2e, 0x30, 0x32, 0x34, 0x36, 0x38, 0x39 },
		/* 1d */ { 0x00, 0x01, 0x03, 0x05, 0x07, 0x09, 0x0b, 0x0d, 0x0f, 0x11, 0x13, 0x15, 0x17, 0x19, 0x1b, 0x1d, 0x1e, 0x20, 0x22, 0x24, 0x26, 0x28, 0x2a, 0x2c, 0x2e, 0x30, 0x32, 0x34, 0x36, 0x38, 0x3a, 0x3b },
		/* 1e */ { 0x00, 0x02, 0x04, 0x06, 0x08, 0x0a, 0x0c, 0x0e, 0x10, 0x12, 0x14, 0x16, 0x18, 0x1a, 0x1c, 0x1e, 0x20, 0x22, 0x24, 0x26, 0x28, 0x2a, 0x2c, 0x2e, 0x30, 0x32, 0x34, 0x36, 0x38, 0x3a, 0x3c, 0x3e },
		/* 1f */ { 0x00, 0x02, 0x04, 0x06, 0x08, 0x0a, 0x0c, 0x0e, 0x10, 0x12, 0x14, 0x16, 0x18, 0x1a, 0x1c, 0x1f, 0x21, 0x23, 0x25, 0x27, 0x29, 0x2b, 0x2d, 0x2f, 0x31, 0x33, 0x35, 0x37, 0x39, 0x3b, 0x3e, 0x40 },
		/* 20 */ { 0x00, 0x02, 0x04, 0x06, 0x08, 0x0a, 0x0c, 0x0e, 0x11, 0x13, 0x15, 0x17, 0x19, 0x1b, 0x1d, 0x20, 0x22, 0x24, 0x26, 0x28, 0x2a, 0x2c, 0x2e, 0x31, 0x33, 0x35, 0x37, 0x39, 0x3b, 0x3d, 0x40, 0x42 },
		/* 21 */ { 0x00, 0x02, 0x04, 0x06, 0x08, 0x0b, 0x0d, 0x0f, 0x11, 0x13, 0x16, 0x18, 0x1a, 0x1c, 0x1e, 0x21, 0x23, 0x25, 0x27, 0x29, 0x2c, 0x2e, 0x30, 0x32, 0x34, 0x37, 0x39, 0x3b, 0x3d, 0x3f, 0x42, 0x44 },
		/* 22 */ { 0x00, 0x02, 0x04, 0x06, 0x09, 0x0b, 0x0d, 0x0f, 0x12, 0x14, 0x16, 0x18, 0x1b, 0x1d, 0x1f, 0x22, 0x24, 0x26, 0x28, 0x2b, 0x2d, 0x2f, 0x31, 0x34, 0x36, 0x38, 0x3a, 0x3d, 0x3f, 0x41, 0x44, 0x46 },
		/* 23 */ { 0x00, 0x02, 0x04, 0x07, 0x09, 0x0b, 0x0e, 0x10, 0x12, 0x15, 0x17, 0x19, 0x1c, 0x1e, 0x20, 0x23, 0x25, 0x27, 0x2a, 0x2c, 0x2e, 0x31, 0x33, 0x35, 0x38, 0x3a, 0x3c, 0x3f, 0x41, 0x43, 0x46, 0x48 },
		/* 24 */ { 0x00, 0x02, 0x04, 0x07, 0x09, 0x0c, 0x0e, 0x10, 0x13, 0x15, 0x18, 0x1a, 0x1c, 0x1f, 0x21, 0x24, 0x26, 0x28, 0x2b, 0x2d, 0x30, 0x32, 0x34, 0x37, 0x39, 0x3c, 0x3e, 0x40, 0x43, 0x45, 0x48, 0x4a },
		/* 25 */ { 0x00, 0x02, 0x04, 0x07, 0x09, 0x0c, 0x0e, 0x11, 0x13, 0x16, 0x18, 0x1b, 0x1d, 0x20, 0x22, 0x25, 0x27, 0x29, 0x2c, 0x2e, 0x31, 0x33, 0x36, 0x38, 0x3b, 0x3d, 0x40, 0x42, 0x45, 0x47, 0x4a, 0x4c },
		/* 26 */ { 0x00, 0x02, 0x05, 0x07, 0x0a, 0x0c, 0x0f, 0x11, 0x14, 0x16, 0x19, 0x1b, 0x1e, 0x20, 0x23, 0x26, 0x28, 0x2b, 0x2d, 0x30, 0x32, 0x35, 0x37, 0x3a, 0x3c, 0x3f, 0x41, 0x44, 0x46, 0x49, 0x4c, 0x4e },
		/* 27 */ { 0x00, 0x02, 0x05, 0x07, 0x0a, 0x0d, 0x0f, 0x12, 0x14, 0x17, 0x1a, 0x1c, 0x1f, 0x21, 0x24, 0x27, 0x29, 0x2c, 0x2e, 0x31, 0x34, 0x36, 0x39, 0x3b, 0x3e, 0x41, 0x43, 0x46, 0x48, 0x4b, 0x4e, 0x50 },
		/* 28 */ { 0x00, 0x02, 0x05, 0x08, 0x0a, 0x0d, 0x10, 0x12, 0x15, 0x18, 0x1a, 0x1d, 0x20, 0x22, 0x25, 0x28, 0x2a, 0x2d, 0x30, 0x32, 0x35, 0x38, 0x3a, 0x3d, 0x40, 0x42, 0x45, 0x48, 0x4a, 0x4d, 0x50, 0x52 },
		/* 29 */ { 0x00, 0x02, 0x05, 0x08, 0x0a, 0x0d, 0x10, 0x13, 0x15, 0x18, 0x1b, 0x1e, 0x20, 0x23, 0x26, 0x29, 0x2b, 0x2e, 0x31, 0x33, 0x36, 0x39, 0x3c, 0x3e, 0x41, 0x44, 0x47, 0x49, 0x4c, 0x4f, 0x52, 0x54 },
		/* 2a */ { 0x00, 0x02, 0x05, 0x08, 0x0b, 0x0e, 0x10, 0x13, 0x16, 0x19, 0x1c, 0x1e, 0x21, 0x24, 0x27, 0x2a, 0x2c, 0x2f, 0x32, 0x35, 0x38, 0x3a, 0x3d, 0x40, 0x43, 0x46, 0x48, 0x4b, 0x4e, 0x51, 0x54, 0x56 },
		/* 2b */ { 0x00, 0x02, 0x05, 0x08, 0x0b, 0x0e, 0x11, 0x14, 0x16, 0x19, 0x1c, 0x1f, 0x22, 0x25, 0x28, 0x2b, 0x2d, 0x30, 0x33, 0x36, 0x39, 0x3c, 0x3f, 0x41, 0x44, 0x47, 0x4a, 0x4d, 0x50, 0x53, 0x56, 0x58 },
		/* 2c */ { 0x00, 0x02, 0x05, 0x08, 0x0b, 0x0e, 0x11, 0x14, 0x17, 0x1a, 0x1d, 0x20, 0x23, 0x26, 0x29, 0x2c, 0x2e, 0x31, 0x34, 0x37, 0x3a, 0x3d, 0x40, 0x43, 0x46, 0x49, 0x4c, 0x4f, 0x52, 0x55, 0x58, 0x5a },
		/* 2d */ { 0x00, 0x03, 0x06, 0x09, 0x0c, 0x0f, 0x12, 0x15, 0x18, 0x1b, 0x1e, 0x21, 0x24, 0x27, 0x2a, 0x2d, 0x30, 0x33, 0x36, 0x39, 0x3c, 0x3f, 0x42, 0x45, 0x48, 0x4b, 0x4e, 0x51, 0x54, 0x57, 0x5a, 0x5d },
		/* 2e */ { 0x00, 0x03, 0x06, 0x09, 0x0c, 0x0f, 0x12, 0x15, 0x18, 0x1b, 0x1e, 0x21, 0x24, 0x27, 0x2a, 0x2e, 0x31, 0x34, 0x37, 0x3a, 0x3d, 0x40, 0x43, 0x46, 0x49, 0x4c, 0x4f, 0x52, 0x55, 0x58, 0x5c, 0x5f },
		/* 2f */ { 0x00, 0x03, 0x06, 0x09, 0x0c, 0x0f, 0x12, 0x15, 0x19, 0x1c, 0x1f, 0x22, 0x25, 0x28, 0x2b, 0x2f, 0x32, 0x35, 0x38, 0x3b, 0x3e, 0x41, 0x44, 0x48, 0x4b, 0x4e, 0x51, 0x54, 0x57, 0x5a, 0x5e, 0x61 },
		/* 30 */ { 0x00, 0x03, 0x06, 0x09, 0x0c, 0x10, 0x13, 0x16, 0x19, 0x1c, 0x20, 0x23, 0x26, 0x29, 0x2c, 0x30, 0x33, 0x36, 0x39, 0x3c, 0x40, 0x43, 0x46, 0x49, 0x4c, 0x50, 0x53, 0x56, 0x59, 0x5c, 0x60, 0x63 },
		/* 31 */ { 0x00, 0x03, 0x06, 0x09, 0x0d, 0x10, 0x13, 0x16, 0x1a, 0x1d, 0x20, 0x23, 0x27, 0x2a, 0x2d, 0x31, 0x34, 0x37, 0x3a, 0x3e, 0x41, 0x44, 0x47, 0x4b, 0x4e, 0x51, 0x54, 0x58, 0x5b, 0x5e, 0x62, 0x65 },
		/* 32 */ { 0x00, 0x03, 0x06, 0x0a, 0x0d, 0x10, 0x14, 0x17, 0x1a, 0x1e, 0x21, 0x24, 0x28, 0x2b, 0x2e, 0x32, 0x35, 0x38, 0x3c, 0x3f, 0x42, 0x46, 0x49, 0x4c, 0x50, 0x53, 0x56, 0x5a, 0x5d, 0x60, 0x64, 0x67 },
		/* 33 */ { 0x00, 0x03, 0x06, 0x0a, 0x0d, 0x11, 0x14, 0x17, 0x1b, 0x1e, 0x22, 0x25, 0x28, 0x2c, 0x2f, 0x33, 0x36, 0x39, 0x3d, 0x40, 0x44, 0x47, 0x4a, 0x4e, 0x51, 0x55, 0x58, 0x5b, 0x5f, 0x62, 0x66, 0x69 },
		/* 34 */ { 0x00, 0x03, 0x06, 0x0a, 0x0d, 0x11, 0x14, 0x18, 0x1b, 0x1f, 0x22, 0x26, 0x29, 0x2d, 0x30, 0x34, 0x37, 0x3a, 0x3e, 0x41, 0x45, 0x48, 0x4c, 0x4f, 0x53, 0x56, 0x5a, 0x5d, 0x61, 0x64, 0x68, 0x6b },
		/* 35 */ { 0x00, 0x03, 0x07, 0x0a, 0x0e, 0x11, 0x15, 0x18, 0x1c, 0x1f, 0x23, 0x26, 0x2a, 0x2d, 0x31, 0x35, 0x38, 0x3c, 0x3f, 0x43, 0x46, 0x4a, 0x4d, 0x51, 0x54, 0x58, 0x5b, 0x5f, 0x62, 0x66, 0x6a, 0x6d },
		/* 36 */ { 0x00, 0x03, 0x07, 0x0a, 0x0e, 0x12, 0x15, 0x19, 0x1c, 0x20, 0x24, 0x27, 0x2b, 0x2e, 0x32, 0x36, 0x39, 0x3d, 0x40, 0x44, 0x48, 0x4b, 0x4f, 0x52, 0x56, 0x5a, 0x5d, 0x61, 0x64, 0x68, 0x6c, 0x6f },
		/* 37 */ { 0x00, 0x03, 0x07, 0x0b, 0x0e, 0x12, 0x16, 0x19, 0x1d, 0x21, 0x24, 0x28, 0x2c, 0x2f, 0x33, 0x37, 0x3a, 0x3e, 0x42, 0x45, 0x49, 0x4d, 0x50, 0x54, 0x58, 0x5b, 0x5f, 0x63, 0x66, 0x6a, 0x6e, 0x71 },
		/* 38 */ { 0x00, 0x03, 0x07, 0x0b, 0x0e, 0x12, 0x16, 0x1a, 0x1d, 0x21, 0x25, 0x29, 0x2c, 0x30, 0x34, 0x38, 0x3b, 0x3f, 0x43, 0x46, 0x4a, 0x4e, 0x52, 0x55, 0x59, 0x5d, 0x61, 0x64, 0x68, 0x6c, 0x70, 0x73 },
		/* 39 */ { 0x00, 0x03, 0x07, 0x0b, 0x0f, 0x13, 0x16, 0x1a, 0x1e, 0x22, 0x26, 0x29, 0x2d, 0x31, 0x35, 0x39, 0x3c, 0x40, 0x44, 0x48, 0x4c, 0x4f, 0x53, 0x57, 0x5b, 0x5f, 0x62, 0x66, 0x6a, 0x6e, 0x72, 0x75 },
		/* 3a */ { 0x00, 0x03, 0x07, 0x0b, 0x0f, 0x13, 0x17, 0x1b, 0x1e, 0x22, 0x26, 0x2a, 0x2e, 0x32, 0x36, 0x3a, 0x3d, 0x41, 0x45, 0x49, 0x4d, 0x51, 0x55, 0x58, 0x5c, 0x60, 0x64, 0x68, 0x6c, 0x70, 0x74, 0x77 },
		/* 3b */ { 0x00, 0x03, 0x07, 0x0b, 0x0f, 0x13, 0x17, 0x1b, 0x1f, 0x23, 0x27, 0x2b, 0x2f, 0x33, 0x37, 0x3b, 0x3e, 0x42, 0x46, 0x4a, 0x4e, 0x52, 0x56, 0x5a, 0x5e, 0x62, 0x66, 0x6a, 0x6e, 0x72, 0x76, 0x79 },
		/* 3c */ { 0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c, 0x20, 0x24, 0x28, 0x2c, 0x30, 0x34, 0x38, 0x3c, 0x40, 0x44, 0x48, 0x4c, 0x50, 0x54, 0x58, 0x5c, 0x60, 0x64, 0x68, 0x6c, 0x70, 0x74, 0x78, 0x7c },
		/* 3d */ { 0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c, 0x20, 0x24, 0x28, 0x2c, 0x30, 0x34, 0x38, 0x3d, 0x41, 0x45, 0x49, 0x4d, 0x51, 0x55, 0x59, 0x5d, 0x61, 0x65, 0x69, 0x6d, 0x71, 0x75, 0x7a, 0x7e },
		/* 3e */ { 0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c, 0x21, 0x25, 0x29, 0x2d, 0x31, 0x35, 0x39, 0x3e, 0x42, 0x46, 0x4a, 0x4e, 0x52, 0x56, 0x5a, 0x5f, 0x63, 0x67, 0x6b, 0x6f, 0x73, 0x77, 0x7c, 0x80 },
		/* 3f */ { 0x00, 0x04, 0x08, 0x0c, 0x10, 0x15, 0x19, 0x1d, 0x21, 0x25, 0x2a, 0x2e, 0x32, 0x36, 0x3a, 0x3f, 0x43, 0x47, 0x4b, 0x4f, 0x54, 0x58, 0x5c, 0x60, 0x64, 0x69, 0x6d, 0x71, 0x75, 0x79, 0x7e, 0x82 },
		/* 40 */ { 0x00, 0x04, 0x08, 0x0c, 0x11, 0x15, 0x19, 0x1d, 0x22, 0x26, 0x2a, 0x2e, 0x33, 0x37, 0x3b, 0x40, 0x44, 0x48, 0x4c, 0x51, 0x55, 0x59, 0x5d, 0x62, 0x66, 0x6a, 0x6e, 0x73, 0x77, 0x7b, 0x80, 0x84 },
		/* 41 */ { 0x00, 0x04, 0x08, 0x0d, 0x11, 0x15, 0x1a, 0x1e, 0x22, 0x27, 0x2b, 0x2f, 0x34, 0x38, 0x3c, 0x41, 0x45, 0x49, 0x4e, 0x52, 0x56, 0x5b, 0x5f, 0x63, 0x68, 0x6c, 0x70, 0x75, 0x79, 0x7d, 0x82, 0x86 },
		/* 42 */ { 0x00, 0x04, 0x08, 0x0d, 0x11, 0x16, 0x1a, 0x1e, 0x23, 0x27, 0x2c, 0x30, 0x34, 0x39, 0x3d, 0x42, 0x46, 0x4a, 0x4f, 0x53, 0x58, 0x5c, 0x60, 0x65, 0x69, 0x6e, 0x72, 0x76, 0x7b, 0x7f, 0x84, 0x88 },
		/* 43 */ { 0x00, 0x04, 0x08, 0x0d, 0x11, 0x16, 0x1a, 0x1f, 0x23, 0x28, 0x2c, 0x31, 0x35, 0x3a, 0x3e, 0x43, 0x47, 0x4b, 0x50, 0x54, 0x59, 0x5d, 0x62, 0x66, 0x6b, 0x6f, 0x74, 0x78, 0x7d, 0x81, 0x86, 0x8a },
		/* 44 */ { 0x00, 0x04, 0x09, 0x0d, 0x12, 0x16, 0x1b, 0x1f, 0x24, 0x28, 0x2d, 0x31, 0x36, 0x3a, 0x3f, 0x44, 0x48, 0x4d, 0x51, 0x56, 0x5a, 0x5f, 0x63, 0x68, 0x6c, 0x71, 0x75, 0x7a, 0x7e, 0x83, 0x88, 0x8c },
		/* 45 */ { 0x00, 0x04, 0x09, 0x0d, 0x12, 0x17, 0x1b, 0x20, 0x24, 0x29, 0x2e, 0x32, 0x37, 0x3b, 0x40, 0x45, 0x49, 0x4e, 0x52, 0x57, 0x5c, 0x60, 0x65, 0x69, 0x6e, 0x73, 0x77, 0x7c, 0x80, 0x85, 0x8a, 0x8e },
		/* 46 */ { 0x00, 0x04, 0x09, 0x0e, 0x12, 0x17, 0x1c, 0x20, 0x25, 0x2a, 0x2e, 0x33, 0x38, 0x3c, 0x41, 0x46, 0x4a, 0x4f, 0x54, 0x58, 0x5d, 0x62, 0x66, 0x6b, 0x70, 0x74, 0x79, 0x7e, 0x82, 0x87, 0x8c, 0x90 },
		/* 47 */ { 0x00, 0x04, 0x09, 0x0e, 0x12, 0x17, 0x1c, 0x21, 0x25, 0x2a, 0x2f, 0x34, 0x38, 0x3d, 0x42, 0x47, 0x4b, 0x50, 0x55, 0x59, 0x5e, 0x63, 0x68, 0x6c, 0x71, 0x76, 0x7b, 0x7f, 0x84, 0x89, 0x8e, 0x92 },
		/* 48 */ { 0x00, 0x04, 0x09, 0x0e, 0x13, 0x18, 0x1c, 0x21, 0x26, 0x2b, 0x30, 0x34, 0x39, 0x3e, 0x43, 0x48, 0x4c, 0x51, 0x56, 0x5b, 0x60, 0x64, 0x69, 0x6e, 0x73, 0x78, 0x7c, 0x81, 0x86, 0x8b, 0x90, 0x94 },
		/* 49 */ { 0x00, 0x04, 0x09, 0x0e, 0x13, 0x18, 0x1d, 0x22, 0x26, 0x2b, 0x30, 0x35, 0x3a, 0x3f, 0x44, 0x49, 0x4d, 0x52, 0x57, 0x5c, 0x61, 0x66, 0x6b, 0x6f, 0x74, 0x79, 0x7e, 0x83, 0x88, 0x8d, 0x92, 0x96 },
		/* 4a */ { 0x00, 0x04, 0x09, 0x0e, 0x13, 0x18, 0x1d, 0x22, 0x27, 0x2c, 0x31, 0x36, 0x3b, 0x40, 0x45, 0x4a, 0x4e, 0x53, 0x58, 0x5d, 0x62, 0x67, 0x6c, 0x71, 0x76, 0x7b, 0x80, 0x85, 0x8a, 0x8f, 0x94, 0x98 },
		/* 4b */ { 0x00, 0x05, 0x0a, 0x0f, 0x14, 0x19, 0x1e, 0x23, 0x28, 0x2d, 0x32, 0x37, 0x3c, 0x41, 0x46, 0x4b, 0x50, 0x55, 0x5a, 0x5f, 0x64, 0x69, 0x6e, 0x73, 0x78, 0x7d, 0x82, 0x87, 0x8c, 0x91, 0x96, 0x9b },
		/* 4c */ { 0x00, 0x05, 0x0a, 0x0f, 0x14, 0x19, 0x1e, 0x23, 0x28, 0x2d, 0x32, 0x37, 0x3c, 0x41, 0x46, 0x4c, 0x51, 0x56, 0x5b, 0x60, 0x65, 0x6a, 0x6f, 0x74, 0x79, 0x7e, 0x83, 0x88, 0x8d, 0x92, 0x98, 0x9d },
		/* 4d */ { 0x00, 0x05, 0x0a, 0x0f, 0x14, 0x19, 0x1e, 0x23, 0x29, 0x2e, 0x33, 0x38, 0x3d, 0x42, 0x47, 0x4d, 0x52, 0x57, 0x5c, 0x61, 0x66, 0x6b, 0x70, 0x76, 0x7b, 0x80, 0x85, 0x8a, 0x8f, 0x94, 0x9a, 0x9f },
		/* 4e */ { 0x00, 0x05, 0x0a, 0x0f, 0x14, 0x1a, 0x1f, 0x24, 0x29, 0x2e, 0x34, 0x39, 0x3e, 0x43, 0x48, 0x4e, 0x53, 0x58, 0x5d, 0x62, 0x68, 0x6d, 0x72, 0x77, 0x7c, 0x82, 0x87, 0x8c, 0x91, 0x96, 0x9c, 0xa1 },
		/* 4f */ { 0x00, 0x05, 0x0a, 0x0f, 0x15, 0x1a, 0x1f, 0x24, 0x2a, 0x2f, 0x34, 0x39, 0x3f, 0x44, 0x49, 0x4f, 0x54, 0x59, 0x5e, 0x64, 0x69, 0x6e, 0x73, 0x79, 0x7e, 0x83, 0x88, 0x8e, 0x93, 0x98, 0x9e, 0xa3 },
		/* 50 */ { 0x00, 0x05, 0x0a, 0x10, 0x15, 0x1a, 0x20, 0x25, 0x2a, 0x30, 0x35, 0x3a, 0x40, 0x45, 0x4a, 0x50, 0x55, 0x5a, 0x60, 0x65, 0x6a, 0x70, 0x75, 0x7a, 0x80, 0x85, 0x8a, 0x90, 0x95, 0x9a, 0xa0, 0xa5 },
		/* 51 */ { 0x00, 0x05, 0x0a, 0x10, 0x15, 0x1b, 0x20, 0x25, 0x2b, 0x30, 0x36, 0x3b, 0x40, 0x46, 0x4b, 0x51, 0x56, 0x5b, 0x61, 0x66, 0x6c, 0x71, 0x76, 0x7c, 0x81, 0x87, 0x8c, 0x91, 0x97, 0x9c, 0xa2, 0xa7 },
		/* 52 */ { 0x00, 0x05, 0x0a, 0x10, 0x15, 0x1b, 0x20, 0x26, 0x2b, 0x31, 0x36, 0x3c, 0x41, 0x47, 0x4c, 0x52, 0x57, 0x5c, 0x62, 0x67, 0x6d, 0x72, 0x78, 0x7d, 0x83, 0x88, 0x8e, 0x93, 0x99, 0x9e, 0xa4, 0xa9 },
		/* 53 */ { 0x00, 0x05, 0x0b, 0x10, 0x16, 0x1b, 0x21, 0x26, 0x2c, 0x31, 0x37, 0x3c, 0x42, 0x47, 0x4d, 0x53, 0x58, 0x5e, 0x63, 0x69, 0x6e, 0x74, 0x79, 0x7f, 0x84, 0x8a, 0x8f, 0x95, 0x9a, 0xa0, 0xa6, 0xab },
		/* 54 */ { 0x00, 0x05, 0x0b, 0x10, 0x16, 0x1c, 0x21, 0x27, 0x2c, 0x32, 0x38, 0x3d, 0x43, 0x48, 0x4e, 0x54, 0x59, 0x5f, 0x64, 0x6a, 0x70, 0x75, 0x7b, 0x80, 0x86, 0x8c, 0x91, 0x97, 0x9c, 0xa2, 0xa8, 0xad },
		/* 55 */ { 0x00, 0x05, 0x0b, 0x11, 0x16, 0x1c, 0x22, 0x27, 0x2d, 0x33, 0x38, 0x3e, 0x44, 0x49, 0x4f, 0x55, 0x5a, 0x60, 0x66, 0x6b, 0x71, 0x77, 0x7c, 0x82, 0x88, 0x8d, 0x93, 0x99, 0x9e, 0xa4, 0xaa, 0xaf },
		/* 56 */ { 0x00, 0x05, 0x0b, 0x11, 0x16, 0x1c, 0x22, 0x28, 0x2d, 0x33, 0x39, 0x3f, 0x44, 0x4a, 0x50, 0x56, 0x5b, 0x61, 0x67, 0x6c, 0x72, 0x78, 0x7e, 0x83, 0x89, 0x8f, 0x95, 0x9a, 0xa0, 0xa6, 0xac, 0xb1 },
		/* 57 */ { 0x00, 0x05, 0x0b, 0x11, 0x17, 0x1d, 0x22, 0x28, 0x2e, 0x34, 0x3a, 0x3f, 0x45, 0x4b, 0x51, 0x57, 0x5c, 0x62, 0x68, 0x6e, 0x74, 0x79, 0x7f, 0x85, 0x8b, 0x91, 0x96, 0x9c, 0xa2, 0xa8, 0xae, 0xb3 },
		/* 58 */ { 0x00, 0x05, 0x0b, 0x11, 0x17, 0x1d, 0x23, 0x29, 0x2e, 0x34, 0x3a, 0x40, 0x46, 0x4c, 0x52, 0x58, 0x5d, 0x63, 0x69, 0x6f, 0x75, 0x7b, 0x81, 0x86, 0x8c, 0x92, 0x98, 0x9e, 0xa4, 0xaa, 0xb0, 0xb5 },
		/* 59 */ { 0x00, 0x05, 0x0b, 0x11, 0x17, 0x1d, 0x23, 0x29, 0x2f, 0x35, 0x3b, 0x41, 0x47, 0x4d, 0x53, 0x59, 0x5e, 0x64, 0x6a, 0x70, 0x76, 0x7c, 0x82, 0x88, 0x8e, 0x94, 0x9a, 0xa0, 0xa6, 0xac, 0xb2, 0xb7 },
		/* 5a */ { 0x00, 0x06, 0x0c, 0x12, 0x18, 0x1e, 0x24, 0x2a, 0x30, 0x36, 0x3c, 0x42, 0x48, 0x4e, 0x54, 0x5a, 0x60, 0x66, 0x6c, 0x72, 0x78, 0x7e, 0x84, 0x8a, 0x90, 0x96, 0x9c, 0xa2, 0xa8, 0xae, 0xb4, 0xba },
		/* 5b */ { 0x00, 0x06, 0x0c, 0x12, 0x18, 0x1e, 0x24, 0x2a, 0x30, 0x36, 0x3c, 0x42, 0x48, 0x4e, 0x54, 0x5b, 0x61, 0x67, 0x6d, 0x73, 0x79, 0x7f, 0x85, 0x8b, 0x91, 0x97, 0x9d, 0xa3, 0xa9, 0xaf, 0xb6, 0xbc },
		/* 5c */ { 0x00, 0x06, 0x0c, 0x12, 0x18, 0x1e, 0x24, 0x2a, 0x31, 0x37, 0x3d, 0x43, 0x49, 0x4f, 0x55, 0x5c, 0x62, 0x68, 0x6e, 0x74, 0x7a, 0x80, 0x86, 0x8d, 0x93, 0x99, 0x9f, 0xa5, 0xab, 0xb1, 0xb8, 0xbe },
		/* 5d */ { 0x00, 0x06, 0x0c, 0x12, 0x18, 0x1f, 0x25, 0x2b, 0x31, 0x37, 0x3e, 0x44, 0x4a, 0x50, 0x56, 0x5d, 0x63, 0x69, 0x6f, 0x75, 0x7c, 0x82, 0x88, 0x8e, 0x94, 0x9b, 0xa1, 0xa7, 0xad, 0xb3, 0xba, 0xc0 },
		/* 5e */ { 0x00, 0x06, 0x0c, 0x12, 0x19, 0x1f, 0x25, 0x2b, 0x32, 0x38, 0x3e, 0x44, 0x4b, 0x51, 0x57, 0x5e, 0x64, 0x6a, 0x70, 0x77, 0x7d, 0x83, 0x89, 0x90, 0x96, 0x9c, 0xa2, 0xa9, 0xaf, 0xb5, 0xbc, 0xc2 },
		/* 5f */ { 0x00, 0x06, 0x0c, 0x13, 0x19, 0x1f, 0x26, 0x2c, 0x32, 0x39, 0x3f, 0x45, 0x4c, 0x52, 0x58, 0x5f, 0x65, 0x6b, 0x72, 0x78, 0x7e, 0x85, 0x8b, 0x91, 0x98, 0x9e, 0xa4, 0xab, 0xb1, 0xb7, 0xbe, 0xc4 },
		/* 60 */ { 0x00, 0x06, 0x0c, 0x13, 0x19, 0x20, 0x26, 0x2c, 0x33, 0x39, 0x40, 0x46, 0x4c, 0x53, 0x59, 0x60, 0x66, 0x6c, 0x73, 0x79, 0x80, 0x86, 0x8c, 0x93, 0x99, 0xa0, 0xa6, 0xac, 0xb3, 0xb9, 0xc0, 0xc6 },
		/* 61 */ { 0x00, 0x06, 0x0c, 0x13, 0x19, 0x20, 0x26, 0x2d, 0x33, 0x3a, 0x40, 0x47, 0x4d, 0x54, 0x5a, 0x61, 0x67, 0x6d, 0x74, 0x7a, 0x81, 0x87, 0x8e, 0x94, 0x9b, 0xa1, 0xa8, 0xae, 0xb5, 0xbb, 0xc2, 0xc8 },
		/* 62 */ { 0x00, 0x06, 0x0d, 0x13, 0x1a, 0x20, 0x27, 0x2d, 0x34, 0x3a, 0x41, 0x47, 0x4e, 0x54, 0x5b, 0x62, 0x68, 0x6f, 0x75, 0x7c, 0x82, 0x89, 0x8f, 0x96, 0x9c, 0xa3, 0xa9, 0xb0, 0xb6, 0xbd, 0xc4, 0xca },
		/* 63 */ { 0x00, 0x06, 0x0d, 0x13, 0x1a, 0x21, 0x27, 0x2e, 0x34, 0x3b, 0x42, 0x48, 0x4f, 0x55, 0x5c, 0x63, 0x69, 0x70, 0x76, 0x7d, 0x84, 0x8a, 0x91, 0x97, 0x9e, 0xa5, 0xab, 0xb2, 0xb8, 0xbf, 0xc6, 0xcc },
		/* 64 */ { 0x00, 0x06, 0x0d, 0x14, 0x1a, 0x21, 0x28, 0x2e, 0x35, 0x3c, 0x42, 0x49, 0x50, 0x56, 0x5d, 0x64, 0x6a, 0x71, 0x78, 0x7e, 0x85, 0x8c, 0x92, 0x99, 0xa0, 0xa6, 0xad, 0xb4, 0xba, 0xc1, 0xc8, 0xce },
		/* 65 */ { 0x00, 0x06, 0x0d, 0x14, 0x1a, 0x21, 0x28, 0x2f, 0x35, 0x3c, 0x43, 0x4a, 0x50, 0x57, 0x5e, 0x65, 0x6b, 0x72, 0x79, 0x7f, 0x86, 0x8d, 0x94, 0x9a, 0xa1, 0xa8, 0xaf, 0xb5, 0xbc, 0xc3, 0xca, 0xd0 },
		/* 66 */ { 0x00, 0x06, 0x0d, 0x14, 0x1b, 0x22, 0x28, 0x2f, 0x36, 0x3d, 0x44, 0x4a, 0x51, 0x58, 0x5f, 0x66, 0x6c, 0x73, 0x7a, 0x81, 0x88, 0x8e, 0x95, 0x9c, 0xa3, 0xaa, 0xb0, 0xb7, 0xbe, 0xc5, 0xcc, 0xd2 },
		/* 67 */ { 0x00, 0x06, 0x0d, 0x14, 0x1b, 0x22, 0x29, 0x30, 0x36, 0x3d, 0x44, 0x4b, 0x52, 0x59, 0x60, 0x67, 0x6d, 0x74, 0x7b, 0x82, 0x89, 0x90, 0x97, 0x9d, 0xa4, 0xab, 0xb2, 0xb9, 0xc0, 0xc7, 0xce, 0xd4 },
		/* 68 */ { 0x00, 0x06, 0x0d, 0x14, 0x1b, 0x22, 0x29, 0x30, 0x37, 0x3e, 0x45, 0x4c, 0x53, 0x5a, 0x61, 0x68, 0x6e, 0x75, 0x7c, 0x83, 0x8a, 0x91, 0x98, 0x9f, 0xa6, 0xad, 0xb4, 0xbb, 0xc2, 0xc9, 0xd0, 0xd6 },
		/* 69 */ { 0x00, 0x07, 0x0e, 0x15, 0x1c, 0x23, 0x2a, 0x31, 0x38, 0x3f, 0x46, 0x4d, 0x54, 0x5b, 0x62, 0x69, 0x70, 0x77, 0x7e, 0x85, 0x8c, 0x93, 0x9a, 0xa1, 0xa8, 0xaf, 0xb6, 0xbd, 0xc4, 0xcb, 0xd2, 0xd9 },
		/* 6a */ { 0x00, 0x07, 0x0e, 0x15, 0x1c, 0x23, 0x2a, 0x31, 0x38, 0x3f, 0x46, 0x4d, 0x54, 0x5b, 0x62, 0x6a, 0x71, 0x78, 0x7f, 0x86, 0x8d, 0x94, 0x9b, 0xa2, 0xa9, 0xb0, 0xb7, 0xbe, 0xc5, 0xcc, 0xd4, 0xdb },
		/* 6b */ { 0x00, 0x07, 0x0e, 0x15, 0x1c, 0x23, 0x2a, 0x31, 0x39, 0x40, 0x47, 0x4e, 0x55, 0x5c, 0x63, 0x6b, 0x72, 0x79, 0x80, 0x87, 0x8e, 0x95, 0x9c, 0xa4, 0xab, 0xb2, 0xb9, 0xc0, 0xc7, 0xce, 0xd6, 0xdd },
		/* 6c */ { 0x00, 0x07, 0x0e, 0x15, 0x1c, 0x24, 0x2b, 0x32, 0x39, 0x40, 0x48, 0x4f, 0x56, 0x5d, 0x64, 0x6c, 0x73, 0x7a, 0x81, 0x88, 0x90, 0x97, 0x9e, 0xa5, 0xac, 0xb4, 0xbb, 0xc2, 0xc9, 0xd0, 0xd8, 0xdf },
		/* 6d */ { 0x00, 0x07, 0x0e, 0x15, 0x1d, 0x24, 0x2b, 0x32, 0x3a, 0x41, 0x48, 0x4f, 0x57, 0x5e, 0x65, 0x6d, 0x74, 0x7b, 0x82, 0x8a, 0x91, 0x98, 0x9f, 0xa7, 0xae, 0xb5, 0xbc, 0xc4, 0xcb, 0xd2, 0xda, 0xe1 },
		/* 6e */ { 0x00, 0x07, 0x0e, 0x16, 0x1d, 0x24, 0x2c, 0x33, 0x3a, 0x42, 0x49, 0x50, 0x58, 0x5f, 0x66, 0x6e, 0x75, 0x7c, 0x84, 0x8b, 0x92, 0x9a, 0xa1, 0xa8, 0xb0, 0xb7, 0xbe, 0xc6, 0xcd, 0xd4, 0xdc, 0xe3 },
		/* 6f */ { 0x00, 0x07, 0x0e, 0x16, 0x1d, 0x25, 0x2c, 0x33, 0x3b, 0x42, 0x4a, 0x51, 0x58, 0x60, 0x67, 0x6f, 0x76, 0x7d, 0x85, 0x8c, 0x94, 0x9b, 0xa2, 0xaa, 0xb1, 0xb9, 0xc0, 0xc7, 0xcf, 0xd6, 0xde, 0xe5 },
		/* 70 */ { 0x00, 0x07, 0x0e, 0x16, 0x1d, 0x25, 0x2c, 0x34, 0x3b, 0x43, 0x4a, 0x52, 0x59, 0x61, 0x68, 0x70, 0x77, 0x7e, 0x86, 0x8d, 0x95, 0x9c, 0xa4, 0xab, 0xb3, 0xba, 0xc2, 0xc9, 0xd1, 0xd8, 0xe0, 0xe7 },
		/* 71 */ { 0x00, 0x07, 0x0f, 0x16, 0x1e, 0x25, 0x2d, 0x34, 0x3c, 0x43, 0x4b, 0x52, 0x5a, 0x61, 0x69, 0x71, 0x78, 0x80, 0x87, 0x8f, 0x96, 0x9e, 0xa5, 0xad, 0xb4, 0xbc, 0xc3, 0xcb, 0xd2, 0xda, 0xe2, 0xe9 },
		/* 72 */ { 0x00, 0x07, 0x0f, 0x16, 0x1e, 0x26, 0x2d, 0x35, 0x3c, 0x44, 0x4c, 0x53, 0x5b, 0x62, 0x6a, 0x72, 0x79, 0x81, 0x88, 0x90, 0x98, 0x9f, 0xa7, 0xae, 0xb6, 0xbe, 0xc5, 0xcd, 0xd4, 0xdc, 0xe4, 0xeb },
		/* 73 */ { 0x00, 0x07, 0x0f, 0x17, 0x1e, 0x26, 0x2e, 0x35, 0x3d, 0x45, 0x4c, 0x54, 0x5c, 0x63, 0x6b, 0x73, 0x7a, 0x82, 0x8a, 0x91, 0x99, 0xa1, 0xa8, 0xb0, 0xb8, 0xbf, 0xc7, 0xcf, 0xd6, 0xde, 0xe6, 0xed },
		/* 74 */ { 0x00, 0x07, 0x0f, 0x17, 0x1e, 0x26, 0x2e, 0x36, 0x3d, 0x45, 0x4d, 0x55, 0x5c, 0x64, 0x6c, 0x74, 0x7b, 0x83, 0x8b, 0x92, 0x9a, 0xa2, 0xaa, 0xb1, 0xb9, 0xc1, 0xc9, 0xd0, 0xd8, 0xe0, 0xe8, 0xef },
		/* 75 */ { 0x00, 0x07, 0x0f, 0x17, 0x1f, 0x27, 0x2e, 0x36, 0x3e, 0x46, 0x4e, 0x55, 0x5d, 0x65, 0x6d, 0x75, 0x7c, 0x84, 0x8c, 0x94, 0x9c, 0xa3, 0xab, 0xb3, 0xbb, 0xc3, 0xca, 0xd2, 0xda, 0xe2, 0xea, 0xf1 },
		/* 76 */ { 0x00, 0x07, 0x0f, 0x17, 0x1f, 0x27, 0x2f, 0x37, 0x3e, 0x46, 0x4e, 0x56, 0x5e, 0x66, 0x6e, 0x76, 0x7d, 0x85, 0x8d, 0x95, 0x9d, 0xa5, 0xad, 0xb4, 0xbc, 0xc4, 0xcc, 0xd4, 0xdc, 0xe4, 0xec, 0xf3 },
		/* 77 */ { 0x00, 0x07, 0x0f, 0x17, 0x1f, 0x27, 0x2f, 0x37, 0x3f, 0x47, 0x4f, 0x57, 0x5f, 0x67, 0x6f, 0x77, 0x7e, 0x86, 0x8e, 0x96, 0x9e, 0xa6, 0xae, 0xb6, 0xbe, 0xc6, 0xce, 0xd6, 0xde, 0xe6, 0xee, 0xf5 },
		/* 78 */ { 0x00, 0x08, 0x10, 0x18, 0x20, 0x28, 0x30, 0x38, 0x40, 0x48, 0x50, 0x58, 0x60, 0x68, 0x70, 0x78, 0x80, 0x88, 0x90, 0x98, 0xa0, 0xa8, 0xb0, 0xb8, 0xc0, 0xc8, 0xd0, 0xd8, 0xe0, 0xe8, 0xf0, 0xf8 },
		/* 79 */ { 0x00, 0x08, 0x10, 0x18, 0x20, 0x28, 0x30, 0x38, 0x40, 0x48, 0x50, 0x58, 0x60, 0x68, 0x70, 0x79, 0x81, 0x89, 0x91, 0x99, 0xa1, 0xa9, 0xb1, 0xb9, 0xc1, 0xc9, 0xd1, 0xd9, 0xe1, 0xe9, 0xf2, 0xfa },
		/* 7a */ { 0x00, 0x08, 0x10, 0x18, 0x20, 0x28, 0x30, 0x38, 0x41, 0x49, 0x51, 0x59, 0x61, 0x69, 0x71, 0x7a, 0x82, 0x8a, 0x92, 0x9a, 0xa2, 0xaa, 0xb2, 0xbb, 0xc3, 0xcb, 0xd3, 0xdb, 0xe3, 0xeb, 0xf4, 0xfc },
		/* 7b */ { 0x00, 0x08, 0x10, 0x18, 0x20, 0x29, 0x31, 0x39, 0x41, 0x49, 0x52, 0x5a, 0x62, 0x6a, 0x72, 0x7b, 0x83, 0x8b, 0x93, 0x9b, 0xa4, 0xac, 0xb4, 0xbc, 0xc4, 0xcd, 0xd5, 0xdd, 0xe5, 0xed, 0xf6, 0xfe },
		/* 7c */ { 0x00, 0x08, 0x10, 0x18, 0x21, 0x29, 0x31, 0x39, 0x42, 0x4a, 0x52, 0x5a, 0x63, 0x6b, 0x73, 0x7c, 0x84, 0x8c, 0x94, 0x9d, 0xa5, 0xad, 0xb5, 0xbe, 0xc6, 0xce, 0xd6, 0xdf, 0xe7, 0xef, 0xf8, 0xff },
		/* 7d */ { 0x00, 0x08, 0x10, 0x19, 0x21, 0x29, 0x32, 0x3a, 0x42, 0x4b, 0x53, 0x5b, 0x64, 0x6c, 0x74, 0x7d, 0x85, 0x8d, 0x96, 0x9e, 0xa6, 0xaf, 0xb7, 0xbf, 0xc8, 0xd0, 0xd8, 0xe1, 0xe9, 0xf1, 0xfa, 0xff },
		/* 7e */ { 0x00, 0x08, 0x10, 0x19, 0x21, 0x2a, 0x32, 0x3a, 0x43, 0x4b, 0x54, 0x5c, 0x64, 0x6d, 0x75, 0x7e, 0x86, 0x8e, 0x97, 0x9f, 0xa8, 0xb0, 0xb8, 0xc1, 0xc9, 0xd2, 0xda, 0xe2, 0xeb, 0xf3, 0xfc, 0xff },
		/* 7f */ { 0x00, 0x08, 0x10, 0x19, 0x21, 0x2a, 0x32, 0x3b, 0x43, 0x4c, 0x54, 0x5d, 0x65, 0x6e, 0x76, 0x7f, 0x87, 0x8f, 0x98, 0xa0, 0xa9, 0xb1, 0xba, 0xc2, 0xcb, 0xd3, 0xdc, 0xe4, 0xed, 0xf5, 0xfe, 0xff },
		/* 80 */ { 0x00, 0x08, 0x11, 0x19, 0x22, 0x2a, 0x33, 0x3b, 0x44, 0x4c, 0x55, 0x5d, 0x66, 0x6e, 0x77, 0x80, 0x88, 0x91, 0x99, 0xa2, 0xaa, 0xb3, 0xbb, 0xc4, 0xcc, 0xd5, 0xdd, 0xe6, 0xee, 0xf7, 0xff, 0xff },
		/* 81 */ { 0x00, 0x08, 0x11, 0x19, 0x22, 0x2b, 0x33, 0x3c, 0x44, 0x4d, 0x56, 0x5e, 0x67, 0x6f, 0x78, 0x81, 0x89, 0x92, 0x9a, 0xa3, 0xac, 0xb4, 0xbd, 0xc5, 0xce, 0xd7, 0xdf, 0xe8, 0xf0, 0xf9, 0xff, 0xff },
		/* 82 */ { 0x00, 0x08, 0x11, 0x1a, 0x22, 0x2b, 0x34, 0x3c, 0x45, 0x4e, 0x56, 0x5f, 0x68, 0x70, 0x79, 0x82, 0x8a, 0x93, 0x9c, 0xa4, 0xad, 0xb6, 0xbe, 0xc7, 0xd0, 0xd8, 0xe1, 0xea, 0xf2, 0xfb, 0xff, 0xff },
		/* 83 */ { 0x00, 0x08, 0x11, 0x1a, 0x22, 0x2b, 0x34, 0x3d, 0x45, 0x4e, 0x57, 0x60, 0x68, 0x71, 0x7a, 0x83, 0x8b, 0x94, 0x9d, 0xa5, 0xae, 0xb7, 0xc0, 0xc8, 0xd1, 0xda, 0xe3, 0xeb, 0xf4, 0xfd, 0xff, 0xff },
		/* 84 */ { 0x00, 0x08, 0x11, 0x1a, 0x23, 0x2c, 0x34, 0x3d, 0x46, 0x4f, 0x58, 0x60, 0x69, 0x72, 0x7b, 0x84, 0x8c, 0x95, 0x9e, 0xa7, 0xb0, 0xb8, 0xc1, 0xca, 0xd3, 0xdc, 0xe4, 0xed, 0xf6, 0xff, 0xff, 0xff },
		/* 85 */ { 0x00, 0x08, 0x11, 0x1a, 0x23, 0x2c, 0x35, 0x3e, 0x46, 0x4f, 0x58, 0x61, 0x6a, 0x73, 0x7c, 0x85, 0x8d, 0x96, 0x9f, 0xa8, 0xb1, 0xba, 0xc3, 0xcb, 0xd4, 0xdd, 0xe6, 0xef, 0xf8, 0xff, 0xff, 0xff },
		/* 86 */ { 0x00, 0x08, 0x11, 0x1a, 0x23, 0x2c, 0x35, 0x3e, 0x47, 0x50, 0x59, 0x62, 0x6b, 0x74, 0x7d, 0x86, 0x8e, 0x97, 0xa0, 0xa9, 0xb2, 0xbb, 0xc4, 0xcd, 0xd6, 0xdf, 0xe8, 0xf1, 0xfa, 0xff, 0xff, 0xff },
		/* 87 */ { 0x00, 0x09, 0x12, 0x1b, 0x24, 0x2d, 0x36, 0x3f, 0x48, 0x51, 0x5a, 0x63, 0x6c, 0x75, 0x7e, 0x87, 0x90, 0x99, 0xa2, 0xab, 0xb4, 0xbd, 0xc6, 0xcf, 0xd8, 0xe1, 0xea, 0xf3, 0xfc, 0xff, 0xff, 0xff },
		/* 88 */ { 0x00, 0x09, 0x12, 0x1b, 0x24, 0x2d, 0x36, 0x3f, 0x48, 0x51, 0x5a, 0x63, 0x6c, 0x75, 0x7e, 0x88, 0x91, 0x9a, 0xa3, 0xac, 0xb5, 0xbe, 0xc7, 0xd0, 0xd9, 0xe2, 0xeb, 0xf4, 0xfd, 0xff, 0xff, 0xff },
		/* 89 */ { 0x00, 0x09, 0x12, 0x1b, 0x24, 0x2d, 0x36, 0x3f, 0x49, 0x52, 0x5b, 0x64, 0x6d, 0x76, 0x7f, 0x89, 0x92, 0x9b, 0xa4, 0xad, 0xb6, 0xbf, 0xc8, 0xd2, 0xdb, 0xe4, 0xed, 0xf6, 0xff, 0xff, 0xff, 0xff },
		/* 8a */ { 0x00, 0x09, 0x12, 0x1b, 0x24, 0x2e, 0x37, 0x40, 0x49, 0x52, 0x5c, 0x65, 0x6e, 0x77, 0x80, 0x8a, 0x93, 0x9c, 0xa5, 0xae, 0xb8, 0xc1, 0xca, 0xd3, 0xdc, 0xe6, 0xef, 0xf8, 0xff, 0xff, 0xff, 0xff },
		/* 8b */ { 0x00, 0x09, 0x12, 0x1b, 0x25, 0x2e, 0x37, 0x40, 0x4a, 0x53, 0x5c, 0x65, 0x6f, 0x78, 0x81, 0x8b, 0x94, 0x9d, 0xa6, 0xb0, 0xb9, 0xc2, 0xcb, 0xd5, 0xde, 0xe7, 0xf0, 0xfa, 0xff, 0xff, 0xff, 0xff },
		/* 8c */ { 0x00, 0x09, 0x12, 0x1c, 0x25, 0x2e, 0x38, 0x41, 0x4a, 0x54, 0x5d, 0x66, 0x70, 0x79, 0x82, 0x8c, 0x95, 0x9e, 0xa8, 0xb1, 0xba, 0xc4, 0xcd, 0xd6, 0xe0, 0xe9, 0xf2, 0xfc, 0xff, 0xff, 0xff, 0xff },
		/* 8d */ { 0x00, 0x09, 0x12, 0x1c, 0x25, 0x2f, 0x38, 0x41, 0x4b, 0x54, 0x5e, 0x67, 0x70, 0x7a, 0x83, 0x8d, 0x96, 0x9f, 0xa9, 0xb2, 0xbc, 0xc5, 0xce, 0xd8, 0xe1, 0xeb, 0xf4, 0xfd, 0xff, 0xff, 0xff, 0xff },
		/* 8e */ { 0x00, 0x09, 0x12, 0x1c, 0x25, 0x2f, 0x38, 0x42, 0x4b, 0x55, 0x5e, 0x68, 0x71, 0x7b, 0x84, 0x8e, 0x97, 0xa0, 0xaa, 0xb3, 0xbd, 0xc6, 0xd0, 0xd9, 0xe3, 0xec, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 8f */ { 0x00, 0x09, 0x13, 0x1c, 0x26, 0x2f, 0x39, 0x42, 0x4c, 0x55, 0x5f, 0x68, 0x72, 0x7b, 0x85, 0x8f, 0x98, 0xa2, 0xab, 0xb5, 0xbe, 0xc8, 0xd1, 0xdb, 0xe4, 0xee, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 90 */ { 0x00, 0x09, 0x13, 0x1c, 0x26, 0x30, 0x39, 0x43, 0x4c, 0x56, 0x60, 0x69, 0x73, 0x7c, 0x86, 0x90, 0x99, 0xa3, 0xac, 0xb6, 0xc0, 0xc9, 0xd3, 0xdc, 0xe6, 0xf0, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 91 */ { 0x00, 0x09, 0x13, 0x1d, 0x26, 0x30, 0x3a, 0x43, 0x4d, 0x57, 0x60, 0x6a, 0x74, 0x7d, 0x87, 0x91, 0x9a, 0xa4, 0xae, 0xb7, 0xc1, 0xcb, 0xd4, 0xde, 0xe8, 0xf1, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 92 */ { 0x00, 0x09, 0x13, 0x1d, 0x26, 0x30, 0x3a, 0x44, 0x4d, 0x57, 0x61, 0x6b, 0x74, 0x7e, 0x88, 0x92, 0x9b, 0xa5, 0xaf, 0xb8, 0xc2, 0xcc, 0xd6, 0xdf, 0xe9, 0xf3, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 93 */ { 0x00, 0x09, 0x13, 0x1d, 0x27, 0x31, 0x3a, 0x44, 0x4e, 0x58, 0x62, 0x6b, 0x75, 0x7f, 0x89, 0x93, 0x9c, 0xa6, 0xb0, 0xba, 0xc4, 0xcd, 0xd7, 0xe1, 0xeb, 0xf5, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 94 */ { 0x00, 0x09, 0x13, 0x1d, 0x27, 0x31, 0x3b, 0x45, 0x4e, 0x58, 0x62, 0x6c, 0x76, 0x80, 0x8a, 0x94, 0x9d, 0xa7, 0xb1, 0xbb, 0xc5, 0xcf, 0xd9, 0xe2, 0xec, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 95 */ { 0x00, 0x09, 0x13, 0x1d, 0x27, 0x31, 0x3b, 0x45, 0x4f, 0x59, 0x63, 0x6d, 0x77, 0x81, 0x8b, 0x95, 0x9e, 0xa8, 0xb2, 0xbc, 0xc6, 0xd0, 0xda, 0xe4, 0xee, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 96 */ { 0x00, 0x0a, 0x14, 0x1e, 0x28, 0x32, 0x3c, 0x46, 0x50, 0x5a, 0x64, 0x6e, 0x78, 0x82, 0x8c, 0x96, 0xa0, 0xaa, 0xb4, 0xbe, 0xc8, 0xd2, 0xdc, 0xe6, 0xf0, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 97 */ { 0x00, 0x0a, 0x14, 0x1e, 0x28, 0x32, 0x3c, 0x46, 0x50, 0x5a, 0x64, 0x6e, 0x78, 0x82, 0x8c, 0x97, 0xa1, 0xab, 0xb5, 0xbf, 0xc9, 0xd3, 0xdd, 0xe7, 0xf1, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 98 */ { 0x00, 0x0a, 0x14, 0x1e, 0x28, 0x32, 0x3c, 0x46, 0x51, 0x5b, 0x65, 0x6f, 0x79, 0x83, 0x8d, 0x98, 0xa2, 0xac, 0xb6, 0xc0, 0xca, 0xd4, 0xde, 0xe9, 0xf3, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 99 */ { 0x00, 0x0a, 0x14, 0x1e, 0x28, 0x33, 0x3d, 0x47, 0x51, 0x5b, 0x66, 0x70, 0x7a, 0x84, 0x8e, 0x99, 0xa3, 0xad, 0xb7, 0xc1, 0xcc, 0xd6, 0xe0, 0xea, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 9a */ { 0x00, 0x0a, 0x14, 0x1e, 0x29, 0x33, 0x3d, 0x47, 0x52, 0x5c, 0x66, 0x70, 0x7b, 0x85, 0x8f, 0x9a, 0xa4, 0xae, 0xb8, 0xc3, 0xcd, 0xd7, 0xe1, 0xec, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 9b */ { 0x00, 0x0a, 0x14, 0x1f, 0x29, 0x33, 0x3e, 0x48, 0x52, 0x5d, 0x67, 0x71, 0x7c, 0x86, 0x90, 0x9b, 0xa5, 0xaf, 0xba, 0xc4, 0xce, 0xd9, 0xe3, 0xed, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 9c */ { 0x00, 0x0a, 0x14, 0x1f, 0x29, 0x34, 0x3e, 0x48, 0x53, 0x5d, 0x68, 0x72, 0x7c, 0x87, 0x91, 0x9c, 0xa6, 0xb0, 0xbb, 0xc5, 0xd0, 0xda, 0xe4, 0xef, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 9d */ { 0x00, 0x0a, 0x14, 0x1f, 0x29, 0x34, 0x3e, 0x49, 0x53, 0x5e, 0x68, 0x73, 0x7d, 0x88, 0x92, 0x9d, 0xa7, 0xb1, 0xbc, 0xc6, 0xd1, 0xdb, 0xe6, 0xf0, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 9e */ { 0x00, 0x0a, 0x15, 0x1f, 0x2a, 0x34, 0x3f, 0x49, 0x54, 0x5e, 0x69, 0x73, 0x7e, 0x88, 0x93, 0x9e, 0xa8, 0xb3, 0xbd, 0xc8, 0xd2, 0xdd, 0xe7, 0xf2, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* 9f */ { 0x00, 0x0a, 0x15, 0x1f, 0x2a, 0x35, 0x3f, 0x4a, 0x54, 0x5f, 0x6a, 0x74, 0x7f, 0x89, 0x94, 0x9f, 0xa9, 0xb4, 0xbe, 0xc9, 0xd4, 0xde, 0xe9, 0xf3, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a0 */ { 0x00, 0x0a, 0x15, 0x20, 0x2a, 0x35, 0x40, 0x4a, 0x55, 0x60, 0x6a, 0x75, 0x80, 0x8a, 0x95, 0xa0, 0xaa, 0xb5, 0xc0, 0xca, 0xd5, 0xe0, 0xea, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a1 */ { 0x00, 0x0a, 0x15, 0x20, 0x2a, 0x35, 0x40, 0x4b, 0x55, 0x60, 0x6b, 0x76, 0x80, 0x8b, 0x96, 0xa1, 0xab, 0xb6, 0xc1, 0xcb, 0xd6, 0xe1, 0xec, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a2 */ { 0x00, 0x0a, 0x15, 0x20, 0x2b, 0x36, 0x40, 0x4b, 0x56, 0x61, 0x6c, 0x76, 0x81, 0x8c, 0x97, 0xa2, 0xac, 0xb7, 0xc2, 0xcd, 0xd8, 0xe2, 0xed, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a3 */ { 0x00, 0x0a, 0x15, 0x20, 0x2b, 0x36, 0x41, 0x4c, 0x56, 0x61, 0x6c, 0x77, 0x82, 0x8d, 0x98, 0xa3, 0xad, 0xb8, 0xc3, 0xce, 0xd9, 0xe4, 0xef, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a4 */ { 0x00, 0x0a, 0x15, 0x20, 0x2b, 0x36, 0x41, 0x4c, 0x57, 0x62, 0x6d, 0x78, 0x83, 0x8e, 0x99, 0xa4, 0xae, 0xb9, 0xc4, 0xcf, 0xda, 0xe5, 0xf0, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a5 */ { 0x00, 0x0b, 0x16, 0x21, 0x2c, 0x37, 0x42, 0x4d, 0x58, 0x63, 0x6e, 0x79, 0x84, 0x8f, 0x9a, 0xa5, 0xb0, 0xbb, 0xc6, 0xd1, 0xdc, 0xe7, 0xf2, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a6 */ { 0x00, 0x0b, 0x16, 0x21, 0x2c, 0x37, 0x42, 0x4d, 0x58, 0x63, 0x6e, 0x79, 0x84, 0x8f, 0x9a, 0xa6, 0xb1, 0xbc, 0xc7, 0xd2, 0xdd, 0xe8, 0xf3, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a7 */ { 0x00, 0x0b, 0x16, 0x21, 0x2c, 0x37, 0x42, 0x4d, 0x59, 0x64, 0x6f, 0x7a, 0x85, 0x90, 0x9b, 0xa7, 0xb2, 0xbd, 0xc8, 0xd3, 0xde, 0xe9, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a8 */ { 0x00, 0x0b, 0x16, 0x21, 0x2c, 0x38, 0x43, 0x4e, 0x59, 0x64, 0x70, 0x7b, 0x86, 0x91, 0x9c, 0xa8, 0xb3, 0xbe, 0xc9, 0xd4, 0xe0, 0xeb, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* a9 */ { 0x00, 0x0b, 0x16, 0x21, 0x2d, 0x38, 0x43, 0x4e, 0x5a, 0x65, 0x70, 0x7b, 0x87, 0x92, 0x9d, 0xa9, 0xb4, 0xbf, 0xca, 0xd6, 0xe1, 0xec, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* aa */ { 0x00, 0x0b, 0x16, 0x22, 0x2d, 0x38, 0x44, 0x4f, 0x5a, 0x66, 0x71, 0x7c, 0x88, 0x93, 0x9e, 0xaa, 0xb5, 0xc0, 0xcc, 0xd7, 0xe2, 0xee, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ab */ { 0x00, 0x0b, 0x16, 0x22, 0x2d, 0x39, 0x44, 0x4f, 0x5b, 0x66, 0x72, 0x7d, 0x88, 0x94, 0x9f, 0xab, 0xb6, 0xc1, 0xcd, 0xd8, 0xe4, 0xef, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ac */ { 0x00, 0x0b, 0x16, 0x22, 0x2d, 0x39, 0x44, 0x50, 0x5b, 0x67, 0x72, 0x7e, 0x89, 0x95, 0xa0, 0xac, 0xb7, 0xc2, 0xce, 0xd9, 0xe5, 0xf0, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ad */ { 0x00, 0x0b, 0x17, 0x22, 0x2e, 0x39, 0x45, 0x50, 0x5c, 0x67, 0x73, 0x7e, 0x8a, 0x95, 0xa1, 0xad, 0xb8, 0xc4, 0xcf, 0xdb, 0xe6, 0xf2, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ae */ { 0x00, 0x0b, 0x17, 0x22, 0x2e, 0x3a, 0x45, 0x51, 0x5c, 0x68, 0x74, 0x7f, 0x8b, 0x96, 0xa2, 0xae, 0xb9, 0xc5, 0xd0, 0xdc, 0xe8, 0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* af */ { 0x00, 0x0b, 0x17, 0x23, 0x2e, 0x3a, 0x46, 0x51, 0x5d, 0x69, 0x74, 0x80, 0x8c, 0x97, 0xa3, 0xaf, 0xba, 0xc6, 0xd2, 0xdd, 0xe9, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b0 */ { 0x00, 0x0b, 0x17, 0x23, 0x2e, 0x3a, 0x46, 0x52, 0x5d, 0x69, 0x75, 0x81, 0x8c, 0x98, 0xa4, 0xb0, 0xbb, 0xc7, 0xd3, 0xde, 0xea, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b1 */ { 0x00, 0x0b, 0x17, 0x23, 0x2f, 0x3b, 0x46, 0x52, 0x5e, 0x6a, 0x76, 0x81, 0x8d, 0x99, 0xa5, 0xb1, 0xbc, 0xc8, 0xd4, 0xe0, 0xec, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b2 */ { 0x00, 0x0b, 0x17, 0x23, 0x2f, 0x3b, 0x47, 0x53, 0x5e, 0x6a, 0x76, 0x82, 0x8e, 0x9a, 0xa6, 0xb2, 0xbd, 0xc9, 0xd5, 0xe1, 0xed, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b3 */ { 0x00, 0x0b, 0x17, 0x23, 0x2f, 0x3b, 0x47, 0x53, 0x5f, 0x6b, 0x77, 0x83, 0x8f, 0x9b, 0xa7, 0xb3, 0xbe, 0xca, 0xd6, 0xe2, 0xee, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b4 */ { 0x00, 0x0c, 0x18, 0x24, 0x30, 0x3c, 0x48, 0x54, 0x60, 0x6c, 0x78, 0x84, 0x90, 0x9c, 0xa8, 0xb4, 0xc0, 0xcc, 0xd8, 0xe4, 0xf0, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b5 */ { 0x00, 0x0c, 0x18, 0x24, 0x30, 0x3c, 0x48, 0x54, 0x60, 0x6c, 0x78, 0x84, 0x90, 0x9c, 0xa8, 0xb5, 0xc1, 0xcd, 0xd9, 0xe5, 0xf1, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b6 */ { 0x00, 0x0c, 0x18, 0x24, 0x30, 0x3c, 0x48, 0x54, 0x61, 0x6d, 0x79, 0x85, 0x91, 0x9d, 0xa9, 0xb6, 0xc2, 0xce, 0xda, 0xe6, 0xf2, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b7 */ { 0x00, 0x0c, 0x18, 0x24, 0x30, 0x3d, 0x49, 0x55, 0x61, 0x6d, 0x7a, 0x86, 0x92, 0x9e, 0xaa, 0xb7, 0xc3, 0xcf, 0xdb, 0xe7, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b8 */ { 0x00, 0x0c, 0x18, 0x24, 0x31, 0x3d, 0x49, 0x55, 0x62, 0x6e, 0x7a, 0x86, 0x93, 0x9f, 0xab, 0xb8, 0xc4, 0xd0, 0xdc, 0xe9, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* b9 */ { 0x00, 0x0c, 0x18, 0x25, 0x31, 0x3d, 0x4a, 0x56, 0x62, 0x6f, 0x7b, 0x87, 0x94, 0xa0, 0xac, 0xb9, 0xc5, 0xd1, 0xde, 0xea, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ba */ { 0x00, 0x0c, 0x18, 0x25, 0x31, 0x3e, 0x4a, 0x56, 0x63, 0x6f, 0x7c, 0x88, 0x94, 0xa1, 0xad, 0xba, 0xc6, 0xd2, 0xdf, 0xeb, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* bb */ { 0x00, 0x0c, 0x18, 0x25, 0x31, 0x3e, 0x4a, 0x57, 0x63, 0x70, 0x7c, 0x89, 0x95, 0xa2, 0xae, 0xbb, 0xc7, 0xd3, 0xe0, 0xec, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* bc */ { 0x00, 0x0c, 0x19, 0x25, 0x32, 0x3e, 0x4b, 0x57, 0x64, 0x70, 0x7d, 0x89, 0x96, 0xa2, 0xaf, 0xbc, 0xc8, 0xd5, 0xe1, 0xee, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* bd */ { 0x00, 0x0c, 0x19, 0x25, 0x32, 0x3f, 0x4b, 0x58, 0x64, 0x71, 0x7e, 0x8a, 0x97, 0xa3, 0xb0, 0xbd, 0xc9, 0xd6, 0xe2, 0xef, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* be */ { 0x00, 0x0c, 0x19, 0x26, 0x32, 0x3f, 0x4c, 0x58, 0x65, 0x72, 0x7e, 0x8b, 0x98, 0xa4, 0xb1, 0xbe, 0xca, 0xd7, 0xe4, 0xf0, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* bf */ { 0x00, 0x0c, 0x19, 0x26, 0x32, 0x3f, 0x4c, 0x59, 0x65, 0x72, 0x7f, 0x8c, 0x98, 0xa5, 0xb2, 0xbf, 0xcb, 0xd8, 0xe5, 0xf1, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c0 */ { 0x00, 0x0c, 0x19, 0x26, 0x33, 0x40, 0x4c, 0x59, 0x66, 0x73, 0x80, 0x8c, 0x99, 0xa6, 0xb3, 0xc0, 0xcc, 0xd9, 0xe6, 0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c1 */ { 0x00, 0x0c, 0x19, 0x26, 0x33, 0x40, 0x4d, 0x5a, 0x66, 0x73, 0x80, 0x8d, 0x9a, 0xa7, 0xb4, 0xc1, 0xcd, 0xda, 0xe7, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c2 */ { 0x00, 0x0c, 0x19, 0x26, 0x33, 0x40, 0x4d, 0x5a, 0x67, 0x74, 0x81, 0x8e, 0x9b, 0xa8, 0xb5, 0xc2, 0xce, 0xdb, 0xe8, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c3 */ { 0x00, 0x0d, 0x1a, 0x27, 0x34, 0x41, 0x4e, 0x5b, 0x68, 0x75, 0x82, 0x8f, 0x9c, 0xa9, 0xb6, 0xc3, 0xd0, 0xdd, 0xea, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c4 */ { 0x00, 0x0d, 0x1a, 0x27, 0x34, 0x41, 0x4e, 0x5b, 0x68, 0x75, 0x82, 0x8f, 0x9c, 0xa9, 0xb6, 0xc4, 0xd1, 0xde, 0xeb, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c5 */ { 0x00, 0x0d, 0x1a, 0x27, 0x34, 0x41, 0x4e, 0x5b, 0x69, 0x76, 0x83, 0x90, 0x9d, 0xaa, 0xb7, 0xc5, 0xd2, 0xdf, 0xec, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c6 */ { 0x00, 0x0d, 0x1a, 0x27, 0x34, 0x42, 0x4f, 0x5c, 0x69, 0x76, 0x84, 0x91, 0x9e, 0xab, 0xb8, 0xc6, 0xd3, 0xe0, 0xed, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c7 */ { 0x00, 0x0d, 0x1a, 0x27, 0x35, 0x42, 0x4f, 0x5c, 0x6a, 0x77, 0x84, 0x91, 0x9f, 0xac, 0xb9, 0xc7, 0xd4, 0xe1, 0xee, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c8 */ { 0x00, 0x0d, 0x1a, 0x28, 0x35, 0x42, 0x50, 0x5d, 0x6a, 0x78, 0x85, 0x92, 0xa0, 0xad, 0xba, 0xc8, 0xd5, 0xe2, 0xf0, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* c9 */ { 0x00, 0x0d, 0x1a, 0x28, 0x35, 0x43, 0x50, 0x5d, 0x6b, 0x78, 0x86, 0x93, 0xa0, 0xae, 0xbb, 0xc9, 0xd6, 0xe3, 0xf1, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ca */ { 0x00, 0x0d, 0x1a, 0x28, 0x35, 0x43, 0x50, 0x5e, 0x6b, 0x79, 0x86, 0x94, 0xa1, 0xaf, 0xbc, 0xca, 0xd7, 0xe4, 0xf2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* cb */ { 0x00, 0x0d, 0x1b, 0x28, 0x36, 0x43, 0x51, 0x5e, 0x6c, 0x79, 0x87, 0x94, 0xa2, 0xaf, 0xbd, 0xcb, 0xd8, 0xe6, 0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* cc */ { 0x00, 0x0d, 0x1b, 0x28, 0x36, 0x44, 0x51, 0x5f, 0x6c, 0x7a, 0x88, 0x95, 0xa3, 0xb0, 0xbe, 0xcc, 0xd9, 0xe7, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* cd */ { 0x00, 0x0d, 0x1b, 0x29, 0x36, 0x44, 0x52, 0x5f, 0x6d, 0x7b, 0x88, 0x96, 0xa4, 0xb1, 0xbf, 0xcd, 0xda, 0xe8, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ce */ { 0x00, 0x0d, 0x1b, 0x29, 0x36, 0x44, 0x52, 0x60, 0x6d, 0x7b, 0x89, 0x97, 0xa4, 0xb2, 0xc0, 0xce, 0xdb, 0xe9, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* cf */ { 0x00, 0x0d, 0x1b, 0x29, 0x37, 0x45, 0x52, 0x60, 0x6e, 0x7c, 0x8a, 0x97, 0xa5, 0xb3, 0xc1, 0xcf, 0xdc, 0xea, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d0 */ { 0x00, 0x0d, 0x1b, 0x29, 0x37, 0x45, 0x53, 0x61, 0x6e, 0x7c, 0x8a, 0x98, 0xa6, 0xb4, 0xc2, 0xd0, 0xdd, 0xeb, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d1 */ { 0x00, 0x0d, 0x1b, 0x29, 0x37, 0x45, 0x53, 0x61, 0x6f, 0x7d, 0x8b, 0x99, 0xa7, 0xb5, 0xc3, 0xd1, 0xde, 0xec, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d2 */ { 0x00, 0x0e, 0x1c, 0x2a, 0x38, 0x46, 0x54, 0x62, 0x70, 0x7e, 0x8c, 0x9a, 0xa8, 0xb6, 0xc4, 0xd2, 0xe0, 0xee, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d3 */ { 0x00, 0x0e, 0x1c, 0x2a, 0x38, 0x46, 0x54, 0x62, 0x70, 0x7e, 0x8c, 0x9a, 0xa8, 0xb6, 0xc4, 0xd3, 0xe1, 0xef, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d4 */ { 0x00, 0x0e, 0x1c, 0x2a, 0x38, 0x46, 0x54, 0x62, 0x71, 0x7f, 0x8d, 0x9b, 0xa9, 0xb7, 0xc5, 0xd4, 0xe2, 0xf0, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d5 */ { 0x00, 0x0e, 0x1c, 0x2a, 0x38, 0x47, 0x55, 0x63, 0x71, 0x7f, 0x8e, 0x9c, 0xaa, 0xb8, 0xc6, 0xd5, 0xe3, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d6 */ { 0x00, 0x0e, 0x1c, 0x2a, 0x39, 0x47, 0x55, 0x63, 0x72, 0x80, 0x8e, 0x9c, 0xab, 0xb9, 0xc7, 0xd6, 0xe4, 0xf2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d7 */ { 0x00, 0x0e, 0x1c, 0x2b, 0x39, 0x47, 0x56, 0x64, 0x72, 0x81, 0x8f, 0x9d, 0xac, 0xba, 0xc8, 0xd7, 0xe5, 0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d8 */ { 0x00, 0x0e, 0x1c, 0x2b, 0x39, 0x48, 0x56, 0x64, 0x73, 0x81, 0x90, 0x9e, 0xac, 0xbb, 0xc9, 0xd8, 0xe6, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* d9 */ { 0x00, 0x0e, 0x1c, 0x2b, 0x39, 0x48, 0x56, 0x65, 0x73, 0x82, 0x90, 0x9f, 0xad, 0xbc, 0xca, 0xd9, 0xe7, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* da */ { 0x00, 0x0e, 0x1d, 0x2b, 0x3a, 0x48, 0x57, 0x65, 0x74, 0x82, 0x91, 0x9f, 0xae, 0xbc, 0xcb, 0xda, 0xe8, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* db */ { 0x00, 0x0e, 0x1d, 0x2b, 0x3a, 0x49, 0x57, 0x66, 0x74, 0x83, 0x92, 0xa0, 0xaf, 0xbd, 0xcc, 0xdb, 0xe9, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* dc */ { 0x00, 0x0e, 0x1d, 0x2c, 0x3a, 0x49, 0x58, 0x66, 0x75, 0x84, 0x92, 0xa1, 0xb0, 0xbe, 0xcd, 0xdc, 0xea, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* dd */ { 0x00, 0x0e, 0x1d, 0x2c, 0x3a, 0x49, 0x58, 0x67, 0x75, 0x84, 0x93, 0xa2, 0xb0, 0xbf, 0xce, 0xdd, 0xeb, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* de */ { 0x00, 0x0e, 0x1d, 0x2c, 0x3b, 0x4a, 0x58, 0x67, 0x76, 0x85, 0x94, 0xa2, 0xb1, 0xc0, 0xcf, 0xde, 0xec, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* df */ { 0x00, 0x0e, 0x1d, 0x2c, 0x3b, 0x4a, 0x59, 0x68, 0x76, 0x85, 0x94, 0xa3, 0xb2, 0xc1, 0xd0, 0xdf, 0xed, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e0 */ { 0x00, 0x0e, 0x1d, 0x2c, 0x3b, 0x4a, 0x59, 0x68, 0x77, 0x86, 0x95, 0xa4, 0xb3, 0xc2, 0xd1, 0xe0, 0xee, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e1 */ { 0x00, 0x0f, 0x1e, 0x2d, 0x3c, 0x4b, 0x5a, 0x69, 0x78, 0x87, 0x96, 0xa5, 0xb4, 0xc3, 0xd2, 0xe1, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e2 */ { 0x00, 0x0f, 0x1e, 0x2d, 0x3c, 0x4b, 0x5a, 0x69, 0x78, 0x87, 0x96, 0xa5, 0xb4, 0xc3, 0xd2, 0xe2, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e3 */ { 0x00, 0x0f, 0x1e, 0x2d, 0x3c, 0x4b, 0x5a, 0x69, 0x79, 0x88, 0x97, 0xa6, 0xb5, 0xc4, 0xd3, 0xe3, 0xf2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e4 */ { 0x00, 0x0f, 0x1e, 0x2d, 0x3c, 0x4c, 0x5b, 0x6a, 0x79, 0x88, 0x98, 0xa7, 0xb6, 0xc5, 0xd4, 0xe4, 0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e5 */ { 0x00, 0x0f, 0x1e, 0x2d, 0x3d, 0x4c, 0x5b, 0x6a, 0x7a, 0x89, 0x98, 0xa7, 0xb7, 0xc6, 0xd5, 0xe5, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e6 */ { 0x00, 0x0f, 0x1e, 0x2e, 0x3d, 0x4c, 0x5c, 0x6b, 0x7a, 0x8a, 0x99, 0xa8, 0xb8, 0xc7, 0xd6, 0xe6, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e7 */ { 0x00, 0x0f, 0x1e, 0x2e, 0x3d, 0x4d, 0x5c, 0x6b, 0x7b, 0x8a, 0x9a, 0xa9, 0xb8, 0xc8, 0xd7, 0xe7, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e8 */ { 0x00, 0x0f, 0x1e, 0x2e, 0x3d, 0x4d, 0x5c, 0x6c, 0x7b, 0x8b, 0x9a, 0xaa, 0xb9, 0xc9, 0xd8, 0xe8, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* e9 */ { 0x00, 0x0f, 0x1f, 0x2e, 0x3e, 0x4d, 0x5d, 0x6c, 0x7c, 0x8b, 0x9b, 0xaa, 0xba, 0xc9, 0xd9, 0xe9, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ea */ { 0x00, 0x0f, 0x1f, 0x2e, 0x3e, 0x4e, 0x5d, 0x6d, 0x7c, 0x8c, 0x9c, 0xab, 0xbb, 0xca, 0xda, 0xea, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* eb */ { 0x00, 0x0f, 0x1f, 0x2f, 0x3e, 0x4e, 0x5e, 0x6d, 0x7d, 0x8d, 0x9c, 0xac, 0xbc, 0xcb, 0xdb, 0xeb, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ec */ { 0x00, 0x0f, 0x1f, 0x2f, 0x3e, 0x4e, 0x5e, 0x6e, 0x7d, 0x8d, 0x9d, 0xad, 0xbc, 0xcc, 0xdc, 0xec, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ed */ { 0x00, 0x0f, 0x1f, 0x2f, 0x3f, 0x4f, 0x5e, 0x6e, 0x7e, 0x8e, 0x9e, 0xad, 0xbd, 0xcd, 0xdd, 0xed, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ee */ { 0x00, 0x0f, 0x1f, 0x2f, 0x3f, 0x4f, 0x5f, 0x6f, 0x7e, 0x8e, 0x9e, 0xae, 0xbe, 0xce, 0xde, 0xee, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ef */ { 0x00, 0x0f, 0x1f, 0x2f, 0x3f, 0x4f, 0x5f, 0x6f, 0x7f, 0x8f, 0x9f, 0xaf, 0xbf, 0xcf, 0xdf, 0xef, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f0 */ { 0x00, 0x10, 0x20, 0x30, 0x40, 0x50, 0x60, 0x70, 0x80, 0x90, 0xa0, 0xb0, 0xc0, 0xd0, 0xe0, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f1 */ { 0x00, 0x10, 0x20, 0x30, 0x40, 0x50, 0x60, 0x70, 0x80, 0x90, 0xa0, 0xb0, 0xc0, 0xd0, 0xe0, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f2 */ { 0x00, 0x10, 0x20, 0x30, 0x40, 0x50, 0x60, 0x70, 0x81, 0x91, 0xa1, 0xb1, 0xc1, 0xd1, 0xe1, 0xf2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f3 */ { 0x00, 0x10, 0x20, 0x30, 0x40, 0x51, 0x61, 0x71, 0x81, 0x91, 0xa2, 0xb2, 0xc2, 0xd2, 0xe2, 0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f4 */ { 0x00, 0x10, 0x20, 0x30, 0x41, 0x51, 0x61, 0x71, 0x82, 0x92, 0xa2, 0xb2, 0xc3, 0xd3, 0xe3, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f5 */ { 0x00, 0x10, 0x20, 0x31, 0x41, 0x51, 0x62, 0x72, 0x82, 0x93, 0xa3, 0xb3, 0xc4, 0xd4, 0xe4, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f6 */ { 0x00, 0x10, 0x20, 0x31, 0x41, 0x52, 0x62, 0x72, 0x83, 0x93, 0xa4, 0xb4, 0xc4, 0xd5, 0xe5, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f7 */ { 0x00, 0x10, 0x20, 0x31, 0x41, 0x52, 0x62, 0x73, 0x83, 0x94, 0xa4, 0xb5, 0xc5, 0xd6, 0xe6, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f8 */ { 0x00, 0x10, 0x21, 0x31, 0x42, 0x52, 0x63, 0x73, 0x84, 0x94, 0xa5, 0xb5, 0xc6, 0xd6, 0xe7, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* f9 */ { 0x00, 0x10, 0x21, 0x31, 0x42, 0x53, 0x63, 0x74, 0x84, 0x95, 0xa6, 0xb6, 0xc7, 0xd7, 0xe8, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* fa */ { 0x00, 0x10, 0x21, 0x32, 0x42, 0x53, 0x64, 0x74, 0x85, 0x96, 0xa6, 0xb7, 0xc8, 0xd8, 0xe9, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* fb */ { 0x00, 0x10, 0x21, 0x32, 0x42, 0x53, 0x64, 0x75, 0x85, 0x96, 0xa7, 0xb8, 0xc8, 0xd9, 0xea, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* fc */ { 0x00, 0x10, 0x21, 0x32, 0x43, 0x54, 0x64, 0x75, 0x86, 0x97, 0xa8, 0xb8, 0xc9, 0xda, 0xeb, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* fd */ { 0x00, 0x10, 0x21, 0x32, 0x43, 0x54, 0x65, 0x76, 0x86, 0x97, 0xa8, 0xb9, 0xca, 0xdb, 0xec, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* fe */ { 0x00, 0x10, 0x21, 0x32, 0x43, 0x54, 0x65, 0x76, 0x87, 0x98, 0xa9, 0xba, 0xcb, 0xdc, 0xed, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
		/* ff */ { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff }
	};

static GP_PALETTEENTRY pentry[256];

typedef struct PAL
	{
		u8 r,g,b;
	} PAL;

static void SetPal ( PAL * palette, int amp )
	{
		int i;

    if (amp>31)
     	amp = 31;
    else
    if (amp<0)
    	amp = 0;

    for (i=0;i<256;i++)
      {
        int r = amptab[palette[i].r][amp];
        int g = amptab[palette[i].g][amp];
        int b = amptab[palette[i].b][amp];
        pentry[i] = GP_RGB24(r,g,b);
      }

		GpPaletteEntryChange ( 0, 256, pentry, 1 );
	}

static int striicmp(const char * s1, const char * s2)
	{
		int l1;
		int l2;
		int i;

		if (s1==NULL || s2==NULL)
			return -1;

		l1 = strlen ( s1 );
		l2 = strlen ( s2 );
		if (l1!=l2)
			return -1;

		for (i=0;i<l1;i++)
			if ( tolower(s1[i]) != tolower(s2[i]) )
				return -1;

		return 0;
	}

static void GpClearSurface ( GPDRAWSURFACE * gpDraw, u8 color )
  {
    int i;

    if (gpDraw==NULL)
      return;
    
    for (i=0;i<320*240;i++)
      gpDraw->ptbuffer[i] = color;
  }

static void draw_focus ( GPDRAWSURFACE * gpDraw, int yp, u8 xor )
  {
    int dest = 240-yp;
    int x,y;
    
    for (y=0;y<16;y++)
      {
        for (x=0;x<320;x++)
          {
            gpDraw->ptbuffer[dest] ^= xor;
            dest+=240;
          }
        dest-=240*320;
        dest--;
      }
  }

static void display_entries ( ZIPFREQUEST * req, int yp, int height )
  {
    int y;
    int pathnamelen = strlen ( req->filename );
    int pathliststartofs;
    int pathlistofs;
    int focusy = 20+(5*16);
    int yb1 = 18;  /* height: 176 */
    int yb2 = yb1+height;

    yp-=5*16;

    if (yp<0)
      {
        focusy += yp;
        yp=0;
      } else
    if ((yp>(req->pathlistsize-11)*16) && req->pathlistsize>11)
      {
        focusy += yp-((req->pathlistsize-11)*16);
        yp=(req->pathlistsize-11)*16;
      } else
    if (req->pathlistsize<=11)
    	{
	    	focusy=yp+(5*16)+20;
	    	yp=0;
    	}

    pathliststartofs = yp>>4;

    if (req->zip)
    	{
    		amiga_textout ( &req->gpDraw[req->surf&(req->numGpDraw-1)], "/", req->col_pathname, 0, 2, 0, 16 );
    	}
 		amiga_textout ( &req->gpDraw[req->surf&(req->numGpDraw-1)], req->filename, req->col_pathname, req->zip?8:0, 2, 0, 16 );

    for (pathlistofs=pathliststartofs,y=-(yp&15);y<208;y+=16, pathlistofs++)
      {
        if (pathlistofs>=0 && pathlistofs<req->pathlistsize)
          {
            if (req->pathlist[pathlistofs]->isdir)
              amiga_printf ( &req->gpDraw[req->surf&(req->numGpDraw-1)], 0, y+20, req->col_dir, yb1, yb2, &req->pathlist[pathlistofs]->name[pathnamelen] );
            else
              {
                char size[32];
                char out[40];
                char * fname;
                int i;

                memset ( out, ' ', 40 );
                sprintf ( size, "%d", req->pathlist[pathlistofs]->size );
                strcpy ( &out[40-strlen(size)], size );
                fname = &req->pathlist[pathlistofs]->name[pathnamelen];
                for (i=0;i<(int)strlen(fname) && i<(40-(int)strlen(size)-1);i++)
                  out[i] = fname[i];
                amiga_textout ( &req->gpDraw[req->surf&(req->numGpDraw-1)], out, req->col_file, 0, y+20, yb1, yb2 );
              }
          }
      }

    if (req->pathlistsize>0)
    	draw_focus ( &req->gpDraw[req->surf&(req->numGpDraw-1)], focusy, req->col_xor );
  }

/* select_entry()
 * returns:
 * Index of selected pathlist entry
 * -1 on error
 * -2 on parent
 * -3 on cancel
 */
static int select_entry ( ZIPFREQUEST * req, int ipathselect, BOOL fade_in )
  {
    int yp=0;
    int ydir=0;
    int timer=0;
    int retval=-1;
    int bytes=0,files=0,dirs=0,i;
    int mode;
    int modecounter=0;

    if (req==NULL)
      return -1;

    if (fade_in && req->fade)
    	mode=1;
    else
    	mode=0;

    if (ipathselect>=0)
      yp = ipathselect*16;

    req->surf=0;

    if (req->pathlistsize>0)
    	{
    		for (i=files=dirs=bytes=0;i<req->pathlistsize;i++)
      		{
        		if (!req->pathlist[i]->isdir)
          		{
            		files++;
            		bytes += req->pathlist[i]->size;
          		} else
          		{
            		dirs++;
          		}
      		}
      }

    while (1)
      {
        int k;

        if (mode==0)
        	{
        		k = GpKeyGet();

        		if (k&GPC_VK_FB && req->enableCancel)
          		{
            		retval = -3;
								mode = 2;
								modecounter = 0;
          		}
        		else
        		if (k&GPC_VK_FL)
          		yp = 0;
        		else
        		if (k&GPC_VK_FR)
          		yp = (req->pathlistsize-1)*16;
        		else
        		if (k&GPC_VK_UP)
          		{
            		if (ydir==0)
              		ydir=-2;
          		}
        		else if (k&GPC_VK_DOWN)
          		{
            		if (ydir==0)
              		ydir=2;
          		}
        		else
        		if (k&GPC_VK_LEFT)
          		{
            		if (req->filename[0]!='\0')
              		{
                		retval = -2;
                		break;
              		}
          		}
        		else
        		if ( (k&(GPC_VK_RIGHT|GPC_VK_FA)) && (req->pathlistsize>0))
          		{
            		retval = yp>>4;
            		if (req->fade && !req->pathlist[retval]->isdir)
            			{
            				mode = 2;
            				modecounter = 0;
            			}
            		else
            			break;
          		}
        		else if ( (yp&~15) ^ ((yp+ydir)&~15) )
          		{
            		if (ydir>0)
              		yp = (yp+ydir) & ~15;
            		else
              		yp = yp & ~15;
            		timer=ydir=0;
          		}
          }

        if (mode==1)
        	{
	        	SetPal ( (PAL*)req->palette, modecounter );
	        	modecounter++;
	        	if (modecounter>16)
	        		{
		        		mode=0;
		        		modecounter=0;
	        		}
        	} else
        if (mode==2)
        	{
	        	SetPal ( (PAL*)req->palette, 16-modecounter );
	        	modecounter++;
	        	if (modecounter>16)
	        		return retval;
        	}

        if (ydir!=0)
          {
            yp+=ydir;
            timer++;
            if (timer>12)
              {
                if (abs(ydir)<16)
                  {
                    if (ydir>0)
                      ydir++;
                    else
                      ydir--;
                  }
                timer=0;
              }
          }

        if (yp<0)
          {
            yp=0;
            timer=ydir=0;
          }
        else
        if (yp>(req->pathlistsize-1)*16)
          {
            yp = (req->pathlistsize-1)*16;
            timer=ydir=0;
          }

        if (req->renderBG==NULL)
          GpClearSurface ( &req->gpDraw[req->surf&(req->numGpDraw-1)], req->col_bg );
        else
          req->renderBG ( &req->gpDraw[req->surf&(req->numGpDraw-1)] );
        display_entries ( req, yp, 176 );
        amiga_printf ( &req->gpDraw[req->surf&(req->numGpDraw-1)], 0, 200, req->col_info, -1, -1, "Total %d entries in %d bytes", req->pathlistsize, bytes );
/*        amiga_printf ( &req->gpDraw[req->surf&(req->numGpDraw-1)], 0, 200, req->col_info, -1, -1, "%d free mem", gm_availablesize() );*/
        amiga_printf ( &req->gpDraw[req->surf&(req->numGpDraw-1)], 0, 216, req->col_info, -1, -1, "%d files and %d dirs", files, dirs );
        amiga_printf ( &req->gpDraw[req->surf&(req->numGpDraw-1)], 320-(3*8), 216, req->col_info, -1, -1, req->zip?"ZIP":"SMC" );
        GpSurfaceSet(&req->gpDraw[req->surf&(req->numGpDraw-1)]);

        req->surf++;
      }

    return retval;
  }

FINFO ** getsmcdir ( const char * pathname, int * dirsize, FINFO ** finfodata )
	{
		int finfosize;
		ulong dsize;
		ulong read_count;
		GPDIRENTRY * gpdir = NULL;
		FINFO ** pdirlist = NULL;
		FINFO * dirlist = NULL;
		int i;

		if (pathname==NULL || dirsize==NULL || finfodata==NULL)
			return NULL;

		if (GpRelativePathSet ( pathname )!=SM_OK)
			return NULL;

		if (GpDirEnumNum("\\", &dsize)!=SM_OK)
			return NULL;

		gpdir = gm_malloc ( dsize * sizeof(GPDIRENTRY) );
		if (gpdir==NULL)
			return NULL;

		if (GpDirEnumList("\\", 0, dsize, gpdir, &read_count)!=SM_OK)
			{
				gm_free ( gpdir );
				return NULL;
			}

		if (read_count!=dsize)
			{
				gm_free ( gpdir );
				return NULL;
			}

		dirlist  = gm_malloc ( dsize * sizeof(FINFO) );
		pdirlist = gm_malloc ( dsize * sizeof(FINFO*) );

		for (i=finfosize=0;i<(int)dsize;i++)
			{
				if (strcmp(gpdir[i].name,".")!=0 &&
				    strcmp(gpdir[i].name,"..")!=0)
					{
						GPFILEATTR attr;

						if (GpFileAttr(gpdir[i].name, &attr)==SM_OK)
							{
								char * fname;
								char tempname[256];

								strcpy ( tempname, pathname );
								if (tempname[strlen(tempname)-1]!='\\')
									strcat ( tempname, "\\" );
								strcat ( tempname, gpdir[i].name );

								if ((attr.attr&0x10) == 0x10)
									{
										dirlist[finfosize].isdir = TRUE;
										dirlist[finfosize].size = -1;
										strcat ( tempname, "\\" );
									} else
									{
										dirlist[finfosize].isdir = FALSE;
										dirlist[finfosize].size = attr.size;
									}

								fname = gm_malloc ( strlen(tempname)+1 );
								strcpy ( fname, tempname );
								dirlist[finfosize].name = fname;

								pdirlist[finfosize] = &dirlist[finfosize];
								finfosize++;
							}
					}
			}

		gm_free(gpdir);
		sortdir ( pdirlist, finfosize );

		*finfodata = dirlist;
		*dirsize = finfosize;
		return pdirlist;
	}

int ZIPFileRequest ( ZIPFREQUEST * req )
  {
    char pathselect[256];
    int ipathselect=-1;
    int retval=0;
    int c;

    if (req==NULL)
      return -1;

    memset ( pathselect, 0, sizeof(pathselect) );
    if (strlen(req->filename)>0)
      {
        int i;
        int l = strlen ( req->filename );
        int s = -1;

        for (i=l-1;i>=0;i--)
          {
            if ( req->filename[i]=='/' )
              {
                s = i;
                break;
              }
          }

        if (s>=0)
          {
            strcpy ( pathselect, &req->filename[s+1] );
            req->filename[s+1] = '\0';
          } else
          {
	          strcpy ( pathselect, req->filename );
	          req->filename[0] = '\0';
          }
      } else
      {
        req->filename[0] = '\0';
      }

    c=0;
    while (1)
      {
        int entry;

        req->pathlist = ziplist_directory ( req->filename, req->ziplist, &req->pathlistsize );

        if (strlen(pathselect)>0)
          {
            int i;
            int o = strlen ( req->filename );

            for (i=0;i<req->pathlistsize;i++)
              {
                if (strcmp(pathselect, &req->pathlist[i]->name[o])==0)
                  {
                    ipathselect=i;
                    break;
                  }
              }
          }

        if (req->pathlist==NULL)
        	{
          	return -1;
          }

        entry = select_entry ( req, ipathselect, c==0 );
        c++;
        ipathselect=-1;
        while (GpKeyGet());
        if (entry>=0)
          {
            if (req->pathlist[entry]->isdir)
              {
                int nameofs = strlen ( req->filename );

                strcat ( req->filename, &req->pathlist[entry]->name[nameofs] );
              } else
              {
                strcpy ( req->filename, req->pathlist[entry]->name );
		gm_free ( req->pathlist );
                break;
              }
          } else
        if (entry==-2)
          {
            int i;
            int l = strlen(req->filename);

            if (l>1)
              {
                for (i=l-2;i>=0;i--)
                  {
                    if (req->filename[i]=='/')
                      break;
                  }
                strcpy ( pathselect, &req->filename[i+1] );
                req->filename[i+1] = '\0';
              }
          } else
          {
            retval=-1;
            gm_free ( req->pathlist );
            break;
          }
        gm_free ( req->pathlist );
      }
    return retval;
  }

static BOOL smcparent ( char * path, char * pathselect )
	{
		int i;
		int l;

		if (path==NULL)
			return FALSE;

    l = strlen(path);

    if (l>5)
      {
	      BOOL found = FALSE;
        for (i=l-2;i>=3;i--)
          {
            if (path[i]=='\\')
            	{
	            	found = TRUE;
              	break;
              }
          }
        if (!found)
        	return FALSE;
        if (pathselect!=NULL)
        	strcpy ( pathselect, &path[i+1] );
        path[i+1] = '\0';
      } else
      return FALSE;

    return TRUE;
	}

int SMCFileRequest ( ZIPFREQUEST * req )
  {
    char pathselect[256];
    int ipathselect=-1;
    int retval=0;
    FINFO * finfodata;
    int i;
    int c;

    if (req==NULL)
      return -1;

    memset ( pathselect, 0, sizeof(pathselect) );
    if (strlen(req->filename)>5)
      {
        int i;
        int l = strlen ( req->filename );
        int s = -1;

        for (i=l-1;i>=5;i--)
          {
            if ( req->filename[i]=='\\' )
              {
                s = i;
                break;
              }
          }

        if (s>=0)
          {
            strcpy ( pathselect, &req->filename[s+1] );
            req->filename[s+1] = '\0';
          } else
          {
	          strcpy ( pathselect, req->filename );
	          strcpy ( req->filename, "gp:\\gpmm\\" );
          }
      } else
      {
	      strcpy ( req->filename, "gp:\\gpmm\\" );
      }

    c=0;
    while (1)
      {
        int entry;

        while ( (req->pathlist = getsmcdir ( req->filename, &req->pathlistsize, &finfodata ))==NULL)
        	{
	        	if (!smcparent ( req->filename, NULL ))
	        		return -2;
        	}

        if (strlen(pathselect)>0)
          {
            int i;
            int o = strlen ( req->filename );

            for (i=0;i<req->pathlistsize;i++)
              {
                if (striicmp(pathselect, &req->pathlist[i]->name[o])==0)
                  {
                    ipathselect=i;
                    break;
                  }
              }
          }

        if (req->pathlist==NULL)
        	{
          	return -1;
          }

        entry = select_entry ( req, ipathselect, c==0 );
        c++;
        ipathselect=-1;
        while (GpKeyGet());
        if (entry>=0)	/* Entry selected */
          {
            if (req->pathlist[entry]->isdir)
              {
                int nameofs = strlen ( req->filename );

                strcat ( req->filename, &req->pathlist[entry]->name[nameofs] );
              } else
              {
                strcpy ( req->filename, req->pathlist[entry]->name );
        				for (i=0;i<req->pathlistsize;i++)
        					gm_free ( finfodata[i].name );
        				gm_free ( req->pathlist );
        				gm_free ( finfodata );
                break;
              }
          } else
        if (entry==-2)	/* Parent directory */
          {
	          smcparent ( req->filename, pathselect );
          } else
          {
            retval=-1;
		        for (i=0;i<req->pathlistsize;i++)
        			gm_free ( finfodata[i].name );
            gm_free ( req->pathlist );
            gm_free ( finfodata );
            break;
          }

        for (i=0;i<req->pathlistsize;i++)
        	gm_free ( finfodata[i].name );
        gm_free ( req->pathlist );
        gm_free ( finfodata );
      }
    return retval;
  }

int GpFileRequest ( ZIPFREQUEST * req )
	{
		if (req==NULL)
			return -2;

		if (req->zip)
			return ZIPFileRequest ( req );
		else
			return SMCFileRequest ( req );
	}

