#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>

#include "config.h"
#include "sdl/SDL_keysym.h"

static int fsize(const char * lpName) {

  FILE *fh;
  long s;
	  
  if (!(fh = fopen(lpName, "rb"))) {

	  return -1;
  }
	  
	fseek(fh, 0, SEEK_END);
	s = ftell(fh);
	  
	fclose(fh);
	  
	return s;
}

static int isVal(const char *c) {

  int l = strlen(c);
  int i = 0;

  if (c[0] == '+' || c[0] == '-') {

    i++;
  }

  for (; i < l; i++) {

    if (!isdigit(c[i])) {

      return 0;
    }
  }
  
  return 1;
}

int Config::splitoption(char *line, char *opt1, char *opt2) {

  unsigned int i;
  int status = 0;
  int oi1=0, oi2=0;

  for (i = 0; i < strlen(line); i++) {

    char c = line[i];

    if (status == 0) {

      if (isalpha(c)) {

        status = 1;
        opt1[oi1++] = c;
      } else
      if (c == '#') {

        return 2;
      } else
      if (!isspace(c)) {

        return 1;
      }
    } else
    if (status == 1) {

	    if (isspace(c)) {

        status = 2;
      } else
      if (isalpha(c)) {

        opt1[oi1++] = c;
      } else
      if (c == '=') {

        status = 3;
      } else {

        return 1;
      }
    } else
    if (status == 2) {

      if (c == '=') {

        status = 3;
      } else
      if (!isspace(c)) {

        return 1;
      }
    } else
    if (status == 3) {

      if (isspace(c)) {

        status = 4;
      } else
      if (isalpha(c)) {

        status = 5;
        opt2[oi2++] = c;
      }
    } else
    if (status == 4) {

      if (isalpha(c) || isdigit(c)) {

        status = 5;
        opt2[oi2++] = c;
      } else
      if (!isspace(c)) {

        return 1;
      }
    } else
    if (status == 5) {

      if (isalpha(c) || isdigit(c)) {

        opt2[oi2++] = c;
      } else
      if (isspace(c)) {

        status = 6;
      } else
      if (c == '#') {

        status = 7;
      } else {

        return 1;
      }
    } else
    if (status == 6) {

      if (c == '#') {

        status = 7;
      } else
      if (!isspace(c)) {

        return 1;
      }
    }
  }

  opt1[oi1] = '\0';
  opt2[oi2] = '\0';
  return oi1 == 0 || oi2 == 0 ? 2 : 0;
}

/* returns number of errors found */
int Config::check() {

  int retval = 0, i, curline;

  if (!isLoaded()) {

    return -1;
  }

  if (configFileSize < 0) {

    return -1;
  }

  for (i = 0, curline = 0; i < configFileSize; curline++) {

    int ret,sl;
    char opt1[256], opt2[256];
    char *line = &((char *)configFile)[i];
    sl = strlen(line);

    if ((ret = splitoption(line, opt1, opt2)) != 0) {

      if (ret != 2) {

        retval++;
        fprintf(stderr, "%s: Syntax error in line %d\n", this->fname, curline + 1);
      }
    }

    i += sl + 1;
  }

  return retval;
}

Config::Config() {
  
  printf("Config()\n");
}

Config::~Config() {

  printf("~Config()\n");
  destroy();
}

void Config::destroy() {
  
  if (isLoaded()) {
	  
    free(configFile);
    configFile = NULL;
  }
}

int Config::load(const char *fname) {

  FILE *f;
  int i,fs;

  if ((fs = fsize(fname)) == -1) {

    return -1;
  }

  this->fname = fname;
  this->configFileSize = fs;

  if (!(f = fopen(fname, "rb"))) {

    destroy();
    return -1;
  }

  this->configFile = malloc(fs);

  if (!this->configFile) {

    destroy();
    return -1;
  }

  fread(this->configFile, fs, 1, f);
  fclose(f);

  for (i = 0; i < fs; i++) {

    if (((char *)this->configFile)[i] == 0x0d) {

      ((char *)this->configFile)[i] = ' ';
    }

    if (((char *)this->configFile)[i] == 0x0a) {

      ((char *)this->configFile)[i] = '\0';
    }

    ((char *)this->configFile)[i] = tolower(((char *)this->configFile)[i]);
  }

  check();

  return 0;
}

int Config::getContentsVal(const char *k, int type) {

  if (!k) {

    return -1;
  }

  switch (type) {

    case 0: /* Value */
      if (!isVal(k)) {

        return -2;
      } else {

      	return atoi(k);
    	}
      break;

    case 1: /* Boolean */ {
      int w = (strcmp(k, "on") == 0)  || (strcmp(k, "true") == 0);
      int f = (strcmp(k, "off") == 0) || (strcmp(k, "false") == 0);

      if (!(w || f)) {

        return -2;
      }

      return w && !f;
    }

    case 2: /* Key value (SDL) */ {

      if (strcmp(k, "backspace") == 0)    { return SDLK_BACKSPACE; } else
      if (strcmp(k, "tab") == 0)          { return SDLK_TAB;  } else
      if (strcmp(k, "return") == 0)       { return SDLK_RETURN; } else
      if (strcmp(k, "pause") == 0)        { return SDLK_PAUSE; } else
      if (strcmp(k, "escape") == 0)       { return SDLK_ESCAPE; } else
      if (strcmp(k, "space") == 0)        { return SDLK_SPACE; } else
      if (strcmp(k, "comma") == 0)        { return SDLK_COMMA; } else
      if (strcmp(k, "period") == 0)       { return SDLK_PERIOD; } else
      if (strcmp(k, "0") == 0)            { return SDLK_0; } else
      if (strcmp(k, "1") == 0)            { return SDLK_1; } else
      if (strcmp(k, "2") == 0)            { return SDLK_2; } else
      if (strcmp(k, "3") == 0)            { return SDLK_3; } else
      if (strcmp(k, "4") == 0)            { return SDLK_4; } else
      if (strcmp(k, "5") == 0)            { return SDLK_5; } else
      if (strcmp(k, "6") == 0)            { return SDLK_6; } else
      if (strcmp(k, "7") == 0)            { return SDLK_7; } else
      if (strcmp(k, "8") == 0)            { return SDLK_8; } else
      if (strcmp(k, "9") == 0)            { return SDLK_9; } else
      if (strcmp(k, "caret") == 0)        { return SDLK_CARET; } else
      if (strcmp(k, "a") == 0)            { return SDLK_a; } else
      if (strcmp(k, "b") == 0)            { return SDLK_b; } else
      if (strcmp(k, "c") == 0)            { return SDLK_c; } else
      if (strcmp(k, "d") == 0)            { return SDLK_d; } else
      if (strcmp(k, "e") == 0)            { return SDLK_e; } else
      if (strcmp(k, "f") == 0)            { return SDLK_f; } else
      if (strcmp(k, "g") == 0)            { return SDLK_g; } else
      if (strcmp(k, "h") == 0)            { return SDLK_h; } else
      if (strcmp(k, "i") == 0)            { return SDLK_i; } else
      if (strcmp(k, "j") == 0)            { return SDLK_j; } else
      if (strcmp(k, "k") == 0)            { return SDLK_k; } else
      if (strcmp(k, "l") == 0)            { return SDLK_l; } else
      if (strcmp(k, "m") == 0)            { return SDLK_m; } else
      if (strcmp(k, "n") == 0)            { return SDLK_n; } else
      if (strcmp(k, "o") == 0)            { return SDLK_o; } else
      if (strcmp(k, "p") == 0)            { return SDLK_p; } else
      if (strcmp(k, "q") == 0)            { return SDLK_q; } else
      if (strcmp(k, "r") == 0)            { return SDLK_r; } else
      if (strcmp(k, "s") == 0)            { return SDLK_s; } else
      if (strcmp(k, "t") == 0)            { return SDLK_t; } else
      if (strcmp(k, "u") == 0)            { return SDLK_u; } else
      if (strcmp(k, "v") == 0)            { return SDLK_v; } else
      if (strcmp(k, "w") == 0)            { return SDLK_w; } else
      if (strcmp(k, "x") == 0)            { return SDLK_x; } else
      if (strcmp(k, "y") == 0)            { return SDLK_y; } else
      if (strcmp(k, "z") == 0)            { return SDLK_z; } else
      if (strcmp(k, "delete") == 0)       { return SDLK_DELETE; } else
      if (strcmp(k, "kp0") == 0)          { return SDLK_KP0; } else
      if (strcmp(k, "kp1") == 0)          { return SDLK_KP1; } else
      if (strcmp(k, "kp2") == 0)          { return SDLK_KP2; } else
      if (strcmp(k, "kp3") == 0)          { return SDLK_KP3; } else
      if (strcmp(k, "kp4") == 0)          { return SDLK_KP4; } else
      if (strcmp(k, "kp5") == 0)          { return SDLK_KP5; } else
      if (strcmp(k, "kp6") == 0)          { return SDLK_KP6; } else
      if (strcmp(k, "kp7") == 0)          { return SDLK_KP7; } else
      if (strcmp(k, "kp8") == 0)          { return SDLK_KP8; } else
      if (strcmp(k, "kp9") == 0)          { return SDLK_KP9; } else
      if (strcmp(k, "kp_period") == 0)    { return SDLK_KP_PERIOD; } else
      if (strcmp(k, "kp_divide") == 0)    { return SDLK_KP_DIVIDE; } else
      if (strcmp(k, "kp_multiply") == 0)  { return SDLK_KP_MULTIPLY; } else
      if (strcmp(k, "kp_minus") == 0)     { return SDLK_KP_MINUS; } else
      if (strcmp(k, "kp_plus") == 0)      { return SDLK_KP_PLUS; } else
      if (strcmp(k, "kp_enter") == 0)     { return SDLK_KP_ENTER; } else
      if (strcmp(k, "kp_equals") == 0)    { return SDLK_KP_EQUALS; } else
      if (strcmp(k, "up") == 0)           { return SDLK_UP; } else
      if (strcmp(k, "down") == 0)         { return SDLK_DOWN; } else
      if (strcmp(k, "left") == 0)         { return SDLK_LEFT; } else
      if (strcmp(k, "right") == 0)        { return SDLK_RIGHT; } else
      if (strcmp(k, "insert") == 0)       { return SDLK_INSERT; } else
      if (strcmp(k, "home") == 0)         { return SDLK_HOME; } else
      if (strcmp(k, "end") == 0)          { return SDLK_END; } else
      if (strcmp(k, "pageup") == 0)       { return SDLK_PAGEUP; } else
      if (strcmp(k, "pagedown") == 0)     { return SDLK_PAGEDOWN; } else
      if (strcmp(k, "f1") == 0)           { return SDLK_F1; } else
      if (strcmp(k, "f2") == 0)           { return SDLK_F2; } else
      if (strcmp(k, "f3") == 0)           { return SDLK_F3; } else
      if (strcmp(k, "f4") == 0)           { return SDLK_F4; } else
      if (strcmp(k, "f5") == 0)           { return SDLK_F5; } else
      if (strcmp(k, "f6") == 0)           { return SDLK_F6; } else
      if (strcmp(k, "f7") == 0)           { return SDLK_F7; } else
      if (strcmp(k, "f8") == 0)           { return SDLK_F8; } else
      if (strcmp(k, "f9") == 0)           { return SDLK_F9; } else
      if (strcmp(k, "f10") == 0)          { return SDLK_F10; } else
      if (strcmp(k, "f11") == 0)          { return SDLK_F11; } else
      if (strcmp(k, "f12") == 0)          { return SDLK_F12; } else
      if (strcmp(k, "f13") == 0)          { return SDLK_F13; } else
      if (strcmp(k, "f14") == 0)          { return SDLK_F14; } else
      if (strcmp(k, "f15") == 0)          { return SDLK_F15; } else
      if (strcmp(k, "rshift") == 0)       { return SDLK_RSHIFT; } else
      if (strcmp(k, "lshift") == 0)       { return SDLK_LSHIFT; } else
      if (strcmp(k, "rctrl") == 0)        { return SDLK_RCTRL; } else
      if (strcmp(k, "lctrl") == 0)        { return SDLK_LCTRL; } else
      if (strcmp(k, "ralt") == 0)         { return SDLK_RALT; } else
      if (strcmp(k, "lalt") == 0)         { return SDLK_LALT; } else
      if (strcmp(k, "lsuper") == 0)       { return SDLK_LSUPER; } else
      if (strcmp(k, "rsuper") == 0)       { return SDLK_RSUPER; } else
      if (strcmp(k, "print") == 0)        { return SDLK_PRINT; } else
                                          { return -2; }
      break;
    }

    default:
      return -3;
      break;
  }
}

bool Config::isLoaded() {
  
  return configFile != NULL;
}

/*
  type:
  0 - Value
  1 - Boolean
  2 - Key value (SDL)
*/
int Config::getValue(const char *var, int type ) {

  int i, curline;

  if (!isLoaded()) {

    return -1;
  }

  if (!var) {

    return -1;
  }

  if (!isLoaded()) {

    return -1;
  }

  if (configFileSize < 0) {

    return -1;
  }

  for (i = 0, curline = 0; i < configFileSize; curline++) {

    int ret, sl;
    char opt1[256], opt2[256];
    char *line = &((char *)configFile)[i];
    sl = strlen(line);

    if ((ret = splitoption(line, opt1, opt2)) == 0) {

      if (strcmp(var, opt1) == 0) {

        ret = getContentsVal (opt2, type);

        if (ret < 0) {

          switch (type) {

            case 0: /* Value */
              fprintf(stderr, "%s: Value expected in line %d\n", this->fname, curline + 1);
              break;

            case 1: /* Boolean */
              fprintf(stderr, "%s: Boolean expected in line %d\n", this->fname, curline + 1);
              break;

            case 2: /* SDL key */
              fprintf(stderr, "%s: Key expected in line %d\n", this->fname, curline + 1);
              break;

            default:
              break;
          }

          return -1;
        }

        return ret;
      }
    }

    i += sl + 1;
  }

  fprintf(stderr, "%s: Setting for '%s' not found\n", this->fname, var);
  return -1;
}
