#include <kos.h>
#include "archdep/dc/filereq.h"
#include "emu/ggms.h"

u8 * screenbuf;
GGMS machine;
u32 palette[256];

void preRender ( GGVDP * vdp )
  {
    vdp->display = screenbuf;
  }

void postRender ( GGVDP * vdp )
  {
  }

void copyDisplay ( GGVDP * vdp )
  {
	  int i;
	  cont_cond_t ctrl;
	  uint8 cont;
  
    if ( vdp->palchange )
      {
        int i;
        /* Set palette */
        if ( vdp->gg )
          {
            for ( i=0; i<32; i++ )
              {
                u16 r = (vdp->cram[(i<<1)+0] << 4) & 0xe0;
                u16 g = (vdp->cram[(i<<1)+0] << 0) & 0xe0;
                u16 b = (vdp->cram[(i<<1)+1] << 4) & 0xe0;
                palette[i] = r<<16 | g<<8 | b;
              }
          } else
          {
            for ( i=0; i<32; i++ )
              {
                u16 r = (vdp->cram[i] << 6) & 0xc0;
                u16 g = (vdp->cram[i] << 4) & 0xc0;
                u16 b = (vdp->cram[i] << 2) & 0xc0;
                palette[i] = r<<16 | g<<8 | b;
              }
          }
        vdp->palchange=FALSE;
      }

    cont = maple_first_controller();
    if (cont)
    	{
      	while (cont_get_cond ( cont, &ctrl ) < 0);
      	
      	machine.p1_keys[0] = ctrl.buttons&CONT_DPAD_LEFT  ? 1 : 0;
      	machine.p1_keys[1] = ctrl.buttons&CONT_DPAD_RIGHT ? 1 : 0;
      	machine.p1_keys[2] = ctrl.buttons&CONT_DPAD_UP    ? 1 : 0;
      	machine.p1_keys[3] = ctrl.buttons&CONT_DPAD_DOWN  ? 1 : 0;
      	machine.p1_keys[4] = ctrl.buttons&CONT_B          ? 1 : 0;
      	machine.p1_keys[5] = ctrl.buttons&CONT_A          ? 1 : 0;
      	machine.p1_keys[6] = ctrl.buttons&CONT_START      ? 1 : 0;
      }
      	

    for (i=0;i<320*240;i++)
	   	vram_l[i] = palette[screenbuf[i]];
  }


int main(int argc, char **argv)
  {
    screenbuf = calloc ( 1, 320*240 );
    /* Init Video */
    vid_set_mode(DM_320x240, PM_RGB888);
    
    palette[128] = 0x00000000;
    palette[129] = 0x00ff0000;
    palette[130] = 0x00ffffff;
    palette[131] = 0x0000ff00;
    palette[132] = 0x000000ff;


    while (1)
    	{
	    	char fname[256];
	    	
	      FSEL fsel;
	            	
	      fsel.fname = fname;
	      fsel.palette = palette;
	      fsel.bgcolor = 128;
	      fsel.filecolor = 130;
	      fsel.dircolor = 132;
	      fsel.curdircolor = 129;
       	vid_set_mode(DM_640x480, PM_RGB888);
       	FileRequest ( &fsel, "/" );
      	vid_set_mode(DM_320x240, PM_RGB888);


        /* Initialize the emulation */
        if (GGMS_Init ( &machine, fname, preRender, postRender, copyDisplay )==0)
          {
            machine.vdp->display = screenbuf;
            machine.bios_en = FALSE;
            machine.memcontrol = 0x08;

            machine.vdp->dx = 320;
            machine.vdp->dy = 240;
            GGVDP_Zoom ( machine.vdp, machine.vdp->screenw, machine.vdp->screenh );
            machine.vdp->xofs = (320-machine.vdp->screenw)>>1;
            machine.vdp->yofs = (240-machine.vdp->screenh)>>1;
            memset ( screenbuf, 0, 320*240 );

            while(1)
              {
	              cont_cond_t ctrl;
	              uint8 cont;
                int i;
        
                for (i=0;i<192;++i)
                  GGMS_Run ( &machine );

    						cont = maple_first_controller();
    						if (cont)
    							{
      							while (cont_get_cond ( cont, &ctrl ) < 0);
      							
      							if (!(ctrl.buttons&CONT_START))
      								break;
      						}

              }
            GGMS_Terminate ( &machine );
          }
      }

    while (1);
    return 0;
  }
