/* emu.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include "archdep/archdep.h"
#include "emu/defines.h"
#include "emu/GGMS.h"
#include "sdl/sdl.h"
#include "emu.h"


extern int debug;
int initgfx(EMU *);

void EMU_JoystickEnable(EMU *emu, BOOL en) {

  int i = 0;

  if (!emu) {

    return;
  }

  if (en) {

    int num = SDL_NumJoysticks();

    if (emu->joy) {

      return;
    }

    for (i = 0; i < num; i++) {

      if ((emu->joy=SDL_JoystickOpen(i))) {

        SDL_JoystickEventState(SDL_IGNORE);
        return;
      }
    }
  } else {

    if (!emu->joy) {

      return;
    }
    SDL_JoystickClose(emu->joy);
  }
}

void EMU_SetFrameSkip(EMU *emu, int fs) {

  if (!emu)
    return;

  emu->renderframe_amt = fs;
}

void EMU_FullStretchEnable(EMU *emu, BOOL en) {

  if (!emu)
    return;

  emu->fullscreen_stretch = en;
}

void EMU_SpeedLimitEnable(EMU *emu, BOOL en) {

  if (!emu) {

    return;
  }

  emu->speedlimit = en;
}

BOOL EMU_GetSRAMFileName(EMU *emu, char *dest) {

  int i;

  if (!emu) {

    return FALSE;
  }

  if (strlen(emu->fname) == 0) {

    return FALSE;
  }

  for (i = (int)(strlen(emu->fname) - 1); i >= 0; i--) {

    char c = emu->fname[i];
    if ((c == ':') || (c == '\\') || (c == '/')) {

      i++;
      if (i >= (int)strlen(emu->fname)) {

        i--;
      }
      break;
    }
  }

  strcpy(dest, emu->progpath);
  strcat(dest, "savegames\\");
  strcat(dest, &emu->fname[i]);
  strcat(dest, ".sav");

  return TRUE;
}

void EMU_SaveSRAM(EMU *emu) {

  char sramfname[256];

  if (!EMU_GetSRAMFileName(emu, sramfname)) {

    return;
  }

  if (!emu->machine) {

    return;
  }

  if (!emu->machine->sram_changed) {

    return;
  }

  if (!GGMS_SaveSRAM(emu->machine, sramfname)) {

    fprintf(stderr,"Couldn't save\n");
  }
}

void EMU_LoadSRAM(EMU *emu) {

  char sramfname[256];

  if (!EMU_GetSRAMFileName(emu, sramfname)) {

    return;
  }  

  if (!emu->machine) {

    return;
  }

  GGMS_LoadSRAM (emu->machine, sramfname);
}

BOOL fexists(char *fname) {

  FILE *fh;
    
  if (!(fh = fopen(fname, "rb"))) {

    return FALSE;
  }

  fclose(fh);
    
  return TRUE;
}

void EMU_EnableBIOS(EMU *emu, BOOL en) {

  if (!emu) {

    return;
  }

  emu->bios_en = en;

  if (!emu->machine) {

    return;
  }

  if (!emu->bios_en) {

    emu->machine->bios_en = FALSE;
    emu->machine->memcontrol = 0x08;
  } else {

    emu->machine->bios_en = TRUE;
  }

  GGMS_UpdateAllPages(emu->machine);
}


BOOL EMU_AudioEnable(EMU *emu) {

  if (!emu) {

    return FALSE;
  }

  if (emu->audio) {

    return FALSE;
  }

  if (SDL_OpenAudio(&emu->audioSpec,NULL) < 0) {

    emu->audioavail = FALSE;
    return FALSE;
  }

  emu->audio = TRUE;
  emu->audioavail = TRUE;

  SDL_PauseAudio(0);

  return TRUE;
}

BOOL EMU_AudioDisable(EMU *emu) {

  if (!emu) {

    return FALSE;
  }

  if (!emu->audio || !emu->audioavail) {

    return FALSE;
  }

  SDL_CloseAudio();
  emu->audio = FALSE;

  return TRUE;
}

void EMU_FullscreenEnable(EMU *emu, BOOL en) {

  if (!emu) {

    return;
  }

  if (!emu->machine) {

    return;
  }

  emu->fullscreen = en;
  initgfx(emu);
}


void preRender(GGVDP *vdp) {

  EMU *emu = (EMU*)((GGMS*)vdp->val)->val; /* vdp->GGMS->EMU */

  if (SDL_MUSTLOCK(emu->screenbuf)) {

    SDL_LockSurface(emu->screenbuf);
  }

  vdp->display = emu->screenbuf->pixels;
}

void postRender(GGVDP *vdp) {

  EMU *emu = (EMU*)((GGMS*)vdp->val)->val; /* vdp->GGMS->EMU */

  if (SDL_MUSTLOCK(emu->screenbuf)) {

    SDL_UnlockSurface(emu->screenbuf);
  }
}


void copyDisplay(GGVDP *vdp) {

  int i;
  SDL_Color colors[256];
  EMU *emu = (EMU*)((GGMS*)vdp->val)->val; /* vdp->GGMS->EMU */

  if (vdp->palchange) {

    /* Set palette */
    if (vdp->gg) {

      for (i = 0; i < 32; i++) {

        colors[i].r = ((vdp->cram[(i << 1) + 0] << 4) & 0xe0);
        colors[i].g = ((vdp->cram[(i << 1) + 0] << 0) & 0xe0);
        colors[i].b = ((vdp->cram[(i << 1) + 1] << 4) & 0xe0);
      }
    } else {

      for (i = 0; i < 32; i++) {

        colors[i].r = (vdp->cram[i] << 6) & 0xc0;
        colors[i].g = (vdp->cram[i] << 4) & 0xc0;
        colors[i].b = (vdp->cram[i] << 2) & 0xc0;
      }
    }

    colors[255].r = 0;
    colors[255].g = 0;
    colors[255].b = 0;

    SDL_SetPalette(emu->screen, SDL_LOGPAL | SDL_PHYSPAL, colors, 0, 32);
    SDL_SetColors(emu->screenbuf,
                  emu->screen->format->palette->colors, 0,
                  emu->screen->format->palette->ncolors);
    vdp->palchange=FALSE;
  }

  SDL_BlitSurface(emu->screenbuf, NULL , emu->screen, NULL);
  SDL_Flip(emu->screen);
  SDL_FillRect(emu->screenbuf, NULL, 0);
}

int initgfx (EMU *emu) {

  int i, x = 256, y = 192;
  SDL_Color colors[256];

  if (emu->fullscreen) {

    x = emu->fullw;
    y = emu->fullh;
  } else {

    if (emu->machine) {

      if (emu->machine->vdp) {

        x = emu->machine->vdp->dx;
        y = emu->machine->vdp->dy;
      }
    } else {

      x=256;
      y=192;
    }
/*        x += xb;
        y += yb; */
  }

  for (i = 0; i < 256; i++) {

    colors[i].r = i;
    colors[i].g = i;
    colors[i].b = i;
  }

  if (emu->screen) {

    SDL_FreeSurface(emu->screen);
  }

  if (emu->screenbuf) {

    SDL_FreeSurface(emu->screenbuf);
  }

  emu->screen = SDL_SetVideoMode(x, y, 8,
                (SDL_GetVideoInfo()->hw_available ? SDL_HWSURFACE : SDL_SWSURFACE) | 
                SDL_HWPALETTE |
                (emu->fullscreen ? SDL_FULLSCREEN : 0) |
                SDL_RESIZABLE);
  if (!emu->screen) {

    fprintf(stderr, "Couldn't set video mode: %s\n", SDL_GetError());
    return 1;
  }

  emu->screenbuf = SDL_DisplayFormat(emu->screen);
  SDL_WM_SetCaption("Masterlator v0.07b", NULL);
  SDL_SetPalette(emu->screen, SDL_LOGPAL | SDL_PHYSPAL, colors, 0, 256);

  if (emu->machine) {

    if (emu->machine->vdp) {

      emu->machine->vdp->dx = emu->screen->w;
      emu->machine->vdp->dy = emu->screen->h;
      emu->machine->vdp->xofs = (emu->machine->vdp->dx - emu->machine->vdp->screenw) >> 1;
      emu->machine->vdp->yofs = (emu->machine->vdp->dy - emu->machine->vdp->screenh) >> 1;
      emu->machine->vdp->palchange = TRUE;
    }
  }

  return 0;
}

void EMU_Zoom(EMU *emu, int x) {

  if (!emu) {
    
    return;
  }

  if (!emu->machine) {
    
    return;
  }
  
  if (emu->fullscreen) {
    
    return;
  }

  emu->machine->vdp->dx = emu->machine->vdp->screenw * x;
  emu->machine->vdp->dy = emu->machine->vdp->screenh * x;
  initgfx(emu);
}

int pollmachine(EMU *emu) {

  SDL_Event event;

  if (emu->joy) {

    int i;
    s16 value;

    SDL_JoystickUpdate();
        /* Axis */
    if (emu->machine) {

      for (i = 0; i < 2; i++) {

        value = SDL_JoystickGetAxis(emu->joy, i);
        if ((value > -16384) && (value < 16384)) {

          /* Axis released */
          emu->machine->p1_keys[(i << 1) | 0] = 1;
          emu->machine->p1_keys[(i << 1) | 1] = 1;
        } else {

          u8 state = value <= -16384 ? 0 : 1;
          emu->machine->p1_keys[(i << 1) | 0] = state;
          emu->machine->p1_keys[(i << 1) | 1] = state ^ 1;
        }
      }

      /* Buttons */
      emu->machine->p1_keys[4] = SDL_JoystickGetButton(emu->joy, 0) ^ 1;
      emu->machine->p1_keys[5] = SDL_JoystickGetButton(emu->joy, 1) ^ 1;
      emu->machine->p1_keys[6] = SDL_JoystickGetButton(emu->joy, 2) ^ 1;
    }
  }

  /* Check event */
  while (SDL_PollEvent(&event) != 0) {

    switch (event.type) {

      case SDL_QUIT:
        EMU_SaveSRAM(emu);
        return 1;
        break;

      case SDL_VIDEORESIZE:
        if (emu) {

          if (emu->machine) {

            if (emu->machine->vdp) {

              emu->machine->vdp->dx = event.resize.w;
              emu->machine->vdp->dy = event.resize.h;
            }
          }

          initgfx(emu);
        }
        break;

      case SDL_KEYDOWN:
        if (emu) {

          switch (event.key.keysym.sym) {

            case SDLK_s:
              GGMS_SaveState(emu->machine, "save.state");
               break;

            case SDLK_l:
              GGMS_LoadState(emu->machine, "save.state");
              break;

            case SDLK_1:
              EMU_Zoom(emu, 1);
              break;

            case SDLK_2:
              EMU_Zoom(emu, 2);
              break;

            case SDLK_3:
              EMU_Zoom(emu, 3);
              break;

            case SDLK_4:
              EMU_Zoom ( emu, 4 );
              break;

            case SDLK_RETURN:
              if (SDL_GetModState() & KMOD_LALT) {

                EMU_FullscreenEnable(emu, !emu->fullscreen);
              }
              break;

            case SDLK_r:
              if (emu->machine != NULL) {

                GGMS_Reset(emu->machine);
              }
              break;

            case SDLK_d:
              debug ^= 1;
/*                        if ( debug == 1 )
                          fprintf(stderr,"Debug on\n");
                        else
                          fprintf(stderr,"Debug off\n");*/
              break;

            case SDLK_ESCAPE:
              return 1;
              break;

            default:
              break;
          }
        }

      case SDL_KEYUP:
        if (emu) {

          if (emu->machine) {

            u8 state = ((event.key.type == SDL_KEYDOWN) ? 0 : 1);
            int key = event.key.keysym.sym;

            if (!emu->joy) {

              if (key == emu->key_left) {

                emu->machine->p1_keys[0] = state;
              } else
              if (key == emu->key_right) {

                emu->machine->p1_keys[1] = state;
              } else
              if (key == emu->key_up) {

                emu->machine->p1_keys[2] = state;
              } else
              if (key == emu->key_down) {

                emu->machine->p1_keys[3] = state;
              } else
              if (key == emu->key_a) {

                emu->machine->p1_keys[4] = state;
              } else
              if (key == emu->key_b) {

                emu->machine->p1_keys[5] = state;
              } else
              if (key == emu->key_start) {

                emu->machine->p1_keys[6] = state;
              }
            }
          }
        }
    }
  }

  return 0;
}

static void mixaudio(void *userdata, Uint8 *stream, int len) {

  EMU *emu = (EMU *)userdata;

  if (emu->machine) {

    if (emu->machine->snd) {

	    emu->machine->snd->buflen = len / 2;
      SN76489_Cycle(emu->machine->snd, stream);
/*            memcpy ( stream, emu->machine->snd->buf[0], len );
            SDL_MixAudio ( stream, emu->machine->snd->buf[1], len, SDL_MIX_MAXVOLUME );
            SDL_MixAudio ( stream, emu->machine->snd->buf[2], len, SDL_MIX_MAXVOLUME );
            SDL_MixAudio ( stream, emu->machine->snd->buf[3], len, SDL_MIX_MAXVOLUME );*/
    }
  }
}

BOOL EMU_LoadROM(EMU *emu, const char *fname) {

  GGMS *m = NULL;

  m = (GGMS*)malloc (sizeof(GGMS));
  if (GGMS_Init(m, fname, preRender, postRender, copyDisplay)) {

    free(m);
    return FALSE;
  }

  m->jap = emu->jap;
  EMU_AudioDisable(emu);
  EMU_SaveSRAM (emu);

  if (emu->machine) {

    GGMS_Terminate(emu->machine, TRUE);
    free(emu->machine);
  }
  emu->machine = m;

  EMU_EnableBIOS(emu, emu->bios_en);
  strcpy (emu->fname, fname);

  emu->machine->val = (void *)emu;
  emu->machine->vdp->display = emu->screenbuf->pixels;

  if (emu->audioen) {

    EMU_AudioEnable(emu);
  }

  strcpy (emu->fname, fname);

  EMU_LoadSRAM(emu);
  initgfx(emu);

  return TRUE;
}

EMU *EMU_Create(char *fname, char *argv[]) {

  EMU *emu = NULL;
  char sgpath[256];
  int i;

  emu = (EMU *)malloc(sizeof(EMU));
  if (!emu) {

    return NULL;
  }

  memset(emu, 0, sizeof(EMU));

/*    init_window(emu, 256, 192);*/

    /* Initialize SDL */
  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) < 0) {

    fprintf(stderr,"Couldn't initialize SDL\n");
/*        return 1; */
  }
  atexit(SDL_Quit);

  emu->audio = FALSE;
  emu->fullscreen = FALSE;
  emu->fullw = 320;
  emu->fullh = 200;

  emu->audioSpec.freq = 22050;
  emu->audioSpec.format = AUDIO_S16;
  emu->audioSpec.channels = 1;
  emu->audioSpec.samples = 512;
  emu->audioSpec.callback = mixaudio;
  emu->audioSpec.userdata = (void *)emu;
  emu->audioavail = TRUE;
  emu->audio = FALSE;
  emu->audioen = FALSE;

  emu->screen = NULL;
  emu->screenbuf = NULL;

  emu->speedlimit = TRUE;

/*    if (emu->audioavail)
      SDL_PauseAudio(0); */
  emu->audio = emu->audioavail;

  initgfx(emu);
  if (emu->machine) {

    emu->machine->vdp->display = emu->screenbuf->pixels;
  }

  emu->joy = NULL;

  emu->tick1 = emu->tick2 = SDL_GetTicks();
  emu->bios_en = TRUE;

  emu->jap = FALSE;

  emu->renderframe_amt = 1;
  emu->renderframe_cnt = 0;

  for (i = strlen(argv[0]) - 1; i >= 0; i--) {

    if ((argv[0][i] == '\\') || (argv[0][i] == '/') || (argv[0][i] == ':')) {

      i++;
      break;
    }
  }

  strcpy(emu->progpath, argv[0]);
  emu->progpath[i] = '\0';

  strcpy(sgpath, emu->progpath);
  strcat(sgpath, "savegames");
/*    if (!fexists(sgpath))
      CreateDirectory ( sgpath, NULL ); */

  emu->fname[0] = '\0';

  if (fname) {

    EMU_LoadROM(emu, fname);
  }

  return emu;
}

void EMU_Terminate ( EMU *emu) {

  if (!emu) {

    return;
  }

  if (emu->machine) {

    GGMS_Terminate(emu->machine, TRUE);
    free(emu->machine);
    emu->machine = NULL;
  }

  if (emu->screenbuf) {

    SDL_FreeSurface(emu->screenbuf);
  }

  if (emu->screen) {

    SDL_FreeSurface(emu->screen);
  }

  if (emu->joy) {

    SDL_JoystickClose(emu->joy);
  }

  if (emu->audioavail) {

    SDL_PauseAudio(1);
    SDL_CloseAudio();
  }
/*    DestroyWindow ( emu->g_hwnd );*/

  free(emu);
}

int EMU_RunFrame(EMU *emu) {

  int i, j;

  j = -1;

  if (emu) {

    j = pollmachine(emu);

    if (emu->machine) {

      for (i = 0; i < 262; i++) {

        GGMS_Run(emu->machine);
      }

      if (emu->renderframe_cnt <= 0 && emu->speedlimit) {

        int tick;
        int r = emu->renderframe_amt == 0 ? 1 : emu->renderframe_amt;

        while (((tick = SDL_GetTicks()) - emu->tick1) < (r*17));
        emu->tick1 = tick;
      }

      if (emu->renderframe_amt < 2) {

        emu->machine->vdp->render = TRUE;
      } else {

        if (emu->renderframe_cnt <= 0) {

          emu->renderframe_cnt = emu->renderframe_amt;
          emu->machine->vdp->render = TRUE;
        } else {

          emu->machine->vdp->render = FALSE;
        }

        emu->renderframe_cnt--;
      }
    }
  }

  return j;
}
