/* emu.h */

#ifndef __EMU_H__
#define __EMU_H__

#include "SDL/SDL.h"
#include "emu/defines.h"
#include "emu/GGMS.h"

typedef struct EMU {

  GGMS *machine;
  SDL_Surface *screen, *screenbuf;
  u32 g_hwnd;
  int tick1, tick2;
  int fullw, fullh;
  BOOL fullscreen;
  BOOL audioavail;  /* Is audio available ? */
  BOOL audio;       /* Is audio playing ? */
  BOOL audioen;     /* _Should_ audio be playing ? */
  void *val;
  u16 menuh;
  SDL_Joystick *joy;
  SDL_AudioSpec audioSpec;
  int renderframe_cnt;
  int renderframe_amt;
  BOOL speedlimit;
  BOOL bios_en;
  char fname[256];  /* File path+name of the ROM */
  char progpath[256]; /* Path of the emulator */
  BOOL fullscreen_stretch;
  int key_up, key_down, key_left, key_right,
      key_a, key_b, key_start;
  BOOL jap;
} EMU;

#ifdef __cplusplus
extern "C" {
#endif

EMU *EMU_Create(char *, char **);
BOOL EMU_LoadROM(EMU *, const char *);
void  EMU_Terminate(EMU *);
int EMU_RunFrame(EMU *);
BOOL EMU_AudioEnable(EMU *);
BOOL EMU_AudioDisable(EMU *);
void EMU_SetFrameSkip(EMU *, int);
void EMU_FullStretchEnable(EMU *, BOOL);
void EMU_SpeedLimitEnable(EMU *, BOOL);
void EMU_EnableBIOS(EMU *, BOOL);
void EMU_JoystickEnable(EMU *, BOOL);
void EMU_FullscreenEnable(EMU *, BOOL);

#ifdef __cplusplus
}
#endif

#endif
