/* GGMS.c */
#ifdef DREAMCAST
#include <kos.h>
#endif

#ifdef GP32
#include <gpstdio.h>
#endif

#include <stdio.h>
/*#include <sys/stat.h>*/
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "GGMS.h"
#include "sn76489/sn76489.h"
#include "z80/z80.h"
#include "smsbios.h"
#include "unzip.h"

#ifdef GP32
#include "gpstdlib.h"
#endif

extern int debug;

/*int stricmp(char *, char *);*/
static int stri_cmp(const char *a, const char *b) {

	size_t l, i;

	if ((l = strlen(a)) != strlen(b)) {

		return -1;
	}

	for (i = 0; i < l; i++) {
		
		if (tolower(a[i]) != tolower(b[i])) {

			return -1;
		}
	}
	
	return 0;
}

int GGMS_Addy2ROM(GGMS *machine, u16 a) {

  u32 i = ((((int)machine->banks[((a >> 14) + 1) & 3]) & ((machine->romsize >> 14) - 1)) << 14) + ((int)a & 16383);

  if ((i >= machine->romsize) && ((machine->memcontrol & 0x08) || machine->vdp->gg)) {

    return 0;
  }

  return i;
}

u8 *GGMS_GetReadPage(GGMS *machine, int page) {

	u8 *addr;
	unsigned int a = page << 10;

	if (a < 0x0400) {

		if ((machine->memcontrol & 0x08) || machine->vdp->gg) {

			if (a >= machine->romsize) {

				addr = machine->rom;
			} else {

				addr = &machine->rom[a];
			}
		} else {

			addr = &smsbios[a];
		}
	} else
	if (a < 0xc000) {

    if (((int)(machine->banks[0] & 0x08) != 0) && (a >= 0x8000)) {

      addr = &machine->sram[(a & 0x1fff) | ((u16)machine->banks[0] & 0x04) << 12];
    } else {

			u8 *data = ((machine->memcontrol & 0x40) && !machine->vdp->gg) ? smsbios : machine->rom;
      addr = &data[GGMS_Addy2ROM(machine, a)];
    }
  } else {

    addr = &machine->ram[a & 0x1fff];
  }

  return addr;
}

void GGMS_UpdatePage(GGMS *machine, int page) {

	if (page == 0) {

		machine->pages[0] = machine->rom;
	} else {

		machine->pages[page] = GGMS_GetReadPage(machine, page);
	}
}

void GGMS_UpdateAllPages(GGMS *machine) {

	int i;

	for (i = 0; i < 64; i++) {

		GGMS_UpdatePage(machine, i);
	}
}

u8 Z80_In(Z80 *cpu, u8 loc) {

  u16 temp;
  GGMS *machine = (GGMS*)cpu->val;

  switch (loc) {

    /* VDP Data Port */
    case 0xbe:
      return GGVDP_ReadData(machine->vdp);
      break;

    case 0xbd:
    case 0xbf: {
      u8 i = (((u8)machine->vdp->frame_irq & 1) << 7) |
             (((u8)machine->vdp->line_irq  & 1) << 6) |
             (((u8)machine->vdp->collision & 1) << 5) /*| 0x1f*/;

      machine->vdp->frame_irq = 0;
      machine->vdp->line_irq = 0;
      machine->vdp->collision = 0;

      if (machine->vdp->irq) {

        machine->vdp->irq = 0;
        machine->cpu->IFF &= ~0x10;
      }
      return i;
      break;
    }

    case 0x05:
      return (u8)0x04;
      break;

    /* CURLINE */
    case 0x7e:
    case 0x7f:
      temp = machine->vdp->vcount;
      if (temp > 0xda) {
        
        return temp - 6;
      } else {

        return temp;
      }
      break;

    /* Joypad 1 */
    case 0xdc:
    case 0xc0:
      return (u8)(0xff & ((machine->p1_keys[0] << 2) | ~0x04) &
                         ((machine->p1_keys[1] << 3) | ~0x08) &
                         ((machine->p1_keys[2] << 0) | ~0x01) &
                         ((machine->p1_keys[3] << 1) | ~0x02) &
                         ((machine->p1_keys[4] << 4) | ~0x10) &
                         ((machine->p1_keys[5] << 5) | ~0x20));
      break;

    /* Joypad 2 */
    case 0xdd:
    case 0xc1:
      return machine->countrydetect | 0x1f;
      break;

    /* GG Start button, localization */
    case 0x00:
      if (machine->vdp->gg == TRUE) {

        return (((machine->p1_keys[6] & 1) << 7) |
                ((machine->jap==TRUE) ? 0x00 : 0x40));
      } else {

        return (u8)0xff;
      }
      break;  

    default:
      return (u8)0xff;
      break;
  }
}

void Z80_Out(Z80 *cpu, u8 loc, u8 d) {

  GGMS *machine = (GGMS*)cpu->val;

  switch(loc) {
	  
    case 0xbd:
    case 0xbf:
      GGVDP_WriteControl(machine->vdp, d);
      break;

    case 0xbe:
      GGVDP_WriteData(machine->vdp, d);
      break;

    /* Sound */
    case 0x7e:
    case 0x7f:
      SN76489_Write(machine->snd, d);
      break;

    case 0x3e:  /* Memory control */
      if (machine->bios_en) {
	      
	      machine->memcontrol = d;

        if (!machine->vdp->gg) {

          machine->banks = (d & 0x08) ? machine->banks_rom : machine->banks_bios;
        }
      }
      break;

    case 0x3f:
      if ((d&0x0f) == 0x05) {

        machine->countrydetect = (d & 0x80) | ((d & 0x20) << 1);
      }

      if (machine->jap == TRUE) {

        machine->countrydetect ^= 0xc0;
      }
      break;
  }
}



u8 Z80_ReadMem(Z80 *cpu, u16 loc) {
	
  GGMS *machine = (GGMS*)cpu->val;

  return machine->pages[loc >> 10][loc & 0x3ff];
}

void Z80_WriteMem(Z80 *cpu, u16 loc, u8 d) {

  u16 a = (u16)loc & 0xffff;
  GGMS *machine = (GGMS*)cpu->val;

  if (a >= 0xc000) { /* WRAM */

    machine->ram[a & 0x1fff] = d;

	  if (a >= 0xfffc) {

		  int i;
		  int p = a & 0x03;

			d &= 0x1f;
/*			machine->banks[p] = d;
			GGMS_UpdateAllPages(machine);*/
			if (machine->banks[p] != d) {

			  machine->banks[p] = d;

			  if (p >= 1) {

  			  for (i = (p - 1) * 16; i < ((p - 1) * 16) + 16; i++) {

  			    GGMS_UpdatePage(machine, i);
			    }
		    } else {

          for (i = 32; i < 48; i++) {

            GGMS_UpdatePage(machine, i);
          }
        }
  	  }
    } else
    if (a >= 0x8000) { /* SRAM */

      if ((int)(machine->banks[0] & 0x08) != 0) {

        machine->sram[(a & 0x1fff) | ((u16)machine->banks[0] & 0x04) << 12] = d;
        machine->sram_changed = TRUE;
      }
    }
/*if ((a>=0xc000)&&(a<0xc040) && debug)printf("*\n"); */
  }
}




/*
static int fsize ( char * lpName )
  {
    struct stat mystat;
    if ( stat ( lpName, &mystat ) == -1 ) return -1;
    return ( (int)mystat.st_size );
  }
*/




#ifdef DREAMCAST
static int fsize(char *fname) {

	file_t f;
	int s;
		
	if (!(f = fs_open(fname, O_RDONLY))) {

		return -1;
	}
		
	fs_seek(f, 0, SEEK_END);
	s = fs_tell(f);
	fs_close(f);
		
	return s;
}
#else
static int fsize(char *fname) {

  FILE *f;
  int s;

  if (!(f = fopen(fname,"rb"))) {

    return -1;
  }

  fseek(f, 0, SEEK_END);
  s = ftell(f);
  fclose(f);

  return s;
}
#endif


void GGMS_Terminate(GGMS *machine, BOOL term) {

  if (!machine) {

    return;
  }

  if (machine->snd) {

		if (term) {

    	SN76489_Terminate(machine->snd);
  	}

    free(machine->snd);
    machine->snd = NULL;
  }

  if (machine->vdp) {
/*        if ( machine->vdp->display ) */
/*          free ( machine->vdp->display ); */
		if (term) {

    	GGVDP_Terminate(machine->vdp);
  	}

    free(machine->vdp);
    machine->vdp = NULL;
  }

  if (machine->cpu) {

		if (term) {

    	Z80_Terminate(machine->cpu);
  	}

    free(machine->cpu);
    machine->cpu = NULL;
  }

  if (machine->ram) {

    free(machine->ram);
    machine->ram = NULL;
  }

  if (machine->rom) {

    free(machine->rom);
    machine->rom = NULL;
  }

  if (machine->sram) {

    free(machine->sram);
    machine->sram = NULL;
  }
}



/*
  Return values:
    1 - Couldn't open file
    2 - ZIP error
    3 - No SMS/GG ROM found
    0 - OK
*/

#ifdef ZIP_SUPPORT
int GGMS_LoadCompressedROM(char *fname, void **mem, int *size, char *inZipName) {

  unsigned int i;
  int retval = 3;
  unzFile uf = NULL;
  unz_global_info gi;
  unz_file_info file_info;

  if (!(uf = unzOpen(fname))) {

    return 1;
  }

  if (unzGetGlobalInfo( uf, &gi) != UNZ_OK) {

    unzClose(uf);
    return 2;
  }

  for (i = 0; i < gi.number_entry; i++) {

    char fname[256];

    if (unzGetCurrentFileInfo(uf, &file_info, fname, sizeof(fname),
                              NULL, 0, NULL, 0) != UNZ_OK) {

      unzClose(uf);
      return 2;
    }

    if (inZipName) {

      strcpy(inZipName, fname);
    }

    if (strlen(fname) > 3) {

      if ((stri_cmp(&fname[strlen(fname) - 3], ".gg")  == 0) ||
          (stri_cmp(&fname[strlen(fname) - 4], ".sms") == 0)) {

        /* File is a SMS/GG image */
        int i = 0, err;

        if (size) {

          *size = file_info.uncompressed_size;
        }

        if (unzOpenCurrentFile(uf) != UNZ_OK) {

          unzClose (uf);
          return 2;
        }

        *mem = malloc(file_info.uncompressed_size < 512 ?
                      512 : file_info.uncompressed_size);
        retval = 0;

        /* Handle ROM Header */
        if (((*size - 512) & 32767) == 0) {

          *size -= 512;
          unzReadCurrentFile(uf, *mem, 512);
        }

        do {

          err = unzReadCurrentFile(uf, *mem + i, 16384);

          if (err < 0) {

            break;
          }

          if (err > 0) {

            i += err;
          }
        } while (err > 0);

        unzCloseCurrentFile(uf);
        unzClose (uf);
        return 0;
      }
    }

    if (i != gi.number_entry - 1) {

      if (unzGoToNextFile(uf) != UNZ_OK) {

        printf("Error in zipfile\n");
        break;
      }
    }
  }

  unzClose (uf);
  return retval;
}
#endif

int GGMS_Init(GGMS *machine, char *fname,
              void (*preRender)(GGVDP*),
              void (*postRender)(GGVDP*),
              void (*copyDisplay)(GGVDP*)) {

  int i;
  BOOL isCompressed = FALSE;

  if (!machine) {

    return 4;
  }

  memset(machine, 0, sizeof(GGMS));

  /* Attempt to load the ROM */
  machine->romsize = fsize (fname);

  if (machine->romsize == (unsigned int)-1)
    return 2;

/*    vmem = (u8*)malloc(320 * 200); */
/*    if (!vmem) { free(vmem); return 1; } */
  machine->ram  = (u8*)calloc(16384, 1);
  machine->sram = (u8*)calloc(32768, 1);
  machine->rom = (u8*)calloc(machine->romsize, 1);
  machine->cpu = (Z80*)calloc(sizeof(Z80), 1);
  machine->vdp = (GGVDP*)calloc(sizeof(GGVDP), 1);
  machine->snd = (SN76489*)calloc(sizeof(SN76489), 1);

  if (!(machine->ram && machine->rom &&
        machine->cpu && machine->vdp && machine->sram)) {

    GGMS_Terminate(machine, FALSE);
    return 1;
  }

  memset(machine->sram, 0, 32768);

  for (i = 0; i < 8; i++) {

    machine->p1_keys[i] = 1;
    machine->p2_keys[i] = 1;
  }

  machine->banks_rom[0] = 0x00;
  machine->banks_rom[1] = 0x00;
  machine->banks_rom[2] = 0x01;
  machine->banks_rom[3] = 0x00;
  machine->banks_bios[0] = 0x00;
  machine->banks_bios[1] = 0x00;
  machine->banks_bios[2] = 0x01;
  machine->banks_bios[3] = 0x00;
  machine->banks = machine->banks_bios;
  machine->jap = TRUE;

#ifdef ZIP_SUPPORT
  if (strlen(fname) >= 4) {

    if (stri_cmp(&fname[strlen(fname) - 4], ".zip") == 0) {

      int err;
      char inZipName[256];

      inZipName[0] = '\0';
      isCompressed = TRUE;

      err = GGMS_LoadCompressedROM(fname,
      														 (void*)&(machine->rom),
      														 &(machine->romsize),
      														 inZipName);

      if (err!=0) {

        GGMS_Terminate(machine, FALSE);
        return 2;
      }

      strcpy(fname, inZipName);
    }
  }
#endif

  if (!isCompressed) {

#ifdef DREAMCAST
	  file_t fhandle;

  	if (!(fhandle = fs_open(fname, O_RDONLY))) {

			GGMS_Terminate(machine, FALSE);
			return 2;
		}

		/* ROM header */
		if (((machine->romsize - 512) & 32767) == 0) {

			fs_seek(fhandle, 512, SEEK_SET);
			machine->romsize -= 512;
		}

		fs_read(fhandle, machine->rom, machine->romsize);
		fs_close(fhandle);
#else
	  FILE *fhandle;

    if (!(fhandle = fopen(fname, "rb"))) {

      GGMS_Terminate(machine, FALSE);
      return 2;
    }

    /* Handle ROM Header */
    if (((machine->romsize - 512) & 32767) == 0) {

      fseek (fhandle, 512, SEEK_SET);
      machine->romsize -= 512;
    }

    fread(machine->rom, machine->romsize, 1, fhandle);
    fclose(fhandle);
#endif
  }

  machine->cpu->val = (void*)machine;
  Z80_Init( machine->cpu, Z80_ReadMem, Z80_WriteMem, Z80_In, Z80_Out);
  GGVDP_Init(machine->vdp, NULL, preRender, postRender, copyDisplay);
  SN76489_Init(machine->snd, 22050, 16, 1, 256);

  /* GG or MS mode ? */
  GGVDP_SetGG(machine->vdp, FALSE);
  if (strlen(fname) >= 4) {

    GGVDP_SetGG(machine->vdp,
                (stri_cmp(&fname[strlen(fname) - 3],".gg")  == 0) ||
                (stri_cmp(&fname[strlen(fname) - 4],".ggg") == 0));
  }

  if (machine->vdp->gg) {

    machine->banks = machine->banks_rom;
  }

  machine->memcontrol = 0x40;

  machine->bios_en = TRUE;
  machine->sram_changed = FALSE;

  machine->vdp->val = (void*)machine;

  GGMS_UpdateAllPages(machine);

	if (machine->vdp->gg) {

		machine->gameName = GG_GetGame(machine->rom, machine->romsize);
	} else {

		machine->gameName = SMS_GetGame(machine->rom, machine->romsize);
	}

	machine->crc32 = calccrc32(machine->rom, machine->romsize, ~0);

  return 0;
}

void GGMS_Reset(GGMS *machine) {

  int i;

  for (i = 0; i < 8; i++) {

    machine->p1_keys[i] = 1;
    machine->p2_keys[i] = 1;
  }

  machine->banks[0] = 0x00;
  machine->banks[1] = 0x00;
  machine->banks[2] = 0x01;
  machine->banks[3] = 0x00;
  machine->jap = FALSE;

  GGVDP_Reset(machine->vdp);
  SN76489_Reset(machine->snd);

  GGMS_UpdateAllPages(machine);

  Z80_Reset(machine->cpu);
}

int GGMS_Run(GGMS *machine) {

	if (machine->pause) {
		
		/* Pause button */
		Z80_Int(machine->cpu, Z80_NMI);
		machine->pause = 0;
	}

  Z80_Run(machine->cpu, /*227*/253);

  if (GGVDP_Cycle(machine->vdp) == 1) {

    Z80_Int(machine->cpu, 0x38);
  }

  return 0;
}

#ifdef GP32
u8 *readFile(const char *fname, int *size) {

	F_HANDLE fh;
	ulong s;
	u8 *d;

	if (GpFileGetSize(fname, &s) != SM_OK) {

		return NULL;
	}

	*size = s;

	if (GpFileOpen(fname, OPEN_R, &fh) != SM_OK) {

		return NULL;
	}

	d = malloc(*size);

	GpFileRead(fh, d, *size, NULL);

	GpFileClose(fh);

	return d;
}

int writeFile(const char *fname, u8 *data, int size) {

	F_HANDLE fh;

	if (GpFileCreate(fname, ALWAYS_CREATE, &fh) != SM_OK) {

		return -1;
	}

	GpFileWrite(fh, data, size);

	GpFileClose(fh);
	
	return 0;
}
#else
u8 *readFile(const char *fname, int *size) {

	FILE *fh;
	u8 *d;

	if (!(fh = fopen(fname, "rb"))) {

		return NULL;
	}

	fseek(fh, 0, SEEK_END);
	*size = ftell(fh);
	fseek(fh, 0, SEEK_SET);
	d = malloc(*size);
	fread(d, *size, 1, fh);
	fclose(fh);

	return d;
}

int writeFile(const char *fname, u8 *data, int size) {

	FILE *fh;

	if (!(fh = fopen(fname, "wb"))) {

		return -1;
	}

	fwrite(data, size, 1, fh);

	fclose(fh);
	
	return 0;
}
#endif

int GGMS_SaveState(GGMS *machine, const char *fname) {

	u8 temp;
	u8 *d = malloc(64 * 1024);
	int s = 0;

	/* GGMS */
	memcpy(&d[s], machine->ram, 16384); s += 16384;
	memcpy(&d[s], &machine->countrydetect, sizeof(machine->countrydetect)); s += sizeof(machine->countrydetect);
	memcpy(&d[s], &machine->memcontrol, sizeof(machine->memcontrol)); s += sizeof(machine->memcontrol);
	memcpy(&d[s], machine->banks_rom, sizeof(machine->banks_rom)); s += sizeof(machine->banks_rom);
  memcpy(&d[s], machine->banks_bios, sizeof(machine->banks_bios)); s += sizeof(machine->banks_bios);
	temp = machine->banks == machine->banks_rom ? 0 : 1;
	memcpy(&d[s], &temp, sizeof(temp)); s += sizeof(temp);

	/* Z80 */
	memcpy(&d[s], &machine->cpu->AF, sizeof(machine->cpu->AF)); s += sizeof(machine->cpu->AF);
	memcpy(&d[s], &machine->cpu->BC, sizeof(machine->cpu->BC)); s += sizeof(machine->cpu->BC);
	memcpy(&d[s], &machine->cpu->DE, sizeof(machine->cpu->DE)); s += sizeof(machine->cpu->DE);
	memcpy(&d[s], &machine->cpu->HL, sizeof(machine->cpu->HL)); s += sizeof(machine->cpu->HL);
	memcpy(&d[s], &machine->cpu->SP, sizeof(machine->cpu->SP)); s += sizeof(machine->cpu->SP);
	memcpy(&d[s], &machine->cpu->PC, sizeof(machine->cpu->PC)); s += sizeof(machine->cpu->PC);
	memcpy(&d[s], &machine->cpu->IX, sizeof(machine->cpu->IX)); s += sizeof(machine->cpu->IX);
	memcpy(&d[s], &machine->cpu->IY, sizeof(machine->cpu->IY)); s += sizeof(machine->cpu->IY);
	memcpy(&d[s], &machine->cpu->af, sizeof(machine->cpu->af)); s += sizeof(machine->cpu->af);
	memcpy(&d[s], &machine->cpu->bc, sizeof(machine->cpu->bc)); s += sizeof(machine->cpu->bc);
	memcpy(&d[s], &machine->cpu->de, sizeof(machine->cpu->de)); s += sizeof(machine->cpu->de);
	memcpy(&d[s], &machine->cpu->hl, sizeof(machine->cpu->hl)); s += sizeof(machine->cpu->hl);
	memcpy(&d[s], &machine->cpu->I, sizeof(machine->cpu->I)); s += sizeof(machine->cpu->I);
	memcpy(&d[s], &machine->cpu->R, sizeof(machine->cpu->R)); s += sizeof(machine->cpu->R);
	memcpy(&d[s], &machine->cpu->IFF, sizeof(machine->cpu->IFF)); s += sizeof(machine->cpu->IFF);
	memcpy(&d[s], &machine->cpu->eicount, sizeof(machine->cpu->eicount)); s += sizeof(machine->cpu->eicount);

	/* VDP */
	memcpy(&d[s], machine->vdp->vram, 16384); s += 16384;
	memcpy(&d[s], &machine->vdp->address, sizeof(machine->vdp->address)); s += sizeof(machine->vdp->address);
	memcpy(&d[s], &machine->vdp->t, sizeof(machine->vdp->t)); s += sizeof(machine->vdp->t);
	memcpy(&d[s], &machine->vdp->hilo, sizeof(machine->vdp->hilo)); s += sizeof(machine->vdp->hilo);
	memcpy(&d[s], &machine->vdp->line_counter, sizeof(machine->vdp->line_counter)); s += sizeof(machine->vdp->line_counter);
	memcpy(&d[s], &machine->vdp->vcount, sizeof(machine->vdp->vcount)); s += sizeof(machine->vdp->vcount);
	memcpy(&d[s], machine->vdp->cram, sizeof(machine->vdp->cram)); s += sizeof(machine->vdp->cram);
	memcpy(&d[s], machine->vdp->regs, sizeof(machine->vdp->regs)); s += sizeof(machine->vdp->regs);
	memcpy(&d[s], &machine->vdp->frame_irq, sizeof(machine->vdp->frame_irq)); s += sizeof(machine->vdp->frame_irq);
	memcpy(&d[s], &machine->vdp->line_irq, sizeof(machine->vdp->line_irq)); s += sizeof(machine->vdp->line_irq);
	memcpy(&d[s], &machine->vdp->irq, sizeof(machine->vdp->irq)); s += sizeof(machine->vdp->irq);
	memcpy(&d[s], &machine->vdp->collision, sizeof(machine->vdp->collision)); s += sizeof(machine->vdp->collision);

	/* SN76489 */
	memcpy(&d[s], machine->snd->chanfreq, sizeof(machine->snd->chanfreq)); s += sizeof(machine->snd->chanfreq);
	memcpy(&d[s], machine->snd->chanvol, sizeof(machine->snd->chanvol)); s += sizeof(machine->snd->chanvol);
	memcpy(&d[s], machine->snd->sqofs, sizeof(machine->snd->sqofs)); s += sizeof(machine->snd->sqofs);
	memcpy(&d[s], machine->snd->sqwidth, sizeof(machine->snd->sqwidth)); s += sizeof(machine->snd->sqwidth);
	memcpy(&d[s], machine->snd->pos, sizeof(machine->snd->pos)); s += sizeof(machine->snd->pos);
	memcpy(&d[s], &machine->snd->lastchan, sizeof(machine->snd->lastchan)); s += sizeof(machine->snd->lastchan);
	memcpy(&d[s], &machine->snd->noiseofs, sizeof(machine->snd->noiseofs)); s += sizeof(machine->snd->noiseofs);
	memcpy(&d[s], &machine->snd->chan3f, sizeof(machine->snd->chan3f)); s += sizeof(machine->snd->chan3f);
	memcpy(&d[s], &machine->snd->FB, sizeof(machine->snd->FB)); s += sizeof(machine->snd->FB);

	if (writeFile(fname, d, s) < 0) {

		free(d);
		return -1;
 	}

	free(d);
	return 0;
}

int GGMS_LoadState(GGMS *machine, const char *fname) {

	u8 *d;
	int s = 0;
	int size;
	u8 temp = 0;

	if (!(d = readFile(fname, &size))) {

		return -1;
	}

	/* GGMS */
	memcpy(machine->ram, &d[s], 16384); s += 16384;
	memcpy(&machine->countrydetect, &d[s], sizeof(machine->countrydetect)); s += sizeof(machine->countrydetect);
	memcpy(&machine->memcontrol, &d[s], sizeof(machine->memcontrol)); s += sizeof(machine->memcontrol);
	memcpy(machine->banks_rom, &d[s], sizeof(machine->banks_rom)); s += sizeof(machine->banks_rom);
	memcpy(machine->banks_bios, &d[s], sizeof(machine->banks_bios)); s += sizeof(machine->banks_bios);
	memcpy(&temp, &d[s], sizeof(temp)); s += sizeof(temp);

	if (!temp) {

		machine->banks = machine->banks_rom;
	} else {

		machine->banks = machine->banks_bios;
	}

	/* Z80 */
	memcpy(&machine->cpu->AF, &d[s], sizeof(machine->cpu->AF)); s += sizeof(machine->cpu->AF);
	memcpy(&machine->cpu->BC, &d[s], sizeof(machine->cpu->BC)); s += sizeof(machine->cpu->BC);
	memcpy(&machine->cpu->DE, &d[s], sizeof(machine->cpu->DE)); s += sizeof(machine->cpu->DE);
	memcpy(&machine->cpu->HL, &d[s], sizeof(machine->cpu->HL)); s += sizeof(machine->cpu->HL);
	memcpy(&machine->cpu->SP, &d[s], sizeof(machine->cpu->SP)); s += sizeof(machine->cpu->SP);
	memcpy(&machine->cpu->PC, &d[s], sizeof(machine->cpu->PC)); s += sizeof(machine->cpu->PC);
	memcpy(&machine->cpu->IX, &d[s], sizeof(machine->cpu->IX)); s += sizeof(machine->cpu->IX);
	memcpy(&machine->cpu->IY, &d[s], sizeof(machine->cpu->IY)); s += sizeof(machine->cpu->IY);
	memcpy(&machine->cpu->af, &d[s], sizeof(machine->cpu->af)); s += sizeof(machine->cpu->af);
	memcpy(&machine->cpu->bc, &d[s], sizeof(machine->cpu->bc)); s += sizeof(machine->cpu->bc);
	memcpy(&machine->cpu->de, &d[s], sizeof(machine->cpu->de)); s += sizeof(machine->cpu->de);
	memcpy(&machine->cpu->hl, &d[s], sizeof(machine->cpu->hl)); s += sizeof(machine->cpu->hl);
	memcpy(&machine->cpu->I, &d[s], sizeof(machine->cpu->I)); s += sizeof(machine->cpu->I);
	memcpy(&machine->cpu->R, &d[s], sizeof(machine->cpu->R)); s += sizeof(machine->cpu->R);
	memcpy(&machine->cpu->IFF, &d[s], sizeof(machine->cpu->IFF)); s += sizeof(machine->cpu->IFF);
	memcpy(&machine->cpu->eicount, &d[s], sizeof(machine->cpu->eicount)); s += sizeof(machine->cpu->eicount);

	/* VDP */
	memcpy(machine->vdp->vram, &d[s], 16384); s += 16384;
	memcpy(&machine->vdp->address, &d[s], sizeof(machine->vdp->address)); s += sizeof(machine->vdp->address);
	memcpy(&machine->vdp->t, &d[s], sizeof(machine->vdp->t)); s += sizeof(machine->vdp->t);
	memcpy(&machine->vdp->hilo, &d[s], sizeof(machine->vdp->hilo)); s += sizeof(machine->vdp->hilo);
	memcpy(&machine->vdp->line_counter, &d[s], sizeof(machine->vdp->line_counter)); s += sizeof(machine->vdp->line_counter);
	memcpy(&machine->vdp->vcount, &d[s], sizeof(machine->vdp->vcount)); s += sizeof(machine->vdp->vcount);
	memcpy(machine->vdp->cram, &d[s], sizeof(machine->vdp->cram)); s += sizeof(machine->vdp->cram);
	memcpy(machine->vdp->regs, &d[s], sizeof(machine->vdp->regs)); s += sizeof(machine->vdp->regs);
	memcpy(&machine->vdp->frame_irq, &d[s], sizeof(machine->vdp->frame_irq)); s += sizeof(machine->vdp->frame_irq);
	memcpy(&machine->vdp->line_irq, &d[s], sizeof(machine->vdp->line_irq)); s += sizeof(machine->vdp->line_irq);
	memcpy(&machine->vdp->irq, &d[s], sizeof(machine->vdp->irq)); s += sizeof(machine->vdp->irq);
	memcpy(&machine->vdp->collision, &d[s], sizeof(machine->vdp->collision)); s += sizeof(machine->vdp->collision);

	/* SN76489 */
	memcpy(machine->snd->chanfreq, &d[s], sizeof(machine->snd->chanfreq)); s += sizeof(machine->snd->chanfreq);
	memcpy(machine->snd->chanvol, &d[s], sizeof(machine->snd->chanvol)); s += sizeof(machine->snd->chanvol);
	memcpy(machine->snd->sqofs, &d[s], sizeof(machine->snd->sqofs)); s += sizeof(machine->snd->sqofs);
	memcpy(machine->snd->sqwidth, &d[s], sizeof(machine->snd->sqwidth)); s += sizeof(machine->snd->sqwidth);
	memcpy(machine->snd->pos, &d[s], sizeof(machine->snd->pos)); s += sizeof(machine->snd->pos);
	memcpy(&machine->snd->lastchan, &d[s], sizeof(machine->snd->lastchan)); s += sizeof(machine->snd->lastchan);
	memcpy(&machine->snd->noiseofs, &d[s], sizeof(machine->snd->noiseofs)); s += sizeof(machine->snd->noiseofs);
	memcpy(&machine->snd->chan3f, &d[s], sizeof(machine->snd->chan3f)); s += sizeof(machine->snd->chan3f);
	memcpy(&machine->snd->FB, &d[s], sizeof(machine->snd->FB)); s += sizeof(machine->snd->FB);

	free(d);

	GGVDP_ForceDirtyTileCache(machine->vdp);
	GGMS_UpdateAllPages(machine);
	machine->vdp->palchange = TRUE;

	return 0;
}


int GGMS_SaveSRAM(GGMS *machine, const char *fname) {

	if (writeFile(fname, machine->sram, 32768) < 0) {

		return -1;
	}

	machine->sram_changed = FALSE;

	return 0;
}

int GGMS_LoadSRAM(GGMS *machine, const char *fname) {

	u8 *sram;
	int size;


	if (!(sram = readFile(fname, &size))) {

		return -1;
	}

	if (size > 32768) {

		return -1;
	}

	memcpy(machine->sram, sram, size);

	free(sram);

	machine->sram_changed = FALSE;

	return 0;
}

