/* GGMS.h */
#ifndef __GGMS_H__
#define __GGMS_H__

#include "Z80/Z80.h"
#include "GGVDP/GGVDP.h"
#include "sn76489/sn76489.h"
#include "crc32.h"
#include "../defines.h"

typedef struct GGMS {

  Z80 *cpu;
  GGVDP *vdp;
  SN76489 *snd;
  u8 *rom;
  u8 *ram;
  u8 *sram;
  unsigned int romsize;
  BOOL jap;      /* TRUE if japanese, FALSE if european/american */
  u8 countrydetect; /* Used as a temporary buffer */
  u8 memcontrol;
  u8 banks_rom[4];
  u8 banks_bios[4];
  u8 *banks;
  u8 p1_keys[8]; /* left, right, up, down, a, b, start (gg only) */
  u8 p2_keys[8];
  void *val;
  BOOL bios_en;
  BOOL sram_changed;
  u8 *pages[64];
	char *gameName;
	u32 crc32;
	int pause;
} GGMS;



void GGMS_Terminate(GGMS *, BOOL);
int GGMS_Init(GGMS *, char *, void (*)(GGVDP*), void (*)(GGVDP*), void (*)(GGVDP*));
int GGMS_Run(GGMS *);
void GGMS_Reset(GGMS *);
int GGMS_Addy2ROM(GGMS *, u16);
void GGMS_UpdatePage(GGMS *, int);
int GGMS_SaveState(GGMS *, const char *);
int GGMS_LoadState(GGMS *, const char *);
int GGMS_SaveSRAM(GGMS *, const char *);
int GGMS_LoadSRAM(GGMS *, const char *);
void GGMS_UpdateAllPages(GGMS *);

#endif

