/* GGVDP.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "..\Z80\Z80.h"
#include "..\GGMS.h"
#include "GGVDP.h"

extern int debug;

void GGVDP_PushTileCache(GGVDP *vdp, u32 tile) {

	if (!vdp->tileCacheDirty[tile]) {

		vdp->tileCacheDirty[tile] = TRUE;
		vdp->tileCacheTop++;
		vdp->tileCacheTiles[vdp->tileCacheTop] = tile;
	}
}

u32 GGVDP_PopTileCache(GGVDP *vdp) {

	int tile;

	if (vdp->tileCacheTop < 0) {

		return 0xffffffff;
	}

	tile = vdp->tileCacheTiles[vdp->tileCacheTop--];
	vdp->tileCacheDirty[tile] = FALSE;

	return tile;
}

void GGVDP_ForceDirtyTileCache(GGVDP *vdp) {

	int i;

  vdp->tileCacheTop = -1;

	for (i = 0; i < GGVDP_NUM_TILES * 8; i++) {

		vdp->tileCacheDirty[i] = FALSE;
  }

	for (i = 0; i < GGVDP_NUM_TILES * 8; i++) {

		GGVDP_PushTileCache(vdp, i);
  }
}


void GGVDP_Reset(GGVDP *vdp) {

  vdp->hilo = 0;
  vdp->t = 0;
  vdp->line_counter = 0;
  vdp->address = 0;
  vdp->vcount = 0;
  vdp->line_irq = 0;
  vdp->frame_irq = 0;
  vdp->irq = 0;
  vdp->regs[0] = 0x00;
  vdp->regs[1] = 0x00;
  vdp->regs[2] = 0x00;
  vdp->regs[3] = 0x00;
  vdp->regs[4] = 0x00;
  vdp->regs[5] = 0x00;  /* Sprite table at 0x3f00 */
  vdp->regs[6] = 0x00;  /* Sprite tiles at 0x2000 */
  vdp->regs[7] = 0x00;
  vdp->regs[8] = 0x00;
  vdp->regs[9] = 0x00;
  vdp->regs[10] = 0x00;
  vdp->regs[11] = 0x00;
  vdp->regs[12] = 0x00;
  vdp->regs[13] = 0x00;
  vdp->regs[14] = 0x00;
  vdp->regs[15] = 0x00;
  vdp->collision = 0;

  vdp->palchange = FALSE;

	GGVDP_ForceDirtyTileCache(vdp);
}

void GGVDP_RenderTileLine(GGVDP *vdp, u32 tile, u32 line) {

	u8 *src;
	int y1 = 8 * line;
	int y2 = 8 * (7 - line);
	int x, nfx;

	src = &vdp->vram[(tile * 4 * 8) + (4 * line)];

	for (x = 0, nfx = 7; x < 8; x++, nfx--) {

		u8 col = (((src[0] >> nfx) & 1) << 0) |
	    			 (((src[1] >> nfx) & 1) << 1) |
	           (((src[2] >> nfx) & 1) << 2) |
	    			 (((src[3] >> nfx) & 1) << 3);


		vdp->tileCache[0][tile][y1 + x] =
		vdp->tileCache[1][tile][y1 + nfx] =
		vdp->tileCache[2][tile][y2 + x] =
		vdp->tileCache[3][tile][y2 + nfx] = col;

		vdp->tileCache[4][tile][y1 + x] =
		vdp->tileCache[5][tile][y1 + nfx] =
		vdp->tileCache[6][tile][y2 + x] =
		vdp->tileCache[7][tile][y2 + nfx] = col | 16;
	}
}

void GGVDP_RenderTile(GGVDP *vdp, u32 tile) {

	u32 line;

	for (line = 0; line < 8; line++) {

		GGVDP_RenderTileLine(vdp, tile, line);
	}
}

void GGVDP_ProcessTileCache(GGVDP *vdp) {

	u32 tile;

	while ((tile = GGVDP_PopTileCache(vdp)) != 0xffffffff) {

		GGVDP_RenderTileLine(vdp, tile >> 3, tile & 7);
	}
}


/*
void GGVDP_RenderTile(GGVDP *vdp, int tile) {

	int y;
	u8 *dest;
	u8 *src;

	if (!vdp)
		return;

	dest = vdp->tileCache[tile];
	src = &vdp->vram[tile * 4 * 8];

	for (y = 0; y < 8; y++) {

		int x;

		for (x = 0; x < 8; x++) {

			int nfx = 7 - x;

			dest[x + (y * 8)] = (((src[(y * 4) + 0] >> nfx) & 1) << 0) |
		                      (((src[(y * 4) + 1] >> nfx) & 1) << 1) |
		                      (((src[(y * 4) + 2] >> nfx) & 1) << 2) |
		                      (((src[(y * 4) + 3] >> nfx) & 1) << 3);
	  }
	}

	vdp->tileCacheValid[tile] = TRUE;
}

void GGVDP_RenderAllTiles(GGVDP *vdp) {
	
	int i;

	if (!vdp)
		return;

	for (i = 0; i < GGVDP_NUM_TILES; i++)
		GGVDP_RenderTile(vdp, i);

	vdp->tileCacheAllInvalid = FALSE;
}
*/
void GGVDP_Init(GGVDP *vdp, u8 *dispbuf,
								void (*preRender)(GGVDP*),
								void (*postRender)(GGVDP*),
								void (*copyDisplay)(GGVDP*)) {

  GGVDP_Reset(vdp);
  vdp->render = TRUE;
  vdp->vram = malloc(16 * 1024);
  vdp->display = /*(u8*) malloc ( 256*192 )*/dispbuf;
  vdp->gg = FALSE;
  vdp->clk = 0;
  vdp->preRender = (void (*)(void*))preRender;
  vdp->postRender = (void (*)(void*))postRender;
  vdp->copyDisplay = (void (*)(void*))copyDisplay;
  vdp->frameskip = 0;
  vdp->framecount = 0;
  vdp->dx = 256;
  vdp->dy = 192;
  vdp->xofs = 0;
  vdp->yofs = 0;
  vdp->screenw = 256;
  vdp->screenh = 192;
  vdp->linedata = NULL;
  vdp->render = TRUE;
}

void GGVDP_SetGG(GGVDP *vdp, BOOL en) {

  vdp->gg = en;
  vdp->screenw = (en ? 160 : 256);
  vdp->screenh = (en ? 144 : 192);
}

void GGVDP_Terminate(GGVDP *vdp) {

    if (vdp->vram) {

	    free(vdp->vram);
	    vdp->vram = NULL;
    }

    if (vdp->linedata) {

	    free(vdp->linedata);
	    vdp->linedata = NULL;
    }
  }

/* Write to the address/control register */
void GGVDP_WriteControl(GGVDP *vdp, u8 d) {

  if (vdp->hilo == 0) {

    vdp->hilo = 1;
    vdp->t = d;
  } else {

    vdp->hilo = 0;

    if ((d & 0xf0) == 0x80) {

      /* Register write */
      vdp->regs[d & 0x0f] = vdp->t;
      vdp->address = 0;
    } else {

      /* VRAM address write */
      vdp->address = vdp->t | (d << 8);
    }
  }
}


void GGVDP_WriteData(GGVDP *vdp, u8 d) {

	/*GGMS * machine = (GGMS*)vdp->val;*/
	u16 t = (vdp->address >> 14) & 3;
	u16 address = vdp->address & 0x3fff;
	u8 *dest;
  /* Reset internal flag */
  vdp->hilo = 0;

  switch (t) {

    case 0x00:
    case 0x01:
    case 0x02:
      /* VRAM */
			dest = &vdp->vram[address];

			if (*dest != d) {

      	*dest = d;

      	if (address < 0x3800)
       		GGVDP_PushTileCache(vdp, address >> 2);
   		}
      break;

    case 0x03:
      vdp->cram[vdp->address & (vdp->gg ? 0x3f : 0x1f)] = d;
      vdp->palchange = TRUE;
      break;
  }

  vdp->address++;
}


u8 GGVDP_ReadData(GGVDP *vdp) {

  u8 d = 0xff;

  /* Reset internal flag */
  vdp->hilo = 0;

  if (vdp->address < 0x8000) {

    /* Video Mem */
    d = vdp->vram[vdp->address & 0x3fff];
  } else
  if ((vdp->address & 0xc000) == 0xc000) {

    /* CRAM */
    d = vdp->cram[vdp->address & 0x003f];
  }

  vdp->address++;
  vdp->address &= 0xffff;

  return d;
}

void GGVDP_RenderBG(GGVDP *vdp, int cl, u8 *oprint, u8 *destline) {

  int xofs, yofs, i;
  int y;
  u8 *dest;
  int t;
  int mapy, mapx;
  int iofs;
  int tiley, tile, tilep;
  int mapsrcofs;

  u8 *map;
  u8 *overprint;
  u8 *tcache;
  u8 *dtemp;

  xofs = -vdp->regs[8];
  yofs =  vdp->regs[9];
  y = cl + yofs;
  mapy = y >> 3;

  if (mapy >= 28) {

  	mapy -= 28;
	}

  /* We are in the active scan period
     Render the current bg line */
  dest = destline;

  tiley = y & 7;

  mapsrcofs = ((((int)vdp->regs[2]) << 10) & 0x3800) + (mapy << 6);

  iofs = ((vdp->regs[0] & 0x40) && (cl < 16) && !vdp->gg ? 0 : xofs) +
               (vdp->gg ? 48 : 0);

	map = &(vdp->vram[mapsrcofs]);

	mapx = (((/*(8 - (iofs & 7)) +*/ iofs) >> 3) & 31) << 1;

	tile = map[mapx];
	tilep = map[mapx + 1];
	tile |= (tilep & 1) << 8;
  tcache = &vdp->tileCache[(tilep >> 1) & 7][tile][tiley << 3];

	dtemp = dest;

	tcache += iofs & 7;

  if (!(vdp->regs[0] & 0x20) || vdp->gg) {

	  for (i = 0; i < 8 - (iofs & 7); i++) {

	    oprint[i] = (tilep & 0x10) << 3;
#ifdef GP32
			*dtemp = *tcache++;
			dtemp += 240;
#else
    	*dtemp++ = *tcache++;
#endif
  	}
  }

  mapx = (mapx + 2) & 63;

	t = vdp->screenw - (iofs & 7);

	i = 8 - (iofs & 7);
	overprint = &oprint[i];
#ifdef GP32
		dtemp = &dest[i * 240];
#else
		dtemp = &dest[i];
#endif

	for (/*i = 8 - (iofs & 7)*/;
	     i < t;
	     i += 8) {

		tile = map[mapx++];
		tilep = map[mapx++];
		mapx &= 63;
		tile |= (tilep & 1) << 8;

		tcache = &vdp->tileCache[(tilep >> 1) & 7][tile][tiley << 3];

		overprint[0] =
		overprint[1] =
		overprint[2] =
		overprint[3] =
		overprint[4] =
		overprint[5] =
		overprint[6] =
		overprint[7] = (tilep & 0x10) << 3;
		overprint += 8;

#ifdef GP32
		*dtemp = *tcache++; dtemp += 240;
		*dtemp = *tcache++; dtemp += 240;
		*dtemp = *tcache++; dtemp += 240;
		*dtemp = *tcache++; dtemp += 240;
		*dtemp = *tcache++; dtemp += 240;
		*dtemp = *tcache++; dtemp += 240;
		*dtemp = *tcache++; dtemp += 240;
		*dtemp = *tcache++; dtemp += 240;
#else
		*dtemp++ = *tcache++;
		*dtemp++ = *tcache++;
		*dtemp++ = *tcache++;
		*dtemp++ = *tcache++;
		*dtemp++ = *tcache++;
		*dtemp++ = *tcache++;
		*dtemp++ = *tcache++;
		*dtemp++ = *tcache++;
#endif

	}

	tile = map[mapx];
	tilep = map[mapx + 1];
	tile |= (tilep & 1) << 8;
  tcache = &vdp->tileCache[(tilep >> 1) & 7][tile][tiley << 3];

#ifdef GP32
	dtemp = &dest[(vdp->screenw - (iofs & 7)) * 240];
#else
	dtemp = &dest[vdp->screenw - (iofs & 7)];
#endif

  for (i = vdp->screenw - (iofs & 7); i < vdp->screenw; i++) {

    oprint[i] = (tilep & 0x10) << 3;
#ifdef GP32
		*dtemp = *tcache++; dtemp += 240;
#else
    *dtemp++ = *tcache++;
#endif
  }

  if ((vdp->regs[0] & 0x20) && !vdp->gg) {

    dtemp = dest;

		oprint[0] =
		oprint[1] =
		oprint[2] =
		oprint[3] =
		oprint[4] =
		oprint[5] =
		oprint[6] =
		oprint[7] = 0x8f;

#ifdef GP32
    *dtemp = 0; dtemp += 240;
    *dtemp = 0; dtemp += 240;
    *dtemp = 0; dtemp += 240;
    *dtemp = 0; dtemp += 240;
    *dtemp = 0; dtemp += 240;
    *dtemp = 0; dtemp += 240;
    *dtemp = 0; dtemp += 240;
    *dtemp = 0; dtemp += 240;
#else
    *dtemp++ = 0;
    *dtemp++ = 0;
    *dtemp++ = 0;
    *dtemp++ = 0;
    *dtemp++ = 0;
    *dtemp++ = 0;
    *dtemp++ = 0;
    *dtemp++ = 0;
#endif
  }
}




/* Render the current raster line */
void GGVDP_RenderLine(GGVDP *vdp) {

  int curLine, i,j;
  u8 *src;
  u8 *dest;
  u8 overprint[256];
  u8 *oprint;
  u8 *destline;
  u8 collision[256];

	/* Get the sprite table */
	u8 *sprites = &vdp->vram[(vdp->regs[5] << 7) & 0x3f00];
	/* Stack with all sprites which are visible in the current rasterline */
  int spriteBuf[8];
  int spriteBufTop;

	/* Refresh the tile cache */
  GGVDP_ProcessTileCache(vdp);

	/* Get the current line */
  curLine = vdp->vcount;

  vdp->preRender(vdp);

	/* Get a pointer to the memory where we are supposed to render the data */
#ifdef GP32
	destline = &vdp->display[(239 - (curLine - (vdp->gg ? 24 : 0)) - vdp->yofs) + (240 * vdp->xofs)];
#else
	destline = &vdp->display[((curLine + vdp->yofs) * vdp->dx) + vdp->xofs];
#endif

  GGVDP_RenderBG(vdp, curLine, overprint, destline);


	/* Get all visible sprites at the current rasterline */
	for (spriteBufTop = -1, i = 0; i < 64; i++) {

    int y = sprites[i];
    int dy = vdp->regs[1] & 2 ? 16 : 8;

    if (y == 208) {

      break;
    }
    else
    if (y >= 240) {

      y -= 256;
    }

	  if ((curLine >= y) && (curLine < (y + dy))) {

		  spriteBufTop++;
		  spriteBuf[spriteBufTop] = i;
			if (spriteBufTop == 7) {

				break;
			}
    }
  }

	/* Render the sprites */
	while (spriteBufTop >= 0) {

		/* Current sprite number */
    int n = spriteBuf[spriteBufTop--];

		/* y position */
    int y = sprites[n]  - (vdp->regs[0] & 8);

		/* Height */
    int dy = vdp->regs[1] & 0x02 ? 16 : 8;

		/* x position */
    int x = sprites[128 + (n << 1)];

		/* Tile number */
    int tile = (sprites[128 + (n << 1) + 1] | ((vdp->regs[6] & 4) << 6)) &
                /* Ignore bit 0 for 8x16 sprites */
                (dy == 8 ? ~0 : ~1);

		/* Start and end pixel for the line */
    int spx1 = 0, spx2 = 8;

    int spriteY;

    /* Wrap y position */
    if (y >= 240) y -= 256;

    /* Get y pos within the sprite */
    spriteY = curLine - y;

		if (vdp->gg) x -= 48;

    /* 8x16 sprites */
    if (spriteY >= 8) {

	    spriteY -= 8;
	    tile++;
	  }

	  if (!((curLine >= y) && (curLine < (y + dy)))) {
  	  
  	  continue;
	  }

    /* Get source tile data */
		src = &vdp->tileCache[4][tile][spriteY << 3];

		if (x + 8 >= vdp->screenw) {

			spx2 = vdp->screenw - x;
		}

		if (x < 0) {

			spx1 = -x;
			x = 0;
  	}

#ifdef GP32
		dest = &destline[x * 240];
#else
		dest = &destline[x];
#endif

		oprint = &overprint[x + spx1];

    for (j = spx1; j < spx2; j++) {

      u8 u;

			u = 0;

     	if (!((*oprint & 0x80) && (*dest & 0xf))) {

				u8 col = src[j] | 16;

	      if (col != 16) {

	 				*dest = col;
        	u = 1;
        }
      }
#ifdef GP32
  		dest += 240;
#else
		 	dest++;
#endif

      oprint++;

      if (collision[j + x] == 1) {

        vdp->collision = 1;
      }

      collision[j + x] = u;
    }
  }

  vdp->postRender(vdp);
}


/* Returns 1 if an INT should be generated */
int GGVDP_Cycle(GGVDP *vdp) {

  int set_irq = 0;
  int y1 = (vdp->gg ? 24 : 0),
      y2 = (vdp->gg ? 168 : 192);

  /* Render a line every 212 clocks */
  vdp->clk = 0;

  if (vdp->vcount == 0) {

/*        if ( (vdp->framecount==0) || (vdp->frameskip==0) ) */
    if (vdp->render) {

      vdp->copyDisplay(vdp);
    }
  }

  if (vdp->vcount <= 192) {

    if (vdp->vcount == 192) {

      vdp->frame_irq = 1;
    }

    if (vdp->vcount == 0) {

      vdp->line_counter = vdp->regs[10];
    }

    if (vdp->line_counter == 0) {

      vdp->line_counter = vdp->regs[10];
      vdp->line_irq = 1;
      if (vdp->regs[0] & 0x10) {

        vdp->irq = 1;
      }
    } else {

    	vdp->line_counter--;
  	}

    if ((vdp->regs[0] & 0x10) && (vdp->line_irq == 1)) {

      vdp->irq = 1;
      set_irq = 1;
    }
  } else {

    if ((vdp->vcount == 193) && (vdp->regs[1] & 0x20) && (vdp->frame_irq == 1)) {

      set_irq = 1;
      vdp->irq = 1;
    }

    vdp->line_counter = vdp->regs[10];
  }

  if ((vdp->vcount >= y1) && (vdp->vcount < y2)) {

    /* Render current line */
    if ((vdp->framecount == 0) && (vdp->regs[1] & 0x40) && vdp->render) {

      GGVDP_RenderLine (vdp);
    }
  }

  /* Go to next line */
  vdp->vcount++;

  if (vdp->vcount > 261) {

    if (vdp->frameskip > 0) {

      if (vdp->framecount == 0) {

        vdp->framecount = vdp->frameskip;
      } else {

        vdp->framecount--;
      }
    }

    vdp->vcount = 0;
  }

  return set_irq;
}
