/* GGVDP.h */

#ifndef __GGVDP_H__
#define __GGVDP_H__

#include "../defines.h"

#define GGVDP_NUM_TILES 512

#define GGVDP_MODE_1_1     0	/* 1:1 */
#define GGVDP_MODE_15_15   1	/* 1.5:1.5 */
#define GGVDP_MODE_125_125 2	/* 1.25:1.25 */

typedef struct GGVDP {

  u16 address;  /* Current address counter */
  u8  t;    /* Temporary byte for selecting */
    /* internal register or address */
  u8  hilo;   /* 0 if the next write to the */
    /* address port will be the lower byte */
  u8 *vram;    /* 16kbyte of VRAM */
  u8 line_counter;  /* Counter for line interrupts */
  u16 vcount;   /* The line currently being rendered */
  u16 clk;    /* Cycle count */
  u8 clatch;    /* Color data latch (GG only) */
  u8 cram[64];  /* 64 bytes of CRAM (32 byte on SMS) */
  u8 regs[16];  /* 16 registers (0x80..0x8f) */
  BOOL gg;    /* TRUE if in GG mode */
  int frame_irq;  /* Frame Interrupt pending */
  int line_irq; /* Line Interrupt pending */
  int irq;    /* Interrupt pending (no INT acknowledge from Z80 yet) */
  int frameskip;  /* Display every nth frame */
  int framecount; /* Counts from frameskip...0 */
  BOOL palchange; /* Gets TRUE when the palette has been changed */

  u16 screenw;      /* Screen width of the MS/GG */
  u16 screenh;      /* Screen height of the MS/GG */

  u8 *display; /* Display buffer */
  int dx,dy;    /* Width, height of display buffer */
  int xofs,yofs;  /* Upper left corner of the frames in display buffer */

  void (*preRender)(void*);
  void (*postRender)(void*);
  void (*copyDisplay)(void*);
  void *val;       /* User data */
  BOOL render;      /* TRUE if the screen should be rendered */
  u8 *linedata;    /* Pointer to the gfx data of one rendered line (incl. zoom) */
  int collision;

  BOOL tileCacheDirty[GGVDP_NUM_TILES * 8]; /* Indicates whether each tile is still valid */
  u32 tileCacheTiles[(GGVDP_NUM_TILES * 8) + 1];
  int tileCacheTop;
  u8 tileCache[8][GGVDP_NUM_TILES][8 * 8];
  u8 *bgCache[32 * 28]; /* Pointers to tiles in the tileCache */
	int vidmode;
} GGVDP;


void GGVDP_Init(GGVDP *, u8 *, void (*preRender)(GGVDP*), void (*postRender)(GGVDP*), void (*copyDisplay)(GGVDP*));
void GGVDP_Terminate(GGVDP *);
void GGVDP_WriteControl(GGVDP *, u8);
void GGVDP_WriteData(GGVDP *, u8);
u8 GGVDP_ReadData(GGVDP *);
void GGVDP_RenderLine(GGVDP *);
int GGVDP_Cycle(GGVDP *);
void GGVDP_Reset(GGVDP *);
void GGVDP_SetGG(GGVDP *, BOOL);
void GGVDP_RenderTile(GGVDP *, u32);
void GGVDP_RenderAllTiles(GGVDP *);
void GGVDP_ForceDirtyTileCache(GGVDP *);

#endif

