/* Z80.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Z80.h"
#include "Z80_macros.h"
#include "Z80_Disasm.h"
#include "../GGMS.h"
/* #include "Z80_instructions.h" */

extern const u8 DAATable[];
extern int debug;

#ifdef GP32
#include "gpfont.h"
#include "gpgraphic.h"
#include "gpstdlib.h"

extern GPDRAWSURFACE gpDraw[2];
#endif

int Z80_IllegalInstruction ( Z80 * cpu )
  {
/*     printf ("Illegal opcode at %08x\n",GGMS_Addy2ROM((GGMS*)cpu->val, cpu->PC.aword)); */
    cpu->PC.aword++;
    return 4;
  }

void Z80_Reset ( Z80 * cpu )
  {
    cpu->I = 0;
    cpu->R = 0;
    cpu->AF.aword = 0;
    cpu->af.aword = 0;
    cpu->BC.aword = 0;
    cpu->bc.aword = 0;
    cpu->DE.aword = 0;
    cpu->de.aword = 0;
    cpu->HL.aword = 0;
    cpu->hl.aword = 0;
    cpu->SP.aword = 0;
    cpu->PC.aword = 0;
    cpu->IX.aword = 0;
    cpu->IY.aword = 0;
    cpu->IFF = 0;
    cpu->cyclecount = 0;
    cpu->eicount = -1;
  }

void Z80_Init ( Z80 * cpu, u8 (*readMem)(Z80*,u16), void (*writeMem)(Z80*,u16,u8), u8 (*in)(Z80*,u8), void (*out)(Z80*,u8,u8) )
  {
/*     int i,j; */

    /*  Initialize registers */
    Z80_Reset ( cpu );
    cpu->readMem  = (u8  (*)(void*,u16))    readMem;
    cpu->writeMem = (void(*)(void*,u16,u8)) writeMem;
    cpu->in       = (u8  (*)(void*,u8))     in;
    cpu->out      = (void(*)(void*,u8,u8))  out;
    cpu->curInst = NULL;
    cpu->rdy = FALSE;
    cpu->vec = 0;
  }

void Z80_Terminate ( Z80 * cpu )
  {
    Z80_Reset ( cpu );
  }

/*  Decode instructions with the DDCB prefix */
int Z80_Decode_DDCB ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,/*tt,*/t2;
    union_word  dw;

    cpu->data[3] = read ( cpu->PC.aword+3 );

    switch (cpu->data[3])
      {

#define XX   IX
#define CYC  23
#define ILEN 4
#define OFS  BYTE2WORD(read(cpu->PC.aword+2))
#include "xx.h"
#undef XX
#undef CYC
#undef ILEN
#undef OFS

        default:
          return Z80_IllegalInstruction(cpu);
          break;
      }
  }


/*  Decode instructions with the FDCB prefix */
int Z80_Decode_FDCB ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,/*tt,*/t2;
    union_word  dw;

    cpu->data[3] = read ( cpu->PC.aword+3 );

    switch (cpu->data[3])
      {

#define XX   IY
#define CYC  23
#define ILEN 4
#define OFS  BYTE2WORD(read(cpu->PC.aword+2))
#include "xx.h"
#undef XX
#undef CYC
#undef ILEN
#undef OFS

        default:
          return Z80_IllegalInstruction(cpu);
          break;
      }
  }


/*  Decode instructions with the ED prefix */
int Z80_Decode_ED ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,tt;
    union_word  dw;

    cpu->data[1] = read ( cpu->PC.aword+1 );

    switch (cpu->data[1])
      {
        case 0x43:            /*  LD (nn),BC */
          dw.abyte.low  = read(cpu->PC.aword+2);
          dw.abyte.high = read(cpu->PC.aword+3);
          write(dw.aword+0, cpu->BC.abyte.low);
          write(dw.aword+1, cpu->BC.abyte.high);
          cpu->PC.aword+=4;return 20;break;
        case 0x53:            /*  LD (nn),DE */
          dw.abyte.low  = read(cpu->PC.aword+2);
          dw.abyte.high = read(cpu->PC.aword+3);
          write(dw.aword+0, cpu->DE.abyte.low);
          write(dw.aword+1, cpu->DE.abyte.high);
          cpu->PC.aword+=4;return 20;break;
        case 0x63:            /*  LD (nn),HL */
          dw.abyte.low  = read(cpu->PC.aword+2);
          dw.abyte.high = read(cpu->PC.aword+3);
          write(dw.aword+0, cpu->HL.abyte.low);
          write(dw.aword+1, cpu->HL.abyte.high);
          cpu->PC.aword+=4;return 20;break;
        case 0x73:            /*  LD (nn),SP */
          dw.abyte.low  = read(cpu->PC.aword+2);
          dw.abyte.high = read(cpu->PC.aword+3);
          write(dw.aword+0, cpu->SP.abyte.low);
          write(dw.aword+1, cpu->SP.abyte.high);
          cpu->PC.aword+=4;return 20;break;

        case 0x4b:            /*  LD BC,(nn) */
          dw.abyte.low  = read(cpu->PC.aword+2);
          dw.abyte.high = read(cpu->PC.aword+3);
          cpu->BC.abyte.low  = read(dw.aword+0);
          cpu->BC.abyte.high = read(dw.aword+1);
          cpu->PC.aword+=4;return 20;break;
        case 0x5b:            /*  LD DE,(nn) */
          dw.abyte.low  = read(cpu->PC.aword+2);
          dw.abyte.high = read(cpu->PC.aword+3);
          cpu->DE.abyte.low  = read(dw.aword+0);
          cpu->DE.abyte.high = read(dw.aword+1);
          cpu->PC.aword+=4;return 20;break;
        case 0x6b:            /*  LD HL,(nn) */
          dw.abyte.low  = read(cpu->PC.aword+2);
          dw.abyte.high = read(cpu->PC.aword+3);
          cpu->HL.abyte.low  = read(dw.aword+0);
          cpu->HL.abyte.high = read(dw.aword+1);
          cpu->PC.aword+=4;return 20;break;
        case 0x7b:            /*  LD SP,(nn) */
          dw.abyte.low  = read(cpu->PC.aword+2);
          dw.abyte.high = read(cpu->PC.aword+3);
          cpu->SP.abyte.low  = read(dw.aword+0);
          cpu->SP.abyte.high = read(dw.aword+1);
          cpu->PC.aword+=4;return 20;break;

        case 0x47:  cpu->I = cpu->AF.abyte.high;cpu->PC.aword+=2;return 9;break;  /*  LD I,A */
        case 0x4f:  cpu->R = cpu->AF.abyte.high;cpu->PC.aword+=2;return 9;break;  /*  LD R,A */
        case 0x57:            /*  LD A,I */
          cpu->AF.abyte.high = cpu->I;
          cpu->AF.abyte.low  = (cpu->AF.abyte.low & FLAG_C) | SZTable[cpu->AF.abyte.high] | ((cpu->IFF&0x02)<<1);
          cpu->PC.aword+=2;return 9;break;
        case 0x5f:            /*  LD A,R */
          cpu->AF.abyte.high = cpu->R;
          cpu->AF.abyte.low  = (cpu->AF.abyte.low & FLAG_C) | SZTable[cpu->AF.abyte.high] | ((cpu->IFF&0x02)<<1);
          cpu->PC.aword+=2;return 9;break;

        case 0x40:            /*  IN B,(C) */
          cpu->BC.abyte.high=in(cpu->BC.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->BC.abyte.high];
          cpu->PC.aword+=2;return 12;break;
        case 0x48:            /*  IN C,(C) */
          cpu->BC.abyte.low=in(cpu->BC.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->BC.abyte.low];
          cpu->PC.aword+=2;return 12;break;
        case 0x50:            /*  IN D,(C) */
          cpu->DE.abyte.high=in(cpu->BC.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->DE.abyte.high];
          cpu->PC.aword+=2;return 12;break;
        case 0x58:            /*  IN E,(C) */
          cpu->DE.abyte.low=in(cpu->BC.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->DE.abyte.low];
          cpu->PC.aword+=2;return 12;break;
        case 0x60:            /*  IN H,(C) */
          cpu->HL.abyte.high=in(cpu->BC.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->HL.abyte.high];
          cpu->PC.aword+=2;return 12;break;
        case 0x68:            /*  IN L,(C) */
          cpu->HL.abyte.low=in(cpu->BC.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->HL.abyte.low];
          cpu->PC.aword+=2;return 12;break;
        case 0x70:            /*  IN (C) */
          dw.abyte.low=in(cpu->BC.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[dw.abyte.low];
          cpu->PC.aword+=2;return 12;break;
        case 0x78:            /*  IN A,(C) */
          cpu->AF.abyte.high=in(cpu->BC.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->AF.abyte.high];
          cpu->PC.aword+=2;return 12;break;

        case 0xa2:  M_INI;cpu->PC.aword+=2;return 16;break; /*  INI */
        case 0xb2:            /*  INIR */
          M_INI;if(cpu->AF.abyte.low & FLAG_Z)
                  {cpu->PC.aword+=2;return 16;} else
                  {return 21;}
          break;

        case 0xaa:  M_IND;cpu->PC.aword+=2;return 16;break; /*  IND */
        case 0xba:            /*  INDR */
          M_IND;if(cpu->AF.abyte.low & FLAG_Z)
                  {cpu->PC.aword+=2;return 16;} else
                  {return 21;}
          break;

        case 0x41:  out(cpu->BC.abyte.low,cpu->BC.abyte.high);cpu->PC.aword+=2;return 12; /*  OUT (C),B */
        case 0x49:  out(cpu->BC.abyte.low,cpu->BC.abyte.low );cpu->PC.aword+=2;return 12; /*  OUT (C),C */
        case 0x51:  out(cpu->BC.abyte.low,cpu->DE.abyte.high);cpu->PC.aword+=2;return 12; /*  OUT (C),D */
        case 0x59:  out(cpu->BC.abyte.low,cpu->DE.abyte.low );cpu->PC.aword+=2;return 12; /*  OUT (C),E */
        case 0x61:  out(cpu->BC.abyte.low,cpu->HL.abyte.high);cpu->PC.aword+=2;return 12; /*  OUT (C),H */
        case 0x69:  out(cpu->BC.abyte.low,cpu->HL.abyte.low );cpu->PC.aword+=2;return 12; /*  OUT (C),L */
        case 0x79:  out(cpu->BC.abyte.low,cpu->AF.abyte.high);cpu->PC.aword+=2;return 12; /*  OUT (C),A */

        case 0xa3:  M_OUTI;cpu->PC.aword+=2;return 16;break;  /*  OUTI */
        case 0xb3:            /*  OTIR */
          M_OUTI;if(cpu->AF.abyte.low & FLAG_Z)
                   {cpu->PC.aword+=2;return 16;} else
                   {return 21;}
          break;

        case 0xab:  M_OUTD;cpu->PC.aword+=2;return 16;break;  /*  OUTD */
        case 0xbb:            /*  OTDR */
          M_OUTD;if(cpu->AF.abyte.low & FLAG_Z)
                   {cpu->PC.aword+=2;return 16;} else
                   {return 21;}
          break;


        case 0x4a:  M_ADC16(HL,BC);cpu->PC.aword+=2;return 15;break;  /*  ADC HL,BC */
        case 0x5a:  M_ADC16(HL,DE);cpu->PC.aword+=2;return 15;break;  /*  ADC HL,DE */
        case 0x6a:  M_ADC16(HL,HL);cpu->PC.aword+=2;return 15;break;  /*  ADC HL,HL */
        case 0x7a:  M_ADC16(HL,SP);cpu->PC.aword+=2;return 15;break;  /*  ADC HL.SP */

        case 0x42:  M_SBC16(HL,BC);cpu->PC.aword+=2;return 15;break;  /*  SBC HL,BC */
        case 0x52:  M_SBC16(HL,DE);cpu->PC.aword+=2;return 15;break;  /*  SBC HL,DE */
        case 0x62:  M_SBC16(HL,HL);cpu->PC.aword+=2;return 15;break;  /*  SBC HL,HL */
        case 0x72:  M_SBC16(HL,SP);cpu->PC.aword+=2;return 15;break;  /*  SBC HL,SP */

        case 0xa0:  M_LDI;cpu->PC.aword+=2;return 16;break;       /*  LDI */
        case 0xb0:                  /*  LDIR */
          M_LDI;if(cpu->AF.abyte.low & FLAG_PV)
                  {return 21;} else
                  {cpu->PC.aword+=2;return 16;}
          break;

        case 0xa8:  M_LDD;cpu->PC.aword+=2;return 16;break;       /*  LDD */
        case 0xb8:                  /*  LDDR */
          M_LDD;if(cpu->AF.abyte.low & FLAG_PV)
                  {return 21;} else
                  {cpu->PC.aword+=2;return 16;}
          break;

        case 0xa1:  M_CPI;cpu->PC.aword+=2;return 16;break;       /*  CPI */
        case 0xb1:                  /*  CPIR */
          M_CPI;if((cpu->AF.abyte.low&FLAG_Z) || (!(cpu->AF.abyte.low&FLAG_PV)))
                  {cpu->PC.aword+=2;return 16;} else
                  {return 21;}
          break;

        case 0xa9:  M_CPD;cpu->PC.aword+=2;return 16;break;       /*  CPD */
        case 0xb9:                  /*  CPDR */
          M_CPD;if((cpu->AF.abyte.low&FLAG_Z) || (!(cpu->AF.abyte.low&FLAG_PV)))
                  {cpu->PC.aword+=2;return 16;} else
                  {return 21;}

        case 0x4d:  POP_PC;return 14;break;           /*  RETI */

        case 0x45:                  /*  RETN */
          cpu->IFF = (cpu->IFF & 0xfe) | ((cpu->IFF&0x02)>>1);POP_PC;
          return 14;break;

        case 0x44:                  /*  NEG */
          dw.abyte.low=cpu->AF.abyte.high;cpu->AF.abyte.high=0;M_SUB(dw.abyte.low);cpu->PC.aword+=2;return 8;break;

        case 0x46:  cpu->IFF &= 0xf3;cpu->PC.aword+=2;return 8;break;   /*  IM0 */
        case 0x56:  cpu->IFF=(cpu->IFF&0xf3)|0x04;cpu->PC.aword+=2;return 8;break;  /*  IM1 */
        case 0x5e:  cpu->IFF=(cpu->IFF&0xf3)|0x08;cpu->PC.aword+=2;return 8;break;  /*  IM2 */


        case 0x6f:                  /*  RLD */
          dw.abyte.low  = read(cpu->HL.aword);
          dw.abyte.high = cpu->AF.abyte.high;
          cpu->AF.abyte.high = (dw.abyte.high&0xf0)|((dw.abyte.low>>4)&0x0f);
          dw.abyte.low = (dw.abyte.low<<4) | (dw.abyte.high&0x0f);
          write(cpu->HL.aword,dw.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->AF.abyte.high];
          cpu->PC.aword+=2;return 18;break;
        case 0x67:                  /*  RRD */
          dw.abyte.low  = read(cpu->HL.aword);
          dw.abyte.high = cpu->AF.abyte.high;
          cpu->AF.abyte.high = (cpu->AF.abyte.high&0xf0)|(dw.abyte.low&0x0f);
          dw.abyte.low = ((dw.abyte.low>>4)&0x0f) | (dw.abyte.high<<4);
          write(cpu->HL.aword,dw.abyte.low);
          cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[cpu->AF.abyte.high];
          cpu->PC.aword+=2;return 18;break;

        default:
          return Z80_IllegalInstruction(cpu);
          break;
      }
/*     return 4; */
  }


/*  Decode instructions with the DD prefix */
int Z80_Decode_DD ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,/*tt,*/t2;
    union_word  dw;

    cpu->data[1] = read ( cpu->PC.aword+1 );

    switch (cpu->data[1])
      {
/*         case 0x03: cpu->BC.aword++;cpu->PC.aword+=2;return 6;break; INC BC */
/*         case 0x13: cpu->DE.aword++;cpu->PC.aword++;return 6;break; INC DE */
/*         case 0x23: cpu->HL.aword++;cpu->PC.aword++;return 6;break; INC HL */
/*         case 0x33: cpu->SP.aword++;cpu->PC.aword++;return 6;break; INC SP */

        case 0xc7:  cpu->PC.aword+=2;PUSH_PC;cpu->PC.aword = 0x00;return 11+3;break;  /*  RST 00h */
        case 0xcf:  cpu->PC.aword+=2;PUSH_PC;cpu->PC.aword = 0x08;return 11+3;break;  /*  RST 08h */
        case 0xd7:  cpu->PC.aword+=2;PUSH_PC;cpu->PC.aword = 0x10;return 11+3;break;  /*  RST 10h */
        case 0xdf:  cpu->PC.aword+=2;PUSH_PC;cpu->PC.aword = 0x18;return 11+3;break;  /*  RST 18h */
        case 0xe7:  cpu->PC.aword+=2;PUSH_PC;cpu->PC.aword = 0x20;return 11+3;break;  /*  RST 20h */
        case 0xef:  cpu->PC.aword+=2;PUSH_PC;cpu->PC.aword = 0x28;return 11+3;break;  /*  RST 28h */
        case 0xf7:  cpu->PC.aword+=2;PUSH_PC;cpu->PC.aword = 0x30;return 11+3;break;  /*  RST 30h */
        case 0xff:  cpu->PC.aword+=2;PUSH_PC;cpu->PC.aword = 0x38;return 11+3;break;  /*  RST 38h */


        case 0x21:  cpu->IX.abyte.low  = read(cpu->PC.aword+2); /*  LD IX,nn */
      cpu->IX.abyte.high = read(cpu->PC.aword+3);
      cpu->PC.aword+=4;return 14;break;

        case 0x22:  dw.abyte.low  = read(cpu->PC.aword+2);    /*  LD (nn),IX */
      dw.abyte.high = read(cpu->PC.aword+3);
      write(dw.aword+0,cpu->IX.abyte.low);
      write(dw.aword+1,cpu->IX.abyte.high);
      cpu->PC.aword+=4;return 20;break;

        case 0x2a:  dw.abyte.low  = read(cpu->PC.aword+2);    /*  LD IX,(nn) */
      dw.abyte.high = read(cpu->PC.aword+3);
      cpu->IX.abyte.low  = read(dw.aword+0);
      cpu->IX.abyte.high = read(dw.aword+1);
      cpu->PC.aword+=4;return 20;break;

        case 0x36:                /*  LD (IX+d),n */
          write(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)), read(cpu->PC.aword+3));
          cpu->PC.aword+=4;return 19;break;

        case 0x46:                /*  LD B,(IX+d) */
          cpu->BC.abyte.high = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x4e:                /*  LD C,(IX+d) */
          cpu->BC.abyte.low  = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x56:                /*  LD D,(IX+d) */
          cpu->DE.abyte.high = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x5e:                /*  LD E,(IX+d) */
          cpu->DE.abyte.low  = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x66:                /*  LD H,(IX+d) */
          cpu->HL.abyte.high = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x6e:                /*  LD L,(IX+d) */
          cpu->HL.abyte.low  = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x7e:                /*  LD A,(IX+d) */
          cpu->AF.abyte.high = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;

        case 0x70:                /*  LD (IX+d),B */
          write(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->BC.abyte.high);
          cpu->PC.aword+=3;return 19;break;
        case 0x71:                /*  LD (IX+d),C */
          write(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->BC.abyte.low );
          cpu->PC.aword+=3;return 19;break;
        case 0x72:                /*  LD (IX+d),D */
          write(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->DE.abyte.high);
          cpu->PC.aword+=3;return 19;break;
        case 0x73:                /*  LD (IX+d),E */
          write(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->DE.abyte.low );
          cpu->PC.aword+=3;return 19;break;
        case 0x74:                /*  LD (IX+d),H */
          write(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->HL.abyte.high);
          cpu->PC.aword+=3;return 19;break;
        case 0x75:                    /*  LD (IX+d),L */
          write(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->HL.abyte.low );
          cpu->PC.aword+=3;return 19;break;
        case 0x77:                /*  LD (IX+d),A */
          write(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->AF.abyte.high);
          cpu->PC.aword+=3;return 19;break;

        case 0xe1:  M_POP(IX);cpu->PC.aword+=2;return 14;break;   /*  POP IX */
        case 0xe5:  M_PUSH(IX);cpu->PC.aword+=2;return 15;break;    /*  PUSH IX */
        case 0xe9:  cpu->PC.aword = cpu->IX.aword;return 8;break;   /*  JP IX */

        case 0xf9:  cpu->SP.aword = cpu->IX.aword;cpu->PC.aword+=2;return 10;break; /*  LD SP,IX */

        case 0x09:  M_ADD16(IX,BC);cpu->PC.aword+=2;return 15;break;    /*  ADD IX,BC */
        case 0x19:  M_ADD16(IX,DE);cpu->PC.aword+=2;return 15;break;    /*  ADD IX,DE */
        case 0x29:  M_ADD16(IX,IX);cpu->PC.aword+=2;return 15;break;    /*  ADD IX,IX */
        case 0x39:  M_ADD16(IX,SP);cpu->PC.aword+=2;return 15;break;    /*  ADD IX,SP */

        case 0x23:  cpu->IX.aword++;cpu->PC.aword+=2;return 10;break;   /*  INC IX */
        case 0x2b:  cpu->IX.aword--;cpu->PC.aword+=2;return 10;break;   /*  DEC IX */

        case 0x86:                /*  ADD A,(IX+d) */
          dw.abyte.low = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_ADD(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0x8e:                /*  ADC A,(IX+d) */
          dw.abyte.low = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_ADC(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0x96:                /*  SUB A,(IX+d) */
          dw.abyte.low = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_SUB(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0x9e:                /*  SBC A,(IX+d) */
          dw.abyte.low = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_SBC(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0xa6:                /*  AND A,(IX+d) */
          dw.abyte.low = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_AND(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0xb6:                /*  OR A,(IX+d) */
          dw.abyte.low = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_OR(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0xae:                /*  XOR A,(IX+d) */
          dw.abyte.low = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_XOR(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0xbe:                /*  CP A,(IX+d) */
          dw.abyte.low = read(cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_CP(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;

        case 0x34:                /*  INC (IX+d) */
          t2.aword.low.aword = cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2));
          dw.abyte.low = read(t2.aword.low.aword);
          M_INC(dw.abyte.low);
          write(t2.aword.low.aword, dw.abyte.low);
          cpu->PC.aword+=3;return 23;break;

        case 0x35:                /*  DEC (IX+d) */
          t2.aword.low.aword = cpu->IX.aword + BYTE2WORD(read(cpu->PC.aword+2));
          dw.abyte.low = read(t2.aword.low.aword);
          M_DEC(dw.abyte.low);
          write(t2.aword.low.aword, dw.abyte.low);
          cpu->PC.aword+=3;return 23;break;

        case 0xe3:                /*  EX (SP),IX */
          dw.abyte.low  = read(cpu->SP.aword+0);
          dw.abyte.high = read(cpu->SP.aword+1);
          write(cpu->SP.aword+0,cpu->IX.abyte.low);
          write(cpu->SP.aword+1,cpu->IX.abyte.high);
          cpu->IX.aword = dw.aword;
          cpu->PC.aword+=2;return 23;break;

        case 0x24: M_INC(cpu->IX.abyte.high);cpu->PC.aword+=2;return 11;break; /*  INC IXh */
        case 0x26: cpu->IX.abyte.high = read(cpu->PC.aword+2);cpu->PC.aword+=3;return 11;break; /*  LD IXh,nn */

        case 0x45: cpu->BC.abyte.high = cpu->IX.abyte.low;cpu->PC.aword+=2;return 7;break;    /*  LD B,IXl */
        case 0x4d: cpu->BC.abyte.low  = cpu->IX.abyte.low;cpu->PC.aword+=2;return 7;break;    /*  LD C,IXl */

        case 0x54: cpu->DE.abyte.high = cpu->IX.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD D,IXh */
        case 0x5d: cpu->DE.abyte.low  = cpu->IX.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD E,IXl */
        case 0x60: cpu->IX.abyte.high = cpu->BC.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IXh,B */
        case 0x62: cpu->IX.abyte.high = cpu->DE.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IXh,D */
        case 0x67: cpu->IX.abyte.high = cpu->AF.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IXh,A */
        case 0x68: cpu->IX.abyte.low  = cpu->BC.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IXl,B */
        case 0x6b: cpu->IX.abyte.low  = cpu->DE.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD IXl,E */
        case 0x6f: cpu->IX.abyte.low  = cpu->AF.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IXl,A */

        case 0x7c: cpu->AF.abyte.high = cpu->IX.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD A,IXh */
        case 0x55: cpu->DE.abyte.high = cpu->IX.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD D,IXl */
        case 0x7d: cpu->AF.abyte.high = cpu->IX.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD A,IXl */

        case 0x85: M_ADD(cpu->IX.abyte.low); cpu->PC.aword+=2;return 7;break;     /*  ADD IXl */
        case 0x8c: M_ADC(cpu->IX.abyte.high);cpu->PC.aword+=2;return 7;break;     /*  ADC IXh */
        case 0x8d: M_ADC(cpu->IX.abyte.low); cpu->PC.aword+=2;return 7;break;     /*  ADC IXl */
        case 0x95: M_SUB(cpu->IX.abyte.low); cpu->PC.aword+=2;return 7;break;     /*  SUB IXl */

        case 0xad: M_XOR(cpu->IX.abyte.low);cpu->PC.aword+=2;return 7;break;    /*  XOR IXl */
        case 0xb5: M_OR(cpu->IX.abyte.low);cpu->PC.aword+=2;return 7;break;     /*  OR A,IXl */
        case 0xbd: M_CP(cpu->IX.abyte.low);cpu->PC.aword+=2;return 7;break;     /*  CP IXl */

        case 0xeb:  M_EX(DE,IX);cpu->PC.aword+=2;return 7;break;    /*  EX DE,IX */

        case 0xcb:
          return Z80_Decode_DDCB ( cpu );
          break;

        default:
          return Z80_IllegalInstruction(cpu);
          break;
      }
/*     return 4; */
  }


/*  Decode instructions with the FD prefix */
int Z80_Decode_FD ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,/*tt,*/t2;
    union_word  dw;

    cpu->data[1] = read ( cpu->PC.aword+1 );

    switch (cpu->data[1])
      {
        case 0x21:  cpu->IY.abyte.low  = read(cpu->PC.aword+2); /*  LD IY,nn */
      cpu->IY.abyte.high = read(cpu->PC.aword+3);
      cpu->PC.aword+=4;return 14;break;

        case 0x22:  dw.abyte.low  = read(cpu->PC.aword+2);    /*  LD (nn),IY */
      dw.abyte.high = read(cpu->PC.aword+3);
      write(dw.aword+0,cpu->IY.abyte.low);
      write(dw.aword+1,cpu->IY.abyte.high);
      cpu->PC.aword+=4;return 20;break;

        case 0x2a:  dw.abyte.low  = read(cpu->PC.aword+2);    /*  LD IY,(nn) */
      dw.abyte.high = read(cpu->PC.aword+3);
      cpu->IY.abyte.low  = read(dw.aword+0);
      cpu->IY.abyte.high = read(dw.aword+1);
      cpu->PC.aword+=4;return 20;break;

        case 0x36:                /*  LD (IY+d),n */
          write(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)), read(cpu->PC.aword+3));
          cpu->PC.aword+=4;return 19;break;

        case 0x46:                /*  LD B,(IY+d) */
          cpu->BC.abyte.high = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x4e:                /*  LD C,(IY+d) */
          cpu->BC.abyte.low  = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x56:                /*  LD D,(IY+d) */
          cpu->DE.abyte.high = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x5e:                /*  LD E,(IY+d) */
          cpu->DE.abyte.low  = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x66:                /*  LD H,(IY+d) */
          cpu->HL.abyte.high = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x6e:                /*  LD L,(IY+d) */
          cpu->HL.abyte.low  = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;
        case 0x7e:                /*  LD A,(IY+d) */
          cpu->AF.abyte.high = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          cpu->PC.aword+=3;return 19;break;

        case 0x70:                /*  LD (IY+d),B */
          write(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->BC.abyte.high);
          cpu->PC.aword+=3;return 19;break;
        case 0x71:                /*  LD (IY+d),C */
          write(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->BC.abyte.low );
          cpu->PC.aword+=3;return 19;break;
        case 0x72:                /*  LD (IY+d),D */
          write(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->DE.abyte.high);
          cpu->PC.aword+=3;return 19;break;
        case 0x73:                /*  LD (IY+d),E */
          write(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->DE.abyte.low );
          cpu->PC.aword+=3;return 19;break;
        case 0x74:                /*  LD (IY+d),H */
          write(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->HL.abyte.high);
          cpu->PC.aword+=3;return 19;break;
        case 0x75:                    /*  LD (IY+d),L */
          write(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->HL.abyte.low );
          cpu->PC.aword+=3;return 19;break;
        case 0x77:                /*  LD (IY+d),A */
          write(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)), cpu->AF.abyte.high);
          cpu->PC.aword+=3;return 19;break;

        case 0xe1:  M_POP(IY);cpu->PC.aword+=2;return 14;break;   /*  POP IY */
        case 0xe5:  M_PUSH(IY);cpu->PC.aword+=2;return 15;break;    /*  PUSH IY */
        case 0xe9:  cpu->PC.aword = cpu->IY.aword;return 8;break;   /*  JP IY */

        case 0xf9:  cpu->SP.aword = cpu->IY.aword;cpu->PC.aword+=2;return 10;break; /*  LD SP,IY */

        case 0x09:  M_ADD16(IY,BC);cpu->PC.aword+=2;return 15;break;    /*  ADD IY,BC */
        case 0x19:  M_ADD16(IY,DE);cpu->PC.aword+=2;return 15;break;    /*  ADD IY,DE */
        case 0x29:  M_ADD16(IY,IY);cpu->PC.aword+=2;return 15;break;    /*  ADD IY,IY */
        case 0x39:  M_ADD16(IY,SP);cpu->PC.aword+=2;return 15;break;    /*  ADD IY,SP */

        case 0x23:  cpu->IY.aword++;cpu->PC.aword+=2;return 10;break;   /*  INC IY */
        case 0x2b:  cpu->IY.aword--;cpu->PC.aword+=2;return 10;break;   /*  DEC IY */

        case 0x86:                /*  ADD A,(IY+d) */
          dw.abyte.low = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_ADD(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0x8e:                /*  ADC A,(IY+d) */
          dw.abyte.low = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_ADC(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0x96:                /*  SUB A,(IY+d) */
          dw.abyte.low = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_SUB(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0x9e:                /*  SBC A,(IY+d) */
          dw.abyte.low = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_SBC(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0xa6:                /*  AND A,(IY+d) */
          dw.abyte.low = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_AND(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0xb6:                /*  OR A,(IY+d) */
          dw.abyte.low = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_OR(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0xae:                /*  XOR A,(IY+d) */
          dw.abyte.low = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_XOR(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;
        case 0xbe:                /*  CP A,(IY+d) */
          dw.abyte.low = read(cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2)));
          M_CP(dw.abyte.low);
          cpu->PC.aword+=3;return 19;break;

        case 0x34:                /*  INC (IY+d) */
          t2.aword.low.aword = cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2));
          dw.abyte.low = read(t2.aword.low.aword);
          M_INC(dw.abyte.low);
          write(t2.aword.low.aword, dw.abyte.low);
          cpu->PC.aword+=3;return 23;break;

        case 0x35:                /*  DEC (IY+d) */
          t2.aword.low.aword = cpu->IY.aword + BYTE2WORD(read(cpu->PC.aword+2));
          dw.abyte.low = read(t2.aword.low.aword);
          M_DEC(dw.abyte.low);
          write(t2.aword.low.aword, dw.abyte.low);
          cpu->PC.aword+=3;return 23;break;

        case 0xe3:                /*  EX (SP),IY */
          dw.abyte.low  = read(cpu->SP.aword+0);
          dw.abyte.high = read(cpu->SP.aword+1);
          write(cpu->SP.aword+0,cpu->IY.abyte.low);
          write(cpu->SP.aword+1,cpu->IY.abyte.high);
          cpu->IY.aword = dw.aword;
          cpu->PC.aword+=2;return 23;break;

        case 0x60: cpu->IY.abyte.high = cpu->BC.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IYh,B */
        case 0x61: cpu->IY.abyte.high = cpu->BC.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD IYh,C */
        case 0x62: cpu->IY.abyte.high = cpu->DE.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IYh,D */
        case 0x63: cpu->IY.abyte.high = cpu->DE.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD IYh,E */
        case 0x64: cpu->IY.abyte.high = cpu->HL.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IYh,H */
        case 0x65: cpu->IY.abyte.high = cpu->HL.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD IYh,L */
        case 0x67: cpu->IY.abyte.high = cpu->AF.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IYh,A */

        case 0x68: cpu->IY.abyte.low  = cpu->BC.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IYl,B */
        case 0x69: cpu->IY.abyte.low  = cpu->BC.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD IYl,C */
        case 0x6a: cpu->IY.abyte.low  = cpu->DE.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IYl,D */
        case 0x6b: cpu->IY.abyte.low  = cpu->DE.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD IYl,E */
        case 0x6c: cpu->IY.abyte.low  = cpu->HL.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IYl,H */
        case 0x6d: cpu->IY.abyte.low  = cpu->HL.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD IYl,L */
        case 0x6f: cpu->IY.abyte.low  = cpu->AF.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD IYl,A */

        case 0x26: cpu->IY.abyte.high = read(cpu->PC.aword+2);cpu->PC.aword+=3;return 19;break; /*  LD IYh,nn */
        case 0x45: cpu->BC.abyte.high = cpu->IY.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD B,IYl */
        case 0x54: cpu->DE.abyte.high = cpu->IY.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD D,IYh */
        case 0x55: cpu->DE.abyte.high = cpu->IY.abyte.low; cpu->PC.aword+=2;return 7;break;  /*  LD D,IYl */
        case 0x5d: cpu->DE.abyte.low  = cpu->IY.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD E,IYl */

        case 0x4d: cpu->BC.abyte.low  = cpu->IY.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD C,IYl */

        case 0x7c: cpu->AF.abyte.high = cpu->IY.abyte.high;cpu->PC.aword+=2;return 7;break; /*  LD A,IYh */
        case 0x7d: cpu->AF.abyte.high = cpu->IY.abyte.low; cpu->PC.aword+=2;return 7;break; /*  LD A,IYl */

        case 0x02: write(cpu->BC.aword,cpu->AF.abyte.high);cpu->PC.aword+=2;return 11;break;  /*  LD (BC),A */
        case 0x0a: cpu->AF.abyte.high = read(cpu->BC.aword);cpu->PC.aword+=2;return 11;break; /*  LD A,(BC) */

        case 0x85: M_ADD(cpu->IY.abyte.low); cpu->PC.aword+=2;return 7;break;     /*  ADD IYl */
        case 0x8d: M_ADC(cpu->IY.abyte.low); cpu->PC.aword+=2;return 7;break;     /*  ADC IYl */
        case 0x9c: M_SBC(cpu->IY.abyte.high);cpu->PC.aword+=2;return 7;break;     /*  SBC IYh */
        case 0xad: M_XOR(cpu->IY.abyte.low); cpu->PC.aword+=2;return 7;break;     /*  XOR IYl */
        case 0xb5: M_OR (cpu->IY.abyte.low); cpu->PC.aword+=2;return 7;break;     /*  OR  IYl */
        case 0xbd: M_CP (cpu->IY.abyte.low); cpu->PC.aword+=2;return 7;break;     /*  CP  IYl */

        case 0x2d: M_DEC(cpu->IY.abyte.low);cpu->PC.aword+=2;return 7;break;      /*  DEC IYl */
        case 0x2e: cpu->IY.abyte.low = read(cpu->PC.aword+2);cpu->PC.aword+=3;return 11;break;  /*  LD IYl,nn */

        case 0xeb:  M_EX(DE,IY);cpu->PC.aword+=2;return 7;break;    /*  EX DE,IY */

        case 0xcb:
          return Z80_Decode_FDCB ( cpu );
          break;

        default:
          return Z80_IllegalInstruction(cpu);
          break;
      }
/*           return Z80_IllegalInstruction(cpu); */
/*     return 4; */
  }


/*  Decode instructions with the CB prefix */
int Z80_Decode_CB ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,/*tt,*/t2;
    union_word  dw;

    cpu->data[1] = read ( cpu->PC.aword+1 );

    switch (cpu->data[1])
      {
        case 0x40:  M_BIT(0x01,cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 0,B */
        case 0x41:  M_BIT(0x01,cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 0,C */
        case 0x42:  M_BIT(0x01,cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 0,D */
        case 0x43:  M_BIT(0x01,cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 0,E */
        case 0x44:  M_BIT(0x01,cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 0,H */
        case 0x45:  M_BIT(0x01,cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 0,L */
        case 0x47:  M_BIT(0x01,cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 0,A */

        case 0x48:  M_BIT(0x02,cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 1,B */
        case 0x49:  M_BIT(0x02,cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 1,C */
        case 0x4a:  M_BIT(0x02,cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 1,D */
        case 0x4b:  M_BIT(0x02,cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 1,E */
        case 0x4c:  M_BIT(0x02,cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 1,H */
        case 0x4d:  M_BIT(0x02,cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 1,L */
        case 0x4f:  M_BIT(0x02,cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 1,A */

        case 0x50:  M_BIT(0x04,cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 2,B */
        case 0x51:  M_BIT(0x04,cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 2,C */
        case 0x52:  M_BIT(0x04,cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 2,D */
        case 0x53:  M_BIT(0x04,cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 2,E */
        case 0x54:  M_BIT(0x04,cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 2,H */
        case 0x55:  M_BIT(0x04,cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 2,L */
        case 0x57:  M_BIT(0x04,cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 2,A */

        case 0x58:  M_BIT(0x08,cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 3,B */
        case 0x59:  M_BIT(0x08,cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 3,C */
        case 0x5a:  M_BIT(0x08,cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 3,D */
        case 0x5b:  M_BIT(0x08,cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 3,E */
        case 0x5c:  M_BIT(0x08,cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 3,H */
        case 0x5d:  M_BIT(0x08,cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 3,L */
        case 0x5f:  M_BIT(0x08,cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 3,A */

        case 0x60:  M_BIT(0x10,cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 4,B */
        case 0x61:  M_BIT(0x10,cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 4,C */
        case 0x62:  M_BIT(0x10,cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 4,D */
        case 0x63:  M_BIT(0x10,cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 4,E */
        case 0x64:  M_BIT(0x10,cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 4,H */
        case 0x65:  M_BIT(0x10,cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 4,L */
        case 0x67:  M_BIT(0x10,cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 4,A */

        case 0x68:  M_BIT(0x20,cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 5,B */
        case 0x69:  M_BIT(0x20,cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 5,C */
        case 0x6a:  M_BIT(0x20,cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 5,D */
        case 0x6b:  M_BIT(0x20,cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 5,E */
        case 0x6c:  M_BIT(0x20,cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 5,H */
        case 0x6d:  M_BIT(0x20,cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 5,L */
        case 0x6f:  M_BIT(0x20,cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 5,A */

        case 0x70:  M_BIT(0x40,cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 6,B */
        case 0x71:  M_BIT(0x40,cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 6,C */
        case 0x72:  M_BIT(0x40,cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 6,D */
        case 0x73:  M_BIT(0x40,cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 6,E */
        case 0x74:  M_BIT(0x40,cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 6,H */
        case 0x75:  M_BIT(0x40,cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 6,L */
        case 0x77:  M_BIT(0x40,cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 6,A */

        case 0x78:  M_BIT(0x80,cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 7,B */
        case 0x79:  M_BIT(0x80,cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 7,C */
        case 0x7a:  M_BIT(0x80,cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 7,D */
        case 0x7b:  M_BIT(0x80,cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 7,E */
        case 0x7c:  M_BIT(0x80,cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 7,H */
        case 0x7d:  M_BIT(0x80,cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break; /*  BIT 7,L */
        case 0x7f:  M_BIT(0x80,cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break; /*  BIT 7,A */

        case 0xc0:  cpu->BC.abyte.high |= 0x01;cpu->PC.aword+=2;return 8;break; /*  SET 0,B */
        case 0xc1:  cpu->BC.abyte.low  |= 0x01;cpu->PC.aword+=2;return 8;break; /*  SET 0,C */
        case 0xc2:  cpu->DE.abyte.high |= 0x01;cpu->PC.aword+=2;return 8;break; /*  SET 0,D */
        case 0xc3:  cpu->DE.abyte.low  |= 0x01;cpu->PC.aword+=2;return 8;break; /*  SET 0,E */
        case 0xc4:  cpu->HL.abyte.high |= 0x01;cpu->PC.aword+=2;return 8;break; /*  SET 0,H */
        case 0xc5:  cpu->HL.abyte.low  |= 0x01;cpu->PC.aword+=2;return 8;break; /*  SET 0,L */
        case 0xc7:  cpu->AF.abyte.high |= 0x01;cpu->PC.aword+=2;return 8;break; /*  SET 0,A */

        case 0xc8:  cpu->BC.abyte.high |= 0x02;cpu->PC.aword+=2;return 8;break; /*  SET 1,B */
        case 0xc9:  cpu->BC.abyte.low  |= 0x02;cpu->PC.aword+=2;return 8;break; /*  SET 1,C */
        case 0xca:  cpu->DE.abyte.high |= 0x02;cpu->PC.aword+=2;return 8;break; /*  SET 1,D */
        case 0xcb:  cpu->DE.abyte.low  |= 0x02;cpu->PC.aword+=2;return 8;break; /*  SET 1,E */
        case 0xcc:  cpu->HL.abyte.high |= 0x02;cpu->PC.aword+=2;return 8;break; /*  SET 1,H */
        case 0xcd:  cpu->HL.abyte.low  |= 0x02;cpu->PC.aword+=2;return 8;break; /*  SET 1,L */
        case 0xcf:  cpu->AF.abyte.high |= 0x02;cpu->PC.aword+=2;return 8;break; /*  SET 1,A */

        case 0xd0:  cpu->BC.abyte.high |= 0x04;cpu->PC.aword+=2;return 8;break; /*  SET 2,B */
        case 0xd1:  cpu->BC.abyte.low  |= 0x04;cpu->PC.aword+=2;return 8;break; /*  SET 2,C */
        case 0xd2:  cpu->DE.abyte.high |= 0x04;cpu->PC.aword+=2;return 8;break; /*  SET 2,D */
        case 0xd3:  cpu->DE.abyte.low  |= 0x04;cpu->PC.aword+=2;return 8;break; /*  SET 2,E */
        case 0xd4:  cpu->HL.abyte.high |= 0x04;cpu->PC.aword+=2;return 8;break; /*  SET 2,H */
        case 0xd5:  cpu->HL.abyte.low  |= 0x04;cpu->PC.aword+=2;return 8;break; /*  SET 2,L */
        case 0xd7:  cpu->AF.abyte.high |= 0x04;cpu->PC.aword+=2;return 8;break; /*  SET 2,A */

        case 0xd8:  cpu->BC.abyte.high |= 0x08;cpu->PC.aword+=2;return 8;break; /*  SET 3,B */
        case 0xd9:  cpu->BC.abyte.low  |= 0x08;cpu->PC.aword+=2;return 8;break; /*  SET 3,C */
        case 0xda:  cpu->DE.abyte.high |= 0x08;cpu->PC.aword+=2;return 8;break; /*  SET 3,D */
        case 0xdb:  cpu->DE.abyte.low  |= 0x08;cpu->PC.aword+=2;return 8;break; /*  SET 3,E */
        case 0xdc:  cpu->HL.abyte.high |= 0x08;cpu->PC.aword+=2;return 8;break; /*  SET 3,H */
        case 0xdd:  cpu->HL.abyte.low  |= 0x08;cpu->PC.aword+=2;return 8;break; /*  SET 3,L */
        case 0xdf:  cpu->AF.abyte.high |= 0x08;cpu->PC.aword+=2;return 8;break; /*  SET 3,A */

        case 0xe0:  cpu->BC.abyte.high |= 0x10;cpu->PC.aword+=2;return 8;break; /*  SET 4,B */
        case 0xe1:  cpu->BC.abyte.low  |= 0x10;cpu->PC.aword+=2;return 8;break; /*  SET 4,C */
        case 0xe2:  cpu->DE.abyte.high |= 0x10;cpu->PC.aword+=2;return 8;break; /*  SET 4,D */
        case 0xe3:  cpu->DE.abyte.low  |= 0x10;cpu->PC.aword+=2;return 8;break; /*  SET 4,E */
        case 0xe4:  cpu->HL.abyte.high |= 0x10;cpu->PC.aword+=2;return 8;break; /*  SET 4,H */
        case 0xe5:  cpu->HL.abyte.low  |= 0x10;cpu->PC.aword+=2;return 8;break; /*  SET 4,L */
        case 0xe7:  cpu->AF.abyte.high |= 0x10;cpu->PC.aword+=2;return 8;break; /*  SET 4,A */

        case 0xe8:  cpu->BC.abyte.high |= 0x20;cpu->PC.aword+=2;return 8;break; /*  SET 5,B */
        case 0xe9:  cpu->BC.abyte.low  |= 0x20;cpu->PC.aword+=2;return 8;break; /*  SET 5,C */
        case 0xea:  cpu->DE.abyte.high |= 0x20;cpu->PC.aword+=2;return 8;break; /*  SET 5,D */
        case 0xeb:  cpu->DE.abyte.low  |= 0x20;cpu->PC.aword+=2;return 8;break; /*  SET 5,E */
        case 0xec:  cpu->HL.abyte.high |= 0x20;cpu->PC.aword+=2;return 8;break; /*  SET 5,H */
        case 0xed:  cpu->HL.abyte.low  |= 0x20;cpu->PC.aword+=2;return 8;break; /*  SET 5,L */
        case 0xef:  cpu->AF.abyte.high |= 0x20;cpu->PC.aword+=2;return 8;break; /*  SET 5,A */

        case 0xf0:  cpu->BC.abyte.high |= 0x40;cpu->PC.aword+=2;return 8;break; /*  SET 6,B */
        case 0xf1:  cpu->BC.abyte.low  |= 0x40;cpu->PC.aword+=2;return 8;break; /*  SET 6,C */
        case 0xf2:  cpu->DE.abyte.high |= 0x40;cpu->PC.aword+=2;return 8;break; /*  SET 6,D */
        case 0xf3:  cpu->DE.abyte.low  |= 0x40;cpu->PC.aword+=2;return 8;break; /*  SET 6,E */
        case 0xf4:  cpu->HL.abyte.high |= 0x40;cpu->PC.aword+=2;return 8;break; /*  SET 6,H */
        case 0xf5:  cpu->HL.abyte.low  |= 0x40;cpu->PC.aword+=2;return 8;break; /*  SET 6,L */
        case 0xf7:  cpu->AF.abyte.high |= 0x40;cpu->PC.aword+=2;return 8;break; /*  SET 6,A */

        case 0xf8:  cpu->BC.abyte.high |= 0x80;cpu->PC.aword+=2;return 8;break; /*  SET 7,B */
        case 0xf9:  cpu->BC.abyte.low  |= 0x80;cpu->PC.aword+=2;return 8;break; /*  SET 7,C */
        case 0xfa:  cpu->DE.abyte.high |= 0x80;cpu->PC.aword+=2;return 8;break; /*  SET 7,D */
        case 0xfb:  cpu->DE.abyte.low  |= 0x80;cpu->PC.aword+=2;return 8;break; /*  SET 7,E */
        case 0xfc:  cpu->HL.abyte.high |= 0x80;cpu->PC.aword+=2;return 8;break; /*  SET 7,H */
        case 0xfd:  cpu->HL.abyte.low  |= 0x80;cpu->PC.aword+=2;return 8;break; /*  SET 7,L */
        case 0xff:  cpu->AF.abyte.high |= 0x80;cpu->PC.aword+=2;return 8;break; /*  SET 7,A */


        case 0x80:  cpu->BC.abyte.high &= ~0x01;cpu->PC.aword+=2;return 8;break;  /*  RES 0,B */
        case 0x81:  cpu->BC.abyte.low  &= ~0x01;cpu->PC.aword+=2;return 8;break;  /*  RES 0,C */
        case 0x82:  cpu->DE.abyte.high &= ~0x01;cpu->PC.aword+=2;return 8;break;  /*  RES 0,D */
        case 0x83:  cpu->DE.abyte.low  &= ~0x01;cpu->PC.aword+=2;return 8;break;  /*  RES 0,E */
        case 0x84:  cpu->HL.abyte.high &= ~0x01;cpu->PC.aword+=2;return 8;break;  /*  RES 0,H */
        case 0x85:  cpu->HL.abyte.low  &= ~0x01;cpu->PC.aword+=2;return 8;break;  /*  RES 0,L */
        case 0x87:  cpu->AF.abyte.high &= ~0x01;cpu->PC.aword+=2;return 8;break;  /*  RES 0,A */
        case 0x88:  cpu->BC.abyte.high &= ~0x02;cpu->PC.aword+=2;return 8;break;  /*  RES 1,B */
        case 0x89:  cpu->BC.abyte.low  &= ~0x02;cpu->PC.aword+=2;return 8;break;  /*  RES 1,C */
        case 0x8a:  cpu->DE.abyte.high &= ~0x02;cpu->PC.aword+=2;return 8;break;  /*  RES 1,D */
        case 0x8b:  cpu->DE.abyte.low  &= ~0x02;cpu->PC.aword+=2;return 8;break;  /*  RES 1,E */
        case 0x8c:  cpu->HL.abyte.high &= ~0x02;cpu->PC.aword+=2;return 8;break;  /*  RES 1,H */
        case 0x8d:  cpu->HL.abyte.low  &= ~0x02;cpu->PC.aword+=2;return 8;break;  /*  RES 1,L */
        case 0x8f:  cpu->AF.abyte.high &= ~0x02;cpu->PC.aword+=2;return 8;break;  /*  RES 1,A */
        case 0x90:  cpu->BC.abyte.high &= ~0x04;cpu->PC.aword+=2;return 8;break;  /*  RES 2,B */
        case 0x91:  cpu->BC.abyte.low  &= ~0x04;cpu->PC.aword+=2;return 8;break;  /*  RES 2,C */
        case 0x92:  cpu->DE.abyte.high &= ~0x04;cpu->PC.aword+=2;return 8;break;  /*  RES 2,D */
        case 0x93:  cpu->DE.abyte.low  &= ~0x04;cpu->PC.aword+=2;return 8;break;  /*  RES 2,E */
        case 0x94:  cpu->HL.abyte.high &= ~0x04;cpu->PC.aword+=2;return 8;break;  /*  RES 2,H */
        case 0x95:  cpu->HL.abyte.low  &= ~0x04;cpu->PC.aword+=2;return 8;break;  /*  RES 2,L */
        case 0x97:  cpu->AF.abyte.high &= ~0x04;cpu->PC.aword+=2;return 8;break;  /*  RES 2,A */
        case 0x98:  cpu->BC.abyte.high &= ~0x08;cpu->PC.aword+=2;return 8;break;  /*  RES 3,B */
        case 0x99:  cpu->BC.abyte.low  &= ~0x08;cpu->PC.aword+=2;return 8;break;  /*  RES 3,C */
        case 0x9a:  cpu->DE.abyte.high &= ~0x08;cpu->PC.aword+=2;return 8;break;  /*  RES 3,D */
        case 0x9b:  cpu->DE.abyte.low  &= ~0x08;cpu->PC.aword+=2;return 8;break;  /*  RES 3,E */
        case 0x9c:  cpu->HL.abyte.high &= ~0x08;cpu->PC.aword+=2;return 8;break;  /*  RES 3,H */
        case 0x9d:  cpu->HL.abyte.low  &= ~0x08;cpu->PC.aword+=2;return 8;break;  /*  RES 3,L */
        case 0x9f:  cpu->AF.abyte.high &= ~0x08;cpu->PC.aword+=2;return 8;break;  /*  RES 3,A */
        case 0xa0:  cpu->BC.abyte.high &= ~0x10;cpu->PC.aword+=2;return 8;break;  /*  RES 4,B */
        case 0xa1:  cpu->BC.abyte.low  &= ~0x10;cpu->PC.aword+=2;return 8;break;  /*  RES 4,C */
        case 0xa2:  cpu->DE.abyte.high &= ~0x10;cpu->PC.aword+=2;return 8;break;  /*  RES 4,D */
        case 0xa3:  cpu->DE.abyte.low  &= ~0x10;cpu->PC.aword+=2;return 8;break;  /*  RES 4,E */
        case 0xa4:  cpu->HL.abyte.high &= ~0x10;cpu->PC.aword+=2;return 8;break;  /*  RES 4,H */
        case 0xa5:  cpu->HL.abyte.low  &= ~0x10;cpu->PC.aword+=2;return 8;break;  /*  RES 4,L */
        case 0xa7:  cpu->AF.abyte.high &= ~0x10;cpu->PC.aword+=2;return 8;break;  /*  RES 4,A */
        case 0xa8:  cpu->BC.abyte.high &= ~0x20;cpu->PC.aword+=2;return 8;break;  /*  RES 5,B */
        case 0xa9:  cpu->BC.abyte.low  &= ~0x20;cpu->PC.aword+=2;return 8;break;  /*  RES 5,C */
        case 0xaa:  cpu->DE.abyte.high &= ~0x20;cpu->PC.aword+=2;return 8;break;  /*  RES 5,D */
        case 0xab:  cpu->DE.abyte.low  &= ~0x20;cpu->PC.aword+=2;return 8;break;  /*  RES 5,E */
        case 0xac:  cpu->HL.abyte.high &= ~0x20;cpu->PC.aword+=2;return 8;break;  /*  RES 5,H */
        case 0xad:  cpu->HL.abyte.low  &= ~0x20;cpu->PC.aword+=2;return 8;break;  /*  RES 5,L */
        case 0xaf:  cpu->AF.abyte.high &= ~0x20;cpu->PC.aword+=2;return 8;break;  /*  RES 5,A */
        case 0xb0:  cpu->BC.abyte.high &= ~0x40;cpu->PC.aword+=2;return 8;break;  /*  RES 6,B */
        case 0xb1:  cpu->BC.abyte.low  &= ~0x40;cpu->PC.aword+=2;return 8;break;  /*  RES 6,C */
        case 0xb2:  cpu->DE.abyte.high &= ~0x40;cpu->PC.aword+=2;return 8;break;  /*  RES 6,D */
        case 0xb3:  cpu->DE.abyte.low  &= ~0x40;cpu->PC.aword+=2;return 8;break;  /*  RES 6,E */
        case 0xb4:  cpu->HL.abyte.high &= ~0x40;cpu->PC.aword+=2;return 8;break;  /*  RES 6,H */
        case 0xb5:  cpu->HL.abyte.low  &= ~0x40;cpu->PC.aword+=2;return 8;break;  /*  RES 6,L */
        case 0xb7:  cpu->AF.abyte.high &= ~0x40;cpu->PC.aword+=2;return 8;break;  /*  RES 6,A */
        case 0xb8:  cpu->BC.abyte.high &= ~0x80;cpu->PC.aword+=2;return 8;break;  /*  RES 7,B */
        case 0xb9:  cpu->BC.abyte.low  &= ~0x80;cpu->PC.aword+=2;return 8;break;  /*  RES 7,C */
        case 0xba:  cpu->DE.abyte.high &= ~0x80;cpu->PC.aword+=2;return 8;break;  /*  RES 7,D */
        case 0xbb:  cpu->DE.abyte.low  &= ~0x80;cpu->PC.aword+=2;return 8;break;  /*  RES 7,E */
        case 0xbc:  cpu->HL.abyte.high &= ~0x80;cpu->PC.aword+=2;return 8;break;  /*  RES 7,H */
        case 0xbd:  cpu->HL.abyte.low  &= ~0x80;cpu->PC.aword+=2;return 8;break;  /*  RES 7,L */
        case 0xbf:  cpu->AF.abyte.high &= ~0x80;cpu->PC.aword+=2;return 8;break;  /*  RES 7,A */


#define XX   HL
#define CYC  15
#define ILEN 2
#define OFS  0
#include "xx.h"
#undef XX
#undef CYC
#undef ILEN
#undef OFS


        case 0x00:  M_RLC(cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  RLC B */
        case 0x01:  M_RLC(cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  RLC C */
        case 0x02:  M_RLC(cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  RLC D */
        case 0x03:  M_RLC(cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  RLC E */
        case 0x04:  M_RLC(cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  RLC H */
        case 0x05:  M_RLC(cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  RLC L */
        case 0x07:  M_RLC(cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  RLC A */

        case 0x08:  M_RRC(cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  RRC B */
        case 0x09:  M_RRC(cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  RRC C */
        case 0x0a:  M_RRC(cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  RRC D */
        case 0x0b:  M_RRC(cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  RRC E */
        case 0x0c:  M_RRC(cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  RRC H */
        case 0x0d:  M_RRC(cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  RRC L */
        case 0x0f:  M_RRC(cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  RRC A */

        case 0x10:  M_RL(cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break;   /*  RL B */
        case 0x11:  M_RL(cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break;   /*  RL C */
        case 0x12:  M_RL(cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break;   /*  RL D */
        case 0x13:  M_RL(cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break;   /*  RL E */
        case 0x14:  M_RL(cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break;   /*  RL H */
        case 0x15:  M_RL(cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break;   /*  RL L */
        case 0x17:  M_RL(cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break;   /*  RL A */

        case 0x18:  M_RR(cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break;   /*  RR B */
        case 0x19:  M_RR(cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break;   /*  RR C */
        case 0x1a:  M_RR(cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break;   /*  RR D */
        case 0x1b:  M_RR(cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break;   /*  RR E */
        case 0x1c:  M_RR(cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break;   /*  RR H */
        case 0x1d:  M_RR(cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break;   /*  RR L */
        case 0x1f:  M_RR(cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break;   /*  RR A */

        case 0x20:  M_SLA(cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SLA B */
        case 0x21:  M_SLA(cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SLA C */
        case 0x22:  M_SLA(cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SLA D */
        case 0x23:  M_SLA(cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SLA E */
        case 0x24:  M_SLA(cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SLA H */
        case 0x25:  M_SLA(cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SLA L */
        case 0x27:  M_SLA(cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SLA A */

        case 0x28:  M_SRA(cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SRA B */
        case 0x29:  M_SRA(cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SRA C */
        case 0x2a:  M_SRA(cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SRA D */
        case 0x2b:  M_SRA(cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SRA E */
        case 0x2c:  M_SRA(cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SRA H */
        case 0x2d:  M_SRA(cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SRA L */
        case 0x2f:  M_SRA(cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SRA A */

        case 0x38:  M_SRL(cpu->BC.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SRL B */
        case 0x39:  M_SRL(cpu->BC.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SRL C */
        case 0x3a:  M_SRL(cpu->DE.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SRL D */
        case 0x3b:  M_SRL(cpu->DE.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SRL E */
        case 0x3c:  M_SRL(cpu->HL.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SRL H */
        case 0x3d:  M_SRL(cpu->HL.abyte.low );cpu->PC.aword+=2;return 8;break;  /*  SRL L */
        case 0x3f:  M_SRL(cpu->AF.abyte.high);cpu->PC.aword+=2;return 8;break;  /*  SRL A */

        default:
          return Z80_IllegalInstruction(cpu);
          break;
      }
/*     return 4; */
  }


/*  Decode the next instruction at cpu->PC */
/*  and determine the corresponding function */
int Z80_Decode ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t/*,tt*/;
    union_word  dw;

    cpu->data[0] = read ( cpu->PC.aword );

    switch (cpu->data[0])
      {
        /**** LD dd,nn ****/
        case 0x01:  read16(cpu->PC.aword+1,cpu->BC);cpu->PC.aword+=3;return 10;break; /*  00000001 LD BC,nn */
        case 0x11:  read16(cpu->PC.aword+1,cpu->DE);cpu->PC.aword+=3;return 10;break; /*  00010001 LD DE,nn */
        case 0x21:  read16(cpu->PC.aword+1,cpu->HL);cpu->PC.aword+=3;return 10;break;  /*  00100001 LD HL,nn */
        case 0x31:  read16(cpu->PC.aword+1,cpu->SP);cpu->PC.aword+=3;return 10;break; /*  00110001 LD SP,nn */

        case 0x22:  read16(cpu->PC.aword+1,dw);write16(dw.aword,cpu->HL);cpu->PC.aword+=3;return 16;break;  /*  00100010 LD (mm),HL */
        case 0x2a:  read16(cpu->PC.aword+1,dw);read16(dw.aword,cpu->HL);cpu->PC.aword+=3;return 16;break; /*  00101010 LD HL,(mm) */

        case 0x02:  write(cpu->BC.aword,cpu->AF.abyte.high);cpu->PC.aword++;return 7;break; /*  00000010 LD (BC),A */
        case 0x12:  write(cpu->DE.aword,cpu->AF.abyte.high);cpu->PC.aword++;return 7;break; /*  00010010 LD (DE),A */
        case 0x32:  read16(cpu->PC.aword+1,dw);write(dw.aword,cpu->AF.abyte.high);cpu->PC.aword+=3;return 13;break; /*  00110010 LD (mm),A */

        case 0x0a:  cpu->AF.abyte.high=read(cpu->BC.aword);cpu->PC.aword++;return 7;break;  /*  00001010 LD A,(BC) */
        case 0x1a:  cpu->AF.abyte.high=read(cpu->DE.aword);cpu->PC.aword++;return 7;break;  /*  00011010 LD A,(DE) */
        case 0x3a:  read16(cpu->PC.aword+1,dw);cpu->AF.abyte.high=read(dw.aword);cpu->PC.aword+=3;return 13;break;  /*  00111010 LD A,(mm) */

        case 0x36:  write(cpu->HL.aword,read(cpu->PC.aword+1));cpu->PC.aword+=2;return 10;break;  /*  00110110 LD (HL),n */


        /**** LD R8,R8 ****/
        case 0x40:  cpu->BC.abyte.high = cpu->BC.abyte.high;cpu->PC.aword++;return 4;break;   /*  01000000 LD B,B */
        case 0x41:  cpu->BC.abyte.high = cpu->BC.abyte.low; cpu->PC.aword++;return 4;break;   /*  01000001 LD B,C */
        case 0x42:  cpu->BC.abyte.high = cpu->DE.abyte.high;cpu->PC.aword++;return 4;break;   /*  01000010 LD B,D */
        case 0x43:  cpu->BC.abyte.high = cpu->DE.abyte.low; cpu->PC.aword++;return 4;break;   /*  01000011 LD B,E */
        case 0x44:  cpu->BC.abyte.high = cpu->HL.abyte.high;cpu->PC.aword++;return 4;break;   /*  01000100 LD B,H */
        case 0x45:  cpu->BC.abyte.high = cpu->HL.abyte.low; cpu->PC.aword++;return 4;break;   /*  01000101 LD B,L */
        case 0x47:  cpu->BC.abyte.high = cpu->AF.abyte.high;cpu->PC.aword++;return 4;break;   /*  01000111 LD B,A */

        case 0x48:  cpu->BC.abyte.low  = cpu->BC.abyte.high;cpu->PC.aword++;return 4;break;   /*  01001000 LD C,B */
        case 0x49:  cpu->BC.abyte.low  = cpu->BC.abyte.low; cpu->PC.aword++;return 4;break;   /*  01001001 LD C,C */
        case 0x4a:  cpu->BC.abyte.low  = cpu->DE.abyte.high;cpu->PC.aword++;return 4;break;   /*  01001010 LD C,D */
        case 0x4b:  cpu->BC.abyte.low  = cpu->DE.abyte.low; cpu->PC.aword++;return 4;break;   /*  01001011 LD C,E */
        case 0x4c:  cpu->BC.abyte.low  = cpu->HL.abyte.high;cpu->PC.aword++;return 4;break;   /*  01001100 LD C,H */
        case 0x4d:  cpu->BC.abyte.low  = cpu->HL.abyte.low; cpu->PC.aword++;return 4;break;   /*  01001101 LD C,L */
        case 0x4f:  cpu->BC.abyte.low  = cpu->AF.abyte.high;cpu->PC.aword++;return 4;break;   /*  01001111 LD C,A */

        case 0x50:  cpu->DE.abyte.high = cpu->BC.abyte.high;cpu->PC.aword++;return 4;break;   /*  01010000 LD D,B */
        case 0x51:  cpu->DE.abyte.high = cpu->BC.abyte.low; cpu->PC.aword++;return 4;break;   /*  01010001 LD D,C */
        case 0x52:  cpu->DE.abyte.high = cpu->DE.abyte.high;cpu->PC.aword++;return 4;break;   /*  01010010 LD D,D */
        case 0x53:  cpu->DE.abyte.high = cpu->DE.abyte.low; cpu->PC.aword++;return 4;break;   /*  01010011 LD D,E */
        case 0x54:  cpu->DE.abyte.high = cpu->HL.abyte.high;cpu->PC.aword++;return 4;break;   /*  01010100 LD D,H */
        case 0x55:  cpu->DE.abyte.high = cpu->HL.abyte.low; cpu->PC.aword++;return 4;break;   /*  01010101 LD D,L */
        case 0x57:  cpu->DE.abyte.high = cpu->AF.abyte.high;cpu->PC.aword++;return 4;break;   /*  01010111 LD D,A */

        case 0x58:  cpu->DE.abyte.low  = cpu->BC.abyte.high;cpu->PC.aword++;return 4;break;   /*  01011000 LD E,B */
        case 0x59:  cpu->DE.abyte.low  = cpu->BC.abyte.low; cpu->PC.aword++;return 4;break;   /*  01011001 LD E,C */
        case 0x5a:  cpu->DE.abyte.low  = cpu->DE.abyte.high;cpu->PC.aword++;return 4;break;   /*  01011010 LD E,D */
        case 0x5b:  cpu->DE.abyte.low  = cpu->DE.abyte.low; cpu->PC.aword++;return 4;break;   /*  01011011 LD E,E */
        case 0x5c:  cpu->DE.abyte.low  = cpu->HL.abyte.high;cpu->PC.aword++;return 4;break;   /*  01011100 LD E,H */
        case 0x5d:  cpu->DE.abyte.low  = cpu->HL.abyte.low; cpu->PC.aword++;return 4;break;   /*  01011101 LD E,L */
        case 0x5f:  cpu->DE.abyte.low  = cpu->AF.abyte.high;cpu->PC.aword++;return 4;break;   /*  01011111 LD E,A */

        case 0x60:  cpu->HL.abyte.high = cpu->BC.abyte.high;cpu->PC.aword++;return 4;break;   /*  01100000 LD H,B */
        case 0x61:  cpu->HL.abyte.high = cpu->BC.abyte.low; cpu->PC.aword++;return 4;break;   /*  01100001 LD H,C */
        case 0x62:  cpu->HL.abyte.high = cpu->DE.abyte.high;cpu->PC.aword++;return 4;break;   /*  01100010 LD H,D */
        case 0x63:  cpu->HL.abyte.high = cpu->DE.abyte.low; cpu->PC.aword++;return 4;break;   /*  01100011 LD H,E */
        case 0x64:  cpu->HL.abyte.high = cpu->HL.abyte.high;cpu->PC.aword++;return 4;break;   /*  01100100 LD H,H */
        case 0x65:  cpu->HL.abyte.high = cpu->HL.abyte.low; cpu->PC.aword++;return 4;break;   /*  01100101 LD H,L */
        case 0x67:  cpu->HL.abyte.high = cpu->AF.abyte.high;cpu->PC.aword++;return 4;break;   /*  01100111 LD H,A */

        case 0x68:  cpu->HL.abyte.low  = cpu->BC.abyte.high;cpu->PC.aword++;return 4;break;   /*  01101000 LD L,B */
        case 0x69:  cpu->HL.abyte.low  = cpu->BC.abyte.low; cpu->PC.aword++;return 4;break;   /*  01101001 LD L,C */
        case 0x6a:  cpu->HL.abyte.low  = cpu->DE.abyte.high;cpu->PC.aword++;return 4;break;   /*  01101010 LD L,D */
        case 0x6b:  cpu->HL.abyte.low  = cpu->DE.abyte.low; cpu->PC.aword++;return 4;break;   /*  01101011 LD L,E */
        case 0x6c:  cpu->HL.abyte.low  = cpu->HL.abyte.high;cpu->PC.aword++;return 4;break;   /*  01101100 LD L,H */
        case 0x6d:  cpu->HL.abyte.low  = cpu->HL.abyte.low; cpu->PC.aword++;return 4;break;   /*  01101101 LD L,L */
        case 0x6f:  cpu->HL.abyte.low  = cpu->AF.abyte.high;cpu->PC.aword++;return 4;break;   /*  01101111 LD L,A */

        case 0x78:  cpu->AF.abyte.high = cpu->BC.abyte.high;cpu->PC.aword++;return 4;break;   /*  01111000 LD A,B */
        case 0x79:  cpu->AF.abyte.high = cpu->BC.abyte.low; cpu->PC.aword++;return 4;break;   /*  01111001 LD A,C */
        case 0x7a:  cpu->AF.abyte.high = cpu->DE.abyte.high;cpu->PC.aword++;return 4;break;   /*  01111010 LD A,D */
        case 0x7b:  cpu->AF.abyte.high = cpu->DE.abyte.low; cpu->PC.aword++;return 4;break;   /*  01111011 LD A,E */
        case 0x7c:  cpu->AF.abyte.high = cpu->HL.abyte.high;cpu->PC.aword++;return 4;break;   /*  01111100 LD A,H */
        case 0x7d:  cpu->AF.abyte.high = cpu->HL.abyte.low; cpu->PC.aword++;return 4;break;   /*  01111101 LD A,L */
        case 0x7f:  cpu->AF.abyte.high = cpu->AF.abyte.high;cpu->PC.aword++;return 4;break;   /*  01111111 LD A,A */


        /**** LD R8,n ****/
        case 0x06:  cpu->BC.abyte.high = read(cpu->PC.aword+1);cpu->PC.aword+=2;return 7;break;   /*  00000110 LD B,N */
        case 0x0e:  cpu->BC.abyte.low  = read(cpu->PC.aword+1);cpu->PC.aword+=2;return 7;break;   /*  00001110 LD C,N */
        case 0x16:  cpu->DE.abyte.high = read(cpu->PC.aword+1);cpu->PC.aword+=2;return 7;break;   /*  00010110 LD D,N */
        case 0x1e:  cpu->DE.abyte.low  = read(cpu->PC.aword+1);cpu->PC.aword+=2;return 7;break;   /*  00011110 LD E,N */
        case 0x26:  cpu->HL.abyte.high = read(cpu->PC.aword+1);cpu->PC.aword+=2;return 7;break;   /*  00100110 LD H,N */
        case 0x2e:  cpu->HL.abyte.low  = read(cpu->PC.aword+1);cpu->PC.aword+=2;return 7;break;   /*  00101110 LD L,N */
        case 0x3e:  cpu->AF.abyte.high = read(cpu->PC.aword+1);cpu->PC.aword+=2;return 7;break;   /*  00111110 LD A,N */


        /**** LD R8,(HL) ****/
        case 0x46:  cpu->BC.abyte.high = read(cpu->HL.aword);cpu->PC.aword++;return 7;break;  /*  01000110 LD B,(HL) */
        case 0x4e:  cpu->BC.abyte.low  = read(cpu->HL.aword);cpu->PC.aword++;return 7;break;  /*  01001110 LD C,(HL) */
        case 0x56:  cpu->DE.abyte.high = read(cpu->HL.aword);cpu->PC.aword++;return 7;break;  /*  01010110 LD D,(HL) */
        case 0x5e:  cpu->DE.abyte.low  = read(cpu->HL.aword);cpu->PC.aword++;return 7;break;  /*  01011110 LD E,(HL) */
        case 0x66:  cpu->HL.abyte.high = read(cpu->HL.aword);cpu->PC.aword++;return 7;break;  /*  01100110 LD H,(HL) */
        case 0x6e:  cpu->HL.abyte.low  = read(cpu->HL.aword);cpu->PC.aword++;return 7;break;  /*  01101110 LD L,(HL) */
        case 0x7e:  cpu->AF.abyte.high = read(cpu->HL.aword);cpu->PC.aword++;return 7;break;  /*  01111110 LD A,(HL) */


        /**** LD (HL),R8 ****/
        case 0x70:  write(cpu->HL.aword,cpu->BC.abyte.high);cpu->PC.aword++;return 7;break; /*  01110000 LD (HL),B */
        case 0x71:  write(cpu->HL.aword,cpu->BC.abyte.low); cpu->PC.aword++;return 7;break; /*  01110001 LD (HL),C */
        case 0x72:  write(cpu->HL.aword,cpu->DE.abyte.high);cpu->PC.aword++;return 7;break; /*  01110010 LD (HL),D */
        case 0x73:  write(cpu->HL.aword,cpu->DE.abyte.low); cpu->PC.aword++;return 7;break; /*  01110011 LD (HL),E */
        case 0x74:  write(cpu->HL.aword,cpu->HL.abyte.high);cpu->PC.aword++;return 7;break; /*  01110100 LD (HL),H */
        case 0x75:  write(cpu->HL.aword,cpu->HL.abyte.low); cpu->PC.aword++;return 7;break; /*  01110101 LD (HL),L */
        case 0x77:  write(cpu->HL.aword,cpu->AF.abyte.high);cpu->PC.aword++;return 7;break; /*  01110111 LD (HL),A */


        /**** PUSH qq ****/
        case 0xc5:  M_PUSH(BC);cpu->PC.aword++;return 11;break; /*  11000101 PUSH BC */
        case 0xd5:  M_PUSH(DE);cpu->PC.aword++;return 11;break; /*  11010101 PUSH DE */
        case 0xe5:  M_PUSH(HL);cpu->PC.aword++;return 11;break; /*  11100101 PUSH HL */
        case 0xf5:  M_PUSH(AF);cpu->PC.aword++;return 11;break; /*  11110101 PUSH AF */


        /**** POP qq ****/
        case 0xc1:  M_POP(BC);cpu->PC.aword++;return 10;break;    /*  11000001 POP BC */
        case 0xd1:  M_POP(DE);cpu->PC.aword++;return 10;break;    /*  11010001 POP DE */
        case 0xe1:  M_POP(HL);cpu->PC.aword++;return 10;break;    /*  11100001 POP HL */
        case 0xf1:  M_POP(AF);cpu->PC.aword++;return 10;break;    /*  11110001 POP AF */

        /**** JP ****/
        case 0xc3:  M_JP_nn;break;    /*  11000011 JP nn */
        case 0xc2:  M_JP_NZ_nn;break; /*  11000010 JP NZ,nn */
        case 0xca:  M_JP_Z_nn;break;  /*  11001010 JP  Z,nn */
        case 0xd2:  M_JP_NC_nn;break; /*  11010010 JP NC,nn */
        case 0xda:  M_JP_C_nn;break;  /*  11011010 JP  C,nn */
        case 0xe2:  M_JP_PO_nn;break; /*  11100010 JP PO,nn */
        case 0xea:  M_JP_PE_nn;break; /*  11101010 JP PE,nn */
        case 0xf2:  M_JP_P_nn;break;  /*  11110010 JP  P,nn */
        case 0xfa:  M_JP_N_nn;break;  /*  11111010 JP  N,nn */
        case 0xe9:  M_JP_HL;break;    /*  11101001 JP (HL) */


        /**** JR ****/
        case 0x18:  M_JR_e;break;   /*  00011000 JR e */
        case 0x20:  M_JR_NZ_e;break;  /*  00100000 JR NZ,e */
        case 0x28:  M_JR_Z_e;break;   /*  00101000 JR  Z,e */
        case 0x30:  M_JR_NC_e;break;  /*  00110000 JR NC,e */
        case 0x38:  M_JR_C_e;break;   /*  00111000 JR  C,e */


        case 0xf9:  cpu->SP.aword = cpu->HL.aword;cpu->PC.aword++;return 6;break; /*  11111001 LD SP,HL */

        case 0xcd:  M_CALL_nn;break;  /*  11001101 CALL nn */

        case 0xc4:  M_CALL_NZ_nn;break; /*  11000100 CALL NZ,nn */
        case 0xcc:  M_CALL_Z_nn;break;  /*  11001100 CALL  Z,nn */
        case 0xd4:  M_CALL_NC_nn;break; /*  11010100 CALL NC,nn */
        case 0xdc:  M_CALL_C_nn;break;  /*  11011100 CALL  C,nn */
        case 0xe4:  M_CALL_PO_nn;break; /*  11100100 CALL PO,nn */
        case 0xec:  M_CALL_PE_nn;break; /*  11101100 CALL PE,nn */
        case 0xf4:  M_CALL_P_nn;break;  /*  11110100 CALL  P,nn */
        case 0xfc:  M_CALL_N_nn;break;  /*  11111100 CALL  N,nn */

        case 0xc9:  M_RET;break;    /*  11001001 RET */

        case 0xc0:  M_RET_NZ;break;   /*  11000000 RET NZ */
        case 0xc8:  M_RET_Z;break;    /*  11001000 RET  Z */
        case 0xd0:  M_RET_NC;break;   /*  11010000 RET NC */
        case 0xd8:  M_RET_C;break;    /*  11011000 RET  C */
        case 0xe0:  M_RET_PO;break;   /*  11100000 RET PO */
        case 0xe8:  M_RET_PE;break;   /*  11101000 RET PE */
        case 0xf0:  M_RET_P;break;    /*  11110000 RET  P */
        case 0xf8:  M_RET_N;break;    /*  11111000 RET  N */

        case 0x10:  M_DJNZ_e;break;   /*  00010000 DJNZ e */

        case 0xdb:  cpu->AF.abyte.high = in (read(cpu->PC.aword+1));cpu->PC.aword+=2;return 11;break;   /*  11011011 IN A,(n) */
        case 0xd3:  out(read(cpu->PC.aword+1),cpu->AF.abyte.high);cpu->PC.aword+=2;return 11;break; /*  11010011 OUT (n),a */

        case 0x80:  M_ADD(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break; /*  ADD A,B */
        case 0x81:  M_ADD(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break; /*  ADD A,C */
        case 0x82:  M_ADD(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break; /*  ADD A,D */
        case 0x83:  M_ADD(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break; /*  ADD A,E */
        case 0x84:  M_ADD(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break; /*  ADD A,H */
        case 0x85:  M_ADD(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break; /*  ADD A,L */
        case 0x87:  M_ADD(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break; /*  ADD A,A */

        case 0x88:  M_ADC(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break; /*  ADC A,B */
        case 0x89:  M_ADC(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break; /*  ADC A,C */
        case 0x8a:  M_ADC(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break; /*  ADC A,D */
        case 0x8b:  M_ADC(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break; /*  ADC A,E */
        case 0x8c:  M_ADC(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break; /*  ADC A,H */
        case 0x8d:  M_ADC(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break; /*  ADC A;L */
        case 0x8f:  M_ADC(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break; /*  ADC A,A */

        case 0x90:  M_SUB(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break; /*  SUB A,B */
        case 0x91:  M_SUB(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break; /*  SUB A,C */
        case 0x92:  M_SUB(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break; /*  SUB A,D */
        case 0x93:  M_SUB(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break; /*  SUB A,E */
        case 0x94:  M_SUB(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break; /*  SUB A,H */
        case 0x95:  M_SUB(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break; /*  SUB A,L */
        case 0x97:  M_SUB(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break; /*  SUB A,A */

        case 0x98:  M_SBC(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break; /*  SBC A,B */
        case 0x99:  M_SBC(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break; /*  SBC A,C */
        case 0x9a:  M_SBC(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break; /*  SBC A,D */
        case 0x9b:  M_SBC(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break; /*  SBC A,E */
        case 0x9c:  M_SBC(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break; /*  SBC A,H */
        case 0x9d:  M_SBC(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break; /*  SBC A,L */
        case 0x9f:  M_SBC(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break; /*  SBC A,A */


        case 0xc6:  /*  ADD A,n */
          dw.abyte.low = read(cpu->PC.aword+1);
          M_ADD(dw.abyte.low);cpu->PC.aword+=2;return 7;break;
        case 0xce:  /*  ADC A,n */
          dw.abyte.low = read(cpu->PC.aword+1);
          M_ADC(dw.abyte.low);cpu->PC.aword+=2;return 7;break;
        case 0xd6:  /*  SUB A,n */
          dw.abyte.low = read(cpu->PC.aword+1);
          M_SUB(dw.abyte.low);cpu->PC.aword+=2;return 7;break;
        case 0xde:  /*  SBC A,n */
          dw.abyte.low = read(cpu->PC.aword+1);
          M_SBC(dw.abyte.low);cpu->PC.aword+=2;return 7;break;

        case 0x86:  /*  ADD A,(HL) */
          dw.abyte.low = read(cpu->HL.aword);
          M_ADD(dw.abyte.low);cpu->PC.aword++;return 7;break;
        case 0x8e:  /*  ADC A,(HL) */
          dw.abyte.low = read(cpu->HL.aword);
          M_ADC(dw.abyte.low);cpu->PC.aword++;return 7;break;
        case 0x96:  /*  SUB A,(HL) */
          dw.abyte.low = read(cpu->HL.aword);
          M_SUB(dw.abyte.low);cpu->PC.aword++;return 7;break;
        case 0x9e:  /*  SBC A,(HL) */
          dw.abyte.low = read(cpu->HL.aword);
          M_SBC(dw.abyte.low);cpu->PC.aword++;return 7;break;

        case 0xb8:  M_CP(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break;  /*  CP A,B */
        case 0xb9:  M_CP(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break;  /*  CP A,C */
        case 0xba:  M_CP(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break;  /*  CP A,D */
        case 0xbb:  M_CP(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break;  /*  CP A,E */
        case 0xbc:  M_CP(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break;  /*  CP A,H */
        case 0xbd:  M_CP(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break;  /*  CP A,L */
        case 0xbf:  M_CP(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break;  /*  CP A,A */

        case 0xfe:  /*  CP A,n */
          dw.abyte.low = read(cpu->PC.aword+1);
          M_CP(dw.abyte.low);cpu->PC.aword+=2;return 7;break;
        case 0xbe:  /*  CP A,(HL) */
          dw.abyte.low = read(cpu->HL.aword);
          M_CP(dw.abyte.low);cpu->PC.aword++;return 7;break;



        case 0xa0:  M_AND(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break; /*  AND A,B */
        case 0xa1:  M_AND(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break; /*  AND A,C */
        case 0xa2:  M_AND(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break; /*  AND A,D */
        case 0xa3:  M_AND(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break; /*  AND A,E */
        case 0xa4:  M_AND(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break; /*  AND A,H */
        case 0xa5:  M_AND(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break; /*  AND A,L */
        case 0xa7:  M_AND(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break; /*  AND A,A */

        case 0xe6:  /*  AND A,n */
          dw.abyte.low = read(cpu->PC.aword+1);
          M_AND(dw.abyte.low);cpu->PC.aword+=2;return 7;break;
        case 0xa6:  /*  AND A,(HL) */
          dw.abyte.low = read(cpu->HL.aword);
          M_AND(dw.abyte.low);cpu->PC.aword++;return 7;break;


        case 0xb0:  M_OR(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break;  /*  OR A,B */
        case 0xb1:  M_OR(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break;  /*  OR A,C */
        case 0xb2:  M_OR(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break;  /*  OR A,D */
        case 0xb3:  M_OR(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break;  /*  OR A,E */
        case 0xb4:  M_OR(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break;  /*  OR A,H */
        case 0xb5:  M_OR(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break;  /*  OR A,L */
        case 0xb7:  M_OR(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break;  /*  OR A,A */

        case 0xf6:  /*  OR A,n */
          dw.abyte.low = read(cpu->PC.aword+1);
          M_OR(dw.abyte.low);cpu->PC.aword+=2;return 7;break;
        case 0xb6:  /*  OR A,(HL) */
          dw.abyte.low = read(cpu->HL.aword);
          M_OR(dw.abyte.low);cpu->PC.aword++;return 7;break;


        case 0xa8:  M_XOR(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break; /*  XOR A,B */
        case 0xa9:  M_XOR(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break; /*  XOR A,C */
        case 0xaa:  M_XOR(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break; /*  XOR A,D */
        case 0xab:  M_XOR(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break; /*  XOR A,E */
        case 0xac:  M_XOR(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break; /*  XOR A,H */
        case 0xad:  M_XOR(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break; /*  XOR A,L */
        case 0xaf:  M_XOR(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break; /*  XOR A,A */

        case 0xee:  /*  XOR A,n */
          dw.abyte.low = read(cpu->PC.aword+1);
          M_XOR(dw.abyte.low);cpu->PC.aword+=2;return 7;break;
        case 0xae:  /*  XOR A,(HL) */
          dw.abyte.low = read(cpu->HL.aword);
          M_XOR(dw.abyte.low);cpu->PC.aword++;return 7;break;


        case 0x07:  /*  RLCA */
          dw.abyte.low = (cpu->AF.abyte.high & 0x80) >> 7;
          cpu->AF.abyte.high = (cpu->AF.abyte.high << 1) | dw.abyte.low;
          cpu->AF.abyte.low  = (cpu->AF.abyte.low & ~(FLAG_H|FLAG_N|FLAG_C)) | dw.abyte.low;
          cpu->PC.aword++;return 4;break;
        case 0x17:  /*  RLA */
          dw.abyte.low = (cpu->AF.abyte.high & 0x80) >> 7;
          cpu->AF.abyte.high = (cpu->AF.abyte.high << 1) | (cpu->AF.abyte.low & FLAG_C);
          cpu->AF.abyte.low  = (cpu->AF.abyte.low & ~(FLAG_H|FLAG_N|FLAG_C)) | dw.abyte.low;
          cpu->PC.aword++;return 4;break;
        case 0x0f:  /*  RRCA */
          dw.abyte.low = cpu->AF.abyte.high & FLAG_C;
          cpu->AF.abyte.high = (cpu->AF.abyte.high >> 1) | (dw.abyte.low << 7);
          cpu->AF.abyte.low  = (cpu->AF.abyte.low & ~(FLAG_H|FLAG_N|FLAG_C)) | dw.abyte.low;
          cpu->PC.aword++;return 4;break;
        case 0x1f:  /*  RRA */
          dw.abyte.low = cpu->AF.abyte.high & FLAG_C;
          cpu->AF.abyte.high = (cpu->AF.abyte.high >> 1) | ((cpu->AF.abyte.low & FLAG_C) << 7);
          cpu->AF.abyte.low  = (cpu->AF.abyte.low & ~(FLAG_H|FLAG_N|FLAG_C)) | dw.abyte.low;
          cpu->PC.aword++;return 4;break;


        case 0x09:  M_ADD16(HL,BC);cpu->PC.aword++;return 11;break; /*  ADD HL,BC */
        case 0x19:  M_ADD16(HL,DE);cpu->PC.aword++;return 11;break; /*  ADD HL,DE */
        case 0x29:  M_ADD16(HL,HL);cpu->PC.aword++;return 11;break; /*  ADD HL,HL */
        case 0x39:  M_ADD16(HL,SP);cpu->PC.aword++;return 11;break; /*  ADD HL,SP */

        case 0x03:  cpu->BC.aword++;cpu->PC.aword++;return 6;break; /*  INC BC */
        case 0x13:  cpu->DE.aword++;cpu->PC.aword++;return 6;break; /*  INC DE */
        case 0x23:  cpu->HL.aword++;cpu->PC.aword++;return 6;break; /*  INC HL */
        case 0x33:  cpu->SP.aword++;cpu->PC.aword++;return 6;break; /*  INC SP */

        case 0x0b:  cpu->BC.aword--;cpu->PC.aword++;return 6;break; /*  DEC BC */
        case 0x1b:  cpu->DE.aword--;cpu->PC.aword++;return 6;break; /*  DEC DE */
        case 0x2b:  cpu->HL.aword--;cpu->PC.aword++;return 6;break; /*  DEC HL */
        case 0x3b:  cpu->SP.aword--;cpu->PC.aword++;return 6;break; /*  DEC SP */


        case 0x04:  M_INC(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break; /*  INC B */
        case 0x0c:  M_INC(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break; /*  INC C */
        case 0x14:  M_INC(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break; /*  INC D */
        case 0x1c:  M_INC(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break; /*  INC E */
        case 0x24:  M_INC(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break; /*  INC H */
        case 0x2c:  M_INC(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break; /*  INC L */
        case 0x3c:  M_INC(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break; /*  INC A */
        case 0x34:  dw.abyte.low = read(cpu->HL.aword);   /*  INC (HL) */
      M_INC(dw.abyte.low);
      write(cpu->HL.aword, dw.abyte.low);
      cpu->PC.aword++;return 11;break;

        case 0x05:  M_DEC(cpu->BC.abyte.high);cpu->PC.aword++;return 4;break; /*  DEC B */
        case 0x0d:  M_DEC(cpu->BC.abyte.low );cpu->PC.aword++;return 4;break; /*  DEC C */
        case 0x15:  M_DEC(cpu->DE.abyte.high);cpu->PC.aword++;return 4;break; /*  DEC D */
        case 0x1d:  M_DEC(cpu->DE.abyte.low );cpu->PC.aword++;return 4;break; /*  DEC E */
        case 0x25:  M_DEC(cpu->HL.abyte.high);cpu->PC.aword++;return 4;break; /*  DEC H */
        case 0x2d:  M_DEC(cpu->HL.abyte.low );cpu->PC.aword++;return 4;break; /*  DEC L */
        case 0x3d:  M_DEC(cpu->AF.abyte.high);cpu->PC.aword++;return 4;break; /*  DEC A */
        case 0x35:  dw.abyte.low = read(cpu->HL.aword);   /*  DEC (HL) */
      M_DEC(dw.abyte.low);
      write(cpu->HL.aword, dw.abyte.low);
      cpu->PC.aword++;return 11;break;


        case 0xeb:  M_EX(DE,HL);cpu->PC.aword++;return 4;break;   /*  EX DE,HL */
        case 0x08:  M_EX(AF,af);cpu->PC.aword++;return 4;break;   /*  EX AF,AF' */
        case 0xd9:  M_EX(BC,bc);M_EX(DE,de);M_EX(HL,hl);cpu->PC.aword++;return 4;break; /*  EXX */
        case 0xe3:  dw.abyte.low  = read(cpu->SP.aword);    /*  EX (SP),HL */
      dw.abyte.high = read(cpu->SP.aword+1);
      write(cpu->SP.aword, cpu->HL.abyte.low);
      write(cpu->SP.aword+1, cpu->HL.abyte.high);
      cpu->HL.aword = dw.aword;
      cpu->PC.aword++;return 19;break;

        case 0xc7:  cpu->PC.aword++;PUSH_PC;cpu->PC.aword = 0x00;return 11;break; /*  RST 00h */
        case 0xcf:  cpu->PC.aword++;PUSH_PC;cpu->PC.aword = 0x08;return 11;break; /*  RST 08h */
        case 0xd7:  cpu->PC.aword++;PUSH_PC;cpu->PC.aword = 0x10;return 11;break; /*  RST 10h */
        case 0xdf:  cpu->PC.aword++;PUSH_PC;cpu->PC.aword = 0x18;return 11;break; /*  RST 18h */
        case 0xe7:  cpu->PC.aword++;PUSH_PC;cpu->PC.aword = 0x20;return 11;break; /*  RST 20h */
        case 0xef:  cpu->PC.aword++;PUSH_PC;cpu->PC.aword = 0x28;return 11;break; /*  RST 28h */
        case 0xf7:  cpu->PC.aword++;PUSH_PC;cpu->PC.aword = 0x30;return 11;break; /*  RST 30h */
        case 0xff:  cpu->PC.aword++;PUSH_PC;cpu->PC.aword = 0x38;return 11;break; /*  RST 38h */

        case 0x2f:  cpu->AF.abyte.high = ~cpu->AF.abyte.high;     /*  CPL */
      cpu->AF.abyte.low = (cpu->AF.abyte.low&~(FLAG_3|FLAG_5)) |
             FLAG_H|FLAG_N |
             (cpu->AF.abyte.high&(FLAG_3|FLAG_5));
      cpu->PC.aword++;return 4;break;

        case 0x3f:  /*  CCF */
      cpu->AF.abyte.low = (cpu->AF.abyte.low&~(FLAG_N|FLAG_H|FLAG_3|FLAG_5)) |
             ((cpu->AF.abyte.low&FLAG_C)<<4) |
             (cpu->AF.abyte.high&(FLAG_3|FLAG_5));
      cpu->AF.abyte.low ^= FLAG_C;
      cpu->PC.aword++;return 4;break;
        case 0x37:  /*  SCF */
      cpu->AF.abyte.low = (cpu->AF.abyte.low & ~(FLAG_N|FLAG_H|FLAG_3|FLAG_5)) |
             (cpu->AF.abyte.high&(FLAG_3|FLAG_5)) |
             FLAG_C;
      cpu->PC.aword++;return 4;break;

        case 0x00:  cpu->PC.aword++;return 4;break;       /*  NOP */

        case 0xf3:  cpu->IFF &= 0xfc;cpu->PC.aword++;return 4;break;    /*  DI */
        case 0xfb:  cpu->IFF |= 0x40;cpu->eicount=1;cpu->PC.aword++;return 4;break;   /*  EI */
        case 0x76:  cpu->IFF |= 0x80;return 4;break;    /*  HALT */

        case 0x27:                /*  DAA */
          dw.aword = (cpu->AF.abyte.high | (((cpu->AF.abyte.low&0x03)|((cpu->AF.abyte.low&0x10)>>2))<<8))<<1;
          cpu->AF.abyte.high = DAATable[dw.aword];
          cpu->AF.abyte.low  = DAATable[dw.aword+1];
          cpu->PC.aword++;return 4;break;

        case 0xed:
          return Z80_Decode_ED ( cpu );
          break;

        case 0xdd:
          return Z80_Decode_DD ( cpu );
          break;

        case 0xfd:
          return Z80_Decode_FD ( cpu );
          break;

        case 0xcb:
          return Z80_Decode_CB ( cpu );
          break;

        default:
          return Z80_IllegalInstruction(cpu);
          break;
      }
/*     return 4; */
  }



/*  Create an interrupt */
void Z80_Int ( Z80 * cpu, u16 vector )
  {
    if ( vector==Z80_NMI )
      cpu->IFF |= 0x20;   /*  NMI */
    else /* if ( cpu->IFF&1 ) */
      cpu->IFF |= 0x10;   /*  INT */
  }


int Z80_ExecInt ( Z80 * cpu )
  {
    /*  Any pending interrupts ? */
    if ( cpu->IFF & 0x20 ) /*  NMI */
      {
        /*  Has the cpu been halted ? */
        if ( cpu->IFF&0x80 )
          {
            cpu->IFF &= ~0x80;
            cpu->PC.aword++;
          }
        /*  ** Reset bit5 and bit1=bit0 */
        cpu->IFF = (cpu->IFF & 0xdc) | ((cpu->IFF & 0x01) << 1);
/*         cpu->IFF &= ~0x01; */
        /*  ** Now restart at 0x66 */
        /*  Push PC onto the stack */
        write(cpu->SP.aword-1, cpu->PC.abyte.high);
        write(cpu->SP.aword-2, cpu->PC.abyte.low);
        cpu->SP.aword -= 2;
        /*  JMP to 0x0066 */
        cpu->PC.aword = 0x66;

        cpu->cyclecount = 11;

        return 11;
      } else
    if ( (cpu->IFF & 0x10) && (cpu->IFF & 1) ) /*  INT */
      {
        /*  Has the cpu been halted ? */
        if ( cpu->IFF&0x80 )
          {
            cpu->IFF &= ~0x80;
            cpu->PC.aword++;
          }
        cpu->IFF &= ~0x10;
        cpu->IFF &= ~0x03; /*  Disable interrupts */
        /*  ** Now restart at 0x38 */
        /*  Push PC onto the stack */
        write(cpu->SP.aword-1, cpu->PC.abyte.high);
        write(cpu->SP.aword-2, cpu->PC.abyte.low);
        cpu->SP.aword -= 2;
        /*  JMP to 0x0038 */
        cpu->PC.aword = 0x38;

        cpu->cyclecount = 11;

        return 11;
      }
    return -1;
  }

int Z80_Disassemble ( u8 * op, char * dest )
  {
    char * inst;
    unsigned int i,j,l,ofs=0;

    switch ( op[0] )
      {
        case 0xcb:
          inst = Z80_DisasmCB[op[1]];
          l    = Z80_LengthCB[op[1]];
          break;
        case 0xdd:
          if (op[1]==0xcb)
            {
              inst = Z80_DisasmDDCB[op[3]];
              l    = Z80_LengthDDCB[op[3]];
            }
          else
            {
              inst = Z80_DisasmDD[op[1]];
              l    = Z80_LengthDD[op[1]];
              if (inst==NULL)
               {
                 ofs=1;
                 inst = Z80_Disasm[op[1]];
                 l    = Z80_Length[op[1]]+1;
               }
            }
          break;
        case 0xed:
          inst = Z80_DisasmED[op[1]];
          l    = Z80_LengthED[op[1]];
          break;
        case 0xfd:
          if (op[1]==0xcb)
            {
              inst = Z80_DisasmFDCB[op[3]];
              l    = Z80_LengthFDCB[op[3]];
            }
          else
            {
              inst = Z80_DisasmFD[op[1]];
              l    = Z80_LengthFD[op[1]];
              if (inst==NULL)
                {
                  ofs=1;
                  inst = Z80_Disasm[op[1]];
                  l    = Z80_Length[op[1]]+1;
                }
            }
          break;
        default:
          inst = Z80_Disasm[op[0]];
          l    = Z80_Length[op[0]];
          break;
      }

    if ( inst==NULL )
      {
        sprintf ( dest, "DB %02Xh", op[0] );
        return 1;
      }

    for (i=j=0;i<strlen(inst);)
      {
        if ( ((inst[i]=='%') || (inst[i]=='$')) &&
             (inst[i+1]>='0') && (inst[i+1]<='9') )
          {
            char t[8];
            u8 r;
            u16 v;

            v = inst[i+1]-'0';
            r = op[v+ofs];

            if ( inst[i]=='$' )
              {
                if (r&0x80)
                  sprintf(t,"-%02X",(-r)&0xff);
                else
                  sprintf(t,"+%02X",r);
              }
            else
              sprintf(t,"%02X",r);

            strcpy(&dest[j],t);
            j+=strlen(t);
            i+=2;
          } else
          {
            dest[j++] = inst[i++];
          }
      }

    dest[j] = '\0';
    return l;
  }

/*  Execute a single instruction */
int Z80_Exec ( Z80 * cpu )
  {
    cpu->R++;
/* cpu->R = (cpu->R + 1) & 0x7f ; */
/*
#ifdef GP32
if (debug)
{
  union_word aw; u8 t;
  u8 bla[4];
  char instr[32];
  char mess[128];
  int j,k,l;
  GGMS * machine = cpu->val;

            u8 i = (((u8)machine->vdp->frame_irq&1) << 7) |
                   (((u8)machine->vdp->line_irq&1)<<6) |
                   (((u8)machine->vdp->collision&1)<<5);

  bla[0] = cpu->readMem(cpu,cpu->PC.aword+0);
  bla[1] = cpu->readMem(cpu,cpu->PC.aword+1);
  bla[2] = cpu->readMem(cpu,cpu->PC.aword+2);
  bla[3] = cpu->readMem(cpu,cpu->PC.aword+3);
  Z80_Disassemble(bla,instr);

  aw.aword = 8;
  t=0xff;
  aw.aword += t | (t&0x80?0xff00:0x0000);
for (l=0;l<48;++l)
  {
    k=239-l;
    for (j=0;j<320;j++,k+=240)
      gpDraw[0].ptbuffer[k] = 0;
  }
sprintf(mess, "%06x - %s\n", GGMS_Addy2ROM((GGMS*)cpu->val,cpu->PC.aword),instr);
GpTextOut(NULL, &gpDraw[0], 0, 0, mess, 128);
sprintf(mess, "AF=%04x BC=%04x DE=%04x HL=%04x vc=%d", cpu->AF.aword, cpu->BC.aword, cpu->DE.aword, cpu->HL.aword, machine->vdp->vcount);
GpTextOut(NULL, &gpDraw[0], 0, 16, mess, 128);
sprintf(mess, "IX=%04x IY=%04x SP=%04x %04x", cpu->IX.aword, cpu->IX.aword, cpu->SP.aword, aw.aword);
GpTextOut(NULL, &gpDraw[0], 0, 32, mess, 128);
while (!(k=GpKeyGet()));
while (GpKeyGet());
if (k&GPC_VK_SELECT) debug=0;
}
#else
if (debug)
  {
  u8 bla[4];
  char instr[32];
  char mess[128];
  int j,k,l;
  GGMS * machine = cpu->val;

            u8 i = (((u8)machine->vdp->frame_irq&1) << 7) |
                   (((u8)machine->vdp->line_irq&1)<<6) |
                   (((u8)machine->vdp->collision&1)<<5);

  bla[0] = cpu->readMem(cpu,cpu->PC.aword+0);
  bla[1] = cpu->readMem(cpu,cpu->PC.aword+1);
  bla[2] = cpu->readMem(cpu,cpu->PC.aword+2);
  bla[3] = cpu->readMem(cpu,cpu->PC.aword+3);
  Z80_Disassemble(bla,instr);

  sprintf (mess, "%06x - %s - AF=%04x BC=%04x DE=%04x HL=%04x IX=%04x IY=%04x SP=%04x\n",
  GGMS_Addy2ROM((GGMS*)cpu->val, cpu->PC.aword), instr,cpu->AF.aword, cpu->BC.aword, cpu->DE.aword, cpu->HL.aword, cpu->IX.aword, cpu->IY.aword, cpu->SP.aword);    
  printf(mess);
  }
#endif
*/
/*     cpu->rdy = TRUE; */
/*     cpu->R = (cpu->R + 1) & 0x7f ; */

    if ( cpu->IFF&0x40 )
      {
        cpu->IFF &= ~0x40;
/*         cpu->IFF |= 0x03; */
      }

    if ( cpu->eicount > 0 )
      cpu->eicount--;
    else if ( cpu->eicount == 0 )
      {
        cpu->IFF |= 0x03;
        cpu->eicount=-1;
      }

    /*  If the last instruction was an EI, */
    /*  set IFF1 and IFF2 to 1 */
    if ( cpu->IFF&0x80 )
      {
        cpu->cyclecount = 4;
        return 4;
      }

    /*  Execute current instruction */
/*     cpu->cyclecount = Z80_Decode ( cpu ); */
/*     return cpu->cyclecount; */

    cpu->cyclecount = Z80_Decode(cpu);

    return (cpu->cyclecount);
  }


/*  One CLK tick */
void Z80_Cycle ( Z80 * cpu )
  {
    cpu->rdy = FALSE;

    if ( cpu->cyclecount == 0 )
      {
        /*  It's the very first cycle */
        if (Z80_ExecInt(cpu)!=-1) return;
        Z80_Exec(cpu);
      }
    else
      {
        cpu->cyclecount--;
        if ( cpu->cyclecount == 0 )
          {
            if (Z80_ExecInt(cpu)!=-1) return;
            Z80_Exec(cpu);
          }
      }
  }



int Z80_Run ( Z80 * cpu, int c )  /*  Run at least c cycles */
  {
    int i=0,x=0;

/*     if ((x=Z80_ExecInt(cpu))!=-1) i+=x; */
    while ( i<c )
     {
       x = Z80_ExecInt ( cpu );
      if (x!=-1)
        i += x;
      else
        i += Z80_Exec(cpu);
     }

    return x;
  }

