/* Z80.h */

#ifndef __Z80_H__
#define __Z80_H__

#include "../../defines.h"

extern const u8 SZTable[];
extern const u8 PSZTable[];

#define FLAG_C    0x01
#define FLAG_N    0x02
#define FLAG_PV   0x04
#define FLAG_3    0x08
#define FLAG_H    0x10
#define FLAG_5    0x20
#define FLAG_Z    0x40
#define FLAG_S    0x80


#define Z80_NMI   0x66


typedef struct
  {
    union_word AF,BC,DE,HL,SP,PC,IX,IY;
    union_word af,bc,de,hl;
    u8 I,R;
    u8 IFF;
    int cyclecount;
    BOOL rdy;   /* Finished executing an instruction ? */
    int (*curInst)(void*);
    u8 (*readMem)(void*,u16);
    void (*writeMem)(void*,u16,u8);
    void (*out)(void*,u8,u8);
    u8 (*in)(void*,u8);
    u16 vec;
    u8 data[4];   /* Data from PC */
    void * val;
    int eicount;
  } Z80;

void Z80_Init ( Z80 * , u8 (*)(Z80*,u16), void (*)(Z80*,u16,u8), u8 (*)(Z80*,u8), void (*)(Z80*,u8,u8) );
void Z80_Terminate ( Z80 * );
void Z80_Int ( Z80 * , u16 );
void Z80_Cycle ( Z80 * );
void Z80_Reset ( Z80 * );
int  Z80_Disassemble ( u8 *, char * );
int Z80_Run ( Z80 *, int );

#endif

