/* Z80_macros.h */

#ifndef __Z80_MACROS_H__
#define __Z80_MACROS_H__

#include "Z80.h"
#include "../defines.h"

#define BYTE2WORD(a)  ((u16)(s16)(s8)a)

#define read(a)   (cpu->readMem(cpu,a))
#define read16(a,b) b.abyte.low=read(a);b.abyte.high=read(a+1);
#define write(a,b)  (cpu->writeMem((Z80*)cpu,a,b))
#define write16(a,b)  write(a,b.abyte.low);write(a+1,b.abyte.high);
#define in(a)   (cpu->in((Z80*)cpu,a))
#define out(a,b)  (cpu->out((Z80*)cpu,a,b))



#define M_PUSH(a)\
    write(cpu->SP.aword-1, cpu->a.abyte.high);\
    write(cpu->SP.aword-2, cpu->a.abyte.low );\
    cpu->SP.aword-=2;

#define M_POP(a)\
    cpu->a.abyte.low  = read(cpu->SP.aword+0);\
    cpu->a.abyte.high = read(cpu->SP.aword+1);\
    cpu->SP.aword+=2;



#define M_READ_PC(c,d)\
    (read(c->PC.aword+d))

#define M_READ_PC16(c,d)\
    (M_READ_PC(c,d) | (M_READ_PC(c,d+1)<<8))

#define M_READ_IX_d(d)\
    (read(cpu->IX.aword+BYTE2WORD(d)))

#define M_WRITE_IX_d(d,a)\
    (write(cpu->IX.aword+BYTE2WORD(d),a)))

#define M_READ_IY_d(d)\
    (read(cpu->IY.aword+BYTE2WORD(d)))

#define M_WRITE_IY_d(d,a)\
    (write(cpu->IY.aword+BYTE2WORD(d),a)))



#define PUSH_PC   write(cpu->SP.aword-2, cpu->PC.abyte.low );\
      write(cpu->SP.aword-1, cpu->PC.abyte.high);\
      cpu->SP.aword-=2;
#define POP_PC    cpu->PC.abyte.low  = read(cpu->SP.aword++);\
      cpu->PC.abyte.high = read(cpu->SP.aword++);






#define M_LD_B_B  cpu->PC.aword++;return 4;
#define M_LD_B_C  cpu->BC.abyte.high = cpu->BC.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_B_D  cpu->BC.abyte.high = cpu->DE.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_B_E  cpu->BC.abyte.high = cpu->DE.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_B_H  cpu->BC.abyte.high = cpu->HL.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_B_L  cpu->BC.abyte.high = cpu->HL.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_B_A  cpu->BC.abyte.high = cpu->AF.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_C_B  cpu->BC.abyte.low  = cpu->BC.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_C_C  cpu->PC.aword++;return 4;
#define M_LD_C_D  cpu->BC.abyte.low  = cpu->DE.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_C_E  cpu->BC.abyte.low  = cpu->DE.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_C_H  cpu->BC.abyte.low  = cpu->HL.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_C_L  cpu->BC.abyte.low  = cpu->HL.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_C_A  cpu->BC.abyte.low  = cpu->AF.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_D_B  cpu->DE.abyte.high = cpu->BC.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_D_C  cpu->DE.abyte.high = cpu->BC.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_D_D  cpu->PC.aword++;return 4;
#define M_LD_D_E  cpu->DE.abyte.high = cpu->DE.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_D_H  cpu->DE.abyte.high = cpu->HL.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_D_L  cpu->DE.abyte.high = cpu->HL.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_D_A  cpu->DE.abyte.high = cpu->AF.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_E_B  cpu->DE.abyte.low  = cpu->BC.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_E_C  cpu->DE.abyte.low  = cpu->BC.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_E_D  cpu->DE.abyte.low  = cpu->DE.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_E_E  cpu->PC.aword++;return 4;
#define M_LD_E_H  cpu->DE.abyte.low  = cpu->HL.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_E_L  cpu->DE.abyte.low  = cpu->HL.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_E_A  cpu->DE.abyte.low  = cpu->AF.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_H_B  cpu->HL.abyte.high = cpu->BC.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_H_C  cpu->HL.abyte.high = cpu->BC.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_H_D  cpu->HL.abyte.high = cpu->DE.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_H_E  cpu->HL.abyte.high = cpu->DE.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_H_H  cpu->PC.aword++;return 4;
#define M_LD_H_L  cpu->HL.abyte.high = cpu->HL.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_H_A  cpu->HL.abyte.high = cpu->AF.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_L_B  cpu->HL.abyte.low  = cpu->BC.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_L_C  cpu->HL.abyte.low  = cpu->BC.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_L_D  cpu->HL.abyte.low  = cpu->DE.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_L_E  cpu->HL.abyte.low  = cpu->DE.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_L_H  cpu->HL.abyte.low  = cpu->HL.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_L_L  cpu->PC.aword++;return 4;
#define M_LD_L_A  cpu->HL.abyte.low  = cpu->AF.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_A_B  cpu->AF.abyte.high = cpu->BC.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_A_C  cpu->AF.abyte.high = cpu->BC.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_A_D  cpu->AF.abyte.high = cpu->DE.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_A_E  cpu->AF.abyte.high = cpu->DE.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_A_H  cpu->AF.abyte.high = cpu->HL.abyte.high; cpu->PC.aword++; return 4;
#define M_LD_A_L  cpu->AF.abyte.high = cpu->HL.abyte.low;  cpu->PC.aword++; return 4;
#define M_LD_A_A  cpu->PC.aword++;return 4;



/**** LD R8,n ****/
#define M_LD_B_n  cpu->BC.abyte.high = M_READ_PC(cpu,1); cpu->PC.aword+=2; return 7;
#define M_LD_C_n  cpu->BC.abyte.low  = M_READ_PC(cpu,1); cpu->PC.aword+=2; return 7;
#define M_LD_D_n  cpu->DE.abyte.high = M_READ_PC(cpu,1); cpu->PC.aword+=2; return 7;
#define M_LD_E_n  cpu->DE.abyte.low  = M_READ_PC(cpu,1); cpu->PC.aword+=2; return 7;
#define M_LD_H_n  cpu->HL.abyte.high = M_READ_PC(cpu,1); cpu->PC.aword+=2; return 7;
#define M_LD_L_n  cpu->HL.abyte.low  = M_READ_PC(cpu,1); cpu->PC.aword+=2; return 7;
#define M_LD_A_n  cpu->AF.abyte.high = M_READ_PC(cpu,1); cpu->PC.aword+=2; return 7;


/**** LD R8,(HL) ****/
#define M_LD_B_HL cpu->BC.abyte.high = read(cpu->HL.aword); cpu->PC.aword++; return 7;
#define M_LD_C_HL cpu->BC.abyte.low  = read(cpu->HL.aword); cpu->PC.aword++; return 7;
#define M_LD_D_HL cpu->DE.abyte.high = read(cpu->HL.aword); cpu->PC.aword++; return 7;
#define M_LD_E_HL cpu->DE.abyte.low  = read(cpu->HL.aword); cpu->PC.aword++; return 7;
#define M_LD_H_HL cpu->HL.abyte.high = read(cpu->HL.aword); cpu->PC.aword++; return 7;
#define M_LD_L_HL cpu->HL.abyte.low  = read(cpu->HL.aword); cpu->PC.aword++; return 7;
#define M_LD_A_HL cpu->AF.abyte.high = read(cpu->HL.aword); cpu->PC.aword++; return 7;


/**** LD (HL),R8 ****/
#define M_LD_HL_B write(cpu->HL.aword, cpu->BC.abyte.high); cpu->PC.aword++; return 7;
#define M_LD_HL_C write(cpu->HL.aword, cpu->BC.abyte.low ); cpu->PC.aword++; return 7;
#define M_LD_HL_D write(cpu->HL.aword, cpu->DE.abyte.high); cpu->PC.aword++; return 7;
#define M_LD_HL_E write(cpu->HL.aword, cpu->DE.abyte.low ); cpu->PC.aword++; return 7;
#define M_LD_HL_H write(cpu->HL.aword, cpu->HL.abyte.high); cpu->PC.aword++; return 7;
#define M_LD_HL_L write(cpu->HL.aword, cpu->HL.abyte.low ); cpu->PC.aword++; return 7;
#define M_LD_HL_A write(cpu->HL.aword, cpu->AF.abyte.high); cpu->PC.aword++; return 7;


/**** PUSH qq ****/
#define M_PUSH_BC write(cpu->SP.aword-2, cpu->BC.abyte.low );\
      write(cpu->SP.aword-1, cpu->BC.abyte.high);\
      cpu->SP.aword-=2;cpu->PC.aword++;\
      return 11;
#define M_PUSH_DE write(cpu->SP.aword-2, cpu->DE.abyte.low );\
      write(cpu->SP.aword-1, cpu->DE.abyte.high);\
      cpu->SP.aword-=2;cpu->PC.aword++;\
      return 11;
#define M_PUSH_HL write(cpu->SP.aword-2, cpu->HL.abyte.low );\
      write(cpu->SP.aword-1, cpu->HL.abyte.high);\
      cpu->SP.aword-=2;cpu->PC.aword++;\
      return 11;
#define M_PUSH_AF write(cpu->SP.aword-2, cpu->AF.abyte.low );\
      write(cpu->SP.aword-1, cpu->AF.abyte.high);\
      cpu->SP.aword-=2;cpu->PC.aword++;\
      return 11;


/**** POP qq ****/
#define M_POP_BC  cpu->BC.abyte.low  = read(cpu->SP.aword++);\
      cpu->BC.abyte.high = read(cpu->SP.aword++);\
                        cpu->PC.aword++;\
                        return 10;
#define M_POP_DE  cpu->DE.abyte.low  = read(cpu->SP.aword++);\
      cpu->DE.abyte.high = read(cpu->SP.aword++);\
                        cpu->PC.aword++;\
                        return 10;
#define M_POP_HL  cpu->HL.abyte.low  = read(cpu->SP.aword++);\
      cpu->HL.abyte.high = read(cpu->SP.aword++);\
                        cpu->PC.aword++;\
                        return 10;
#define M_POP_AF  cpu->AF.abyte.low  = read(cpu->SP.aword++);\
      cpu->AF.abyte.high = read(cpu->SP.aword++);\
                        cpu->PC.aword++;\
                        return 10;


/**** LD dd,nn ****/
#define M_LD_BC_nn  cpu->PC.aword++;\
      cpu->BC.abyte.low  = read(cpu->PC.aword++);\
      cpu->BC.abyte.high = read(cpu->PC.aword++);\
      return 10;
#define M_LD_DE_nn  cpu->PC.aword++;\
      cpu->DE.abyte.low  = read(cpu->PC.aword++);\
      cpu->DE.abyte.high = read(cpu->PC.aword++);\
      return 10;
#define M_LD_HL_nn  cpu->PC.aword++;\
      cpu->HL.abyte.low  = read(cpu->PC.aword++);\
      cpu->HL.abyte.high = read(cpu->PC.aword++);\
      return 10;
#define M_LD_SP_nn  cpu->PC.aword++;\
      cpu->SP.abyte.low  = read(cpu->PC.aword++);\
      cpu->SP.abyte.high = read(cpu->PC.aword++);\
      return 10;



#define M_LD_mm_HL  cpu->PC.aword++;\
      t.aword.low.abyte.low  = read(cpu->PC.aword++);\
      t.aword.low.abyte.high = read(cpu->PC.aword++);\
      write(t.aword.low.aword++, cpu->HL.abyte.low );\
      write(t.aword.low.aword  , cpu->HL.abyte.high);\
      return 16;
#define M_LD_HL_mm  cpu->PC.aword++;\
      t.aword.low.abyte.low  = read(cpu->PC.aword++);\
      t.aword.low.abyte.high = read(cpu->PC.aword++);\
      cpu->HL.abyte.low  = read(t.aword.low.aword++);\
      cpu->HL.abyte.high = read(t.aword.low.aword  );\
      return 16;



#define M_LD_BC_A write(cpu->BC.aword, cpu->AF.abyte.high); cpu->PC.aword++; return 7;
#define M_LD_DE_A write(cpu->DE.aword, cpu->AF.abyte.high); cpu->PC.aword++; return 7;
#define M_LD_mm_A cpu->PC.aword++;\
      t.aword.low.abyte.low  = read(cpu->PC.aword++);\
      t.aword.low.abyte.high = read(cpu->PC.aword++);\
      write(t.aword.low.aword, cpu->AF.abyte.high);\
      return 13;
#define M_LD_A_BC cpu->AF.abyte.high = read(cpu->BC.aword); cpu->PC.aword++; return 7;
#define M_LD_A_DE cpu->AF.abyte.high = read(cpu->DE.aword); cpu->PC.aword++; return 7;
#define M_LD_A_mm cpu->PC.aword++;\
      t.aword.low.abyte.low  = read(cpu->PC.aword++);\
      t.aword.low.abyte.high = read(cpu->PC.aword++);\
      cpu->AF.abyte.high = read(t.aword.low.aword);\
      return 13;

#define M_LD_HL_n write(cpu->HL.aword, read(cpu->PC.aword+1)); cpu->PC.aword+=2;return 10;

#define M_LD_SP_HL  cpu->SP.aword = cpu->HL.aword; cpu->PC.aword++; return 6;



/**** JP ****/
#define M_JP_nn   cpu->PC.aword++;\
      t.aword.low.abyte.low  = read(cpu->PC.aword++);\
      t.aword.low.abyte.high = read(cpu->PC.aword++);\
      cpu->PC.aword = t.aword.low.aword;\
      return 10;
#define M_JP_NZ_nn  if ( cpu->AF.abyte.low & FLAG_Z ) { cpu->PC.aword+=3; return 10; } else { M_JP_nn; }
#define M_JP_Z_nn if ( cpu->AF.abyte.low & FLAG_Z ) { M_JP_nn; } else { cpu->PC.aword+=3; return 10; }
#define M_JP_NC_nn  if ( cpu->AF.abyte.low & FLAG_C ) { cpu->PC.aword+=3; return 10; } else { M_JP_nn; }
#define M_JP_C_nn if ( cpu->AF.abyte.low & FLAG_C ) { M_JP_nn; } else { cpu->PC.aword+=3; return 10; }
#define M_JP_PO_nn  if ( cpu->AF.abyte.low & FLAG_PV) { cpu->PC.aword+=3; return 10; } else { M_JP_nn; }
#define M_JP_PE_nn  if ( cpu->AF.abyte.low & FLAG_PV) { M_JP_nn; } else { cpu->PC.aword+=3; return 10; }
#define M_JP_P_nn if ( cpu->AF.abyte.low & FLAG_S ) { cpu->PC.aword+=3; return 10; } else { M_JP_nn; }
#define M_JP_N_nn if ( cpu->AF.abyte.low & FLAG_S ) { M_JP_nn; } else { cpu->PC.aword+=3; return 10; }
#define M_JP_HL   cpu->PC.aword = cpu->HL.aword; return 4;


/**** JR ****/
#define JR_e    cpu->PC.aword += BYTE2WORD(read(cpu->PC.aword+1))+2;
#define M_JR_e    JR_e; return 12;
#define M_JR_C_e  if ( cpu->AF.abyte.low & FLAG_C ) { M_JR_e; } else { cpu->PC.aword+=2; return 7; }
#define M_JR_NC_e if ( cpu->AF.abyte.low & FLAG_C ) { cpu->PC.aword+=2; return 7; } else { M_JR_e; }
#define M_JR_Z_e  if ( cpu->AF.abyte.low & FLAG_Z ) { M_JR_e; } else { cpu->PC.aword+=2; return 7; }
#define M_JR_NZ_e if ( cpu->AF.abyte.low & FLAG_Z ) { cpu->PC.aword+=2; return 7; } else { M_JR_e; }


#define M_CALL_nn t.aword.low.abyte.low  = read(cpu->PC.aword+1);\
      t.aword.low.abyte.high = read(cpu->PC.aword+2);\
      cpu->PC.aword+=3;PUSH_PC;\
      cpu->PC.aword = t.aword.low.aword;\
      return 17;

#define M_CALL_NZ_nn  if ( cpu->AF.abyte.low & FLAG_Z ) { cpu->PC.aword+=3; return 10; } else { M_CALL_nn; }
#define M_CALL_Z_nn if ( cpu->AF.abyte.low & FLAG_Z ) { M_CALL_nn; } else { cpu->PC.aword+=3; return 10; }
#define M_CALL_NC_nn  if ( cpu->AF.abyte.low & FLAG_C ) { cpu->PC.aword+=3; return 10; } else { M_CALL_nn; }
#define M_CALL_C_nn if ( cpu->AF.abyte.low & FLAG_C ) { M_CALL_nn; } else { cpu->PC.aword+=3; return 10; }
#define M_CALL_PO_nn  if ( cpu->AF.abyte.low & FLAG_PV) { cpu->PC.aword+=3; return 10; } else { M_CALL_nn; }
#define M_CALL_PE_nn  if ( cpu->AF.abyte.low & FLAG_PV) { M_CALL_nn; } else { cpu->PC.aword+=3; return 10; }
#define M_CALL_P_nn if ( cpu->AF.abyte.low & FLAG_S ) { cpu->PC.aword+=3; return 10; } else { M_CALL_nn; }
#define M_CALL_N_nn if ( cpu->AF.abyte.low & FLAG_S ) { M_CALL_nn; } else { cpu->PC.aword+=3; return 10; }

#define M_RET   POP_PC; return 10;
#define M_RET_NZ  if ( cpu->AF.abyte.low & FLAG_Z ) { cpu->PC.aword++; return 5; } else { POP_PC; return 11; }
#define M_RET_Z   if ( cpu->AF.abyte.low & FLAG_Z ) { POP_PC; return 11; } else { cpu->PC.aword++; return 5; }
#define M_RET_NC  if ( cpu->AF.abyte.low & FLAG_C ) { cpu->PC.aword++; return 5; } else { POP_PC; return 11; }
#define M_RET_C   if ( cpu->AF.abyte.low & FLAG_C ) { POP_PC; return 11; } else { cpu->PC.aword++; return 5; }
#define M_RET_PO  if ( cpu->AF.abyte.low & FLAG_PV) { cpu->PC.aword++; return 5; } else { POP_PC; return 11; }
#define M_RET_PE  if ( cpu->AF.abyte.low & FLAG_PV) { POP_PC; return 11; } else { cpu->PC.aword++; return 5; }
#define M_RET_P   if ( cpu->AF.abyte.low & FLAG_S ) { cpu->PC.aword++; return 5; } else { POP_PC; return 11; }
#define M_RET_N   if ( cpu->AF.abyte.low & FLAG_S ) { POP_PC; return 11; } else { cpu->PC.aword++; return 5; }

#define M_DJNZ_e  cpu->BC.abyte.high--; if (cpu->BC.abyte.high==0) { cpu->PC.aword+=2; return 8; } else { JR_e; return 13; }


#define M_IN_A_n  cpu->AF.abyte.high = in(read(cpu->PC.aword+1)); cpu->PC.aword+=2; return 11;
#define M_OUT_n_A out(read(cpu->PC.aword+1),cpu->AF.abyte.high); cpu->PC.aword+=2; return 11;

#define M_ADD16(a,b)  t.adword = (u32)cpu->a.aword + (u32)cpu->b.aword;\
      cpu->AF.abyte.low = (cpu->AF.abyte.low & ~(FLAG_H|FLAG_N|FLAG_C|FLAG_3|FLAG_5)) |\
             (t.aword.high.abyte.low & FLAG_C) |\
             ((t.aword.low.aword^cpu->a.aword^cpu->b.aword)&0x1000?FLAG_H:0)|\
             (t.aword.low.abyte.high&(FLAG_3|FLAG_5));\
      cpu->a.aword = t.aword.low.aword;

#define M_ADC16(a,b)  tt.adword = (u32)cpu->b.aword + (u32)(cpu->AF.abyte.low & FLAG_C);\
      t.adword = (u32)cpu->a.aword + tt.adword;\
      cpu->AF.abyte.low = (t.aword.low.abyte.high & FLAG_S) |\
             (t.aword.low.aword==0?FLAG_Z:0) |\
             (t.aword.high.abyte.low & FLAG_C) |\
             (((t.aword.low.abyte.high^cpu->a.abyte.high^cpu->b.abyte.high) & 0x01)?FLAG_H:0)|\
             ((~(cpu->a.aword^cpu->b.aword)&(cpu->b.aword^t.aword.low.aword)&0x8000)?FLAG_PV:0)|\
             (t.aword.low.abyte.high&(FLAG_3|FLAG_5));\
      cpu->a.aword = t.aword.low.aword;

#define M_SBC16(a,b)  tt.adword = (u32)cpu->b.aword + (u32)(cpu->AF.abyte.low & FLAG_C);\
      t.adword = (u32)cpu->a.aword - tt.adword;\
      cpu->AF.abyte.low = (t.aword.low.abyte.high & FLAG_S) |\
             (t.aword.low.aword==0?FLAG_Z:0) |\
             (t.aword.high.abyte.low & FLAG_C) |\
             (((t.aword.low.aword^cpu->a.aword^cpu->b.aword) & 0x0100)?FLAG_H:0)|\
             (((cpu->a.aword^cpu->b.aword)&(cpu->a.aword^t.aword.low.aword)&0x8000)?FLAG_PV:0)|\
             (t.aword.low.abyte.high&(FLAG_3|FLAG_5));\
      cpu->a.aword = t.aword.low.aword;

#define M_INC(a)  a++;\
      cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) |\
             SZTable[a] |\
             ((a&0x0f)?0:FLAG_H) |\
             (a==0x80?FLAG_PV:0) |\
             (a&(FLAG_3|FLAG_5));

#define M_DEC(a)  a--;\
      cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) |\
             SZTable[a] |\
             ((a&0x0f)==0x0f?FLAG_H:0) |\
             (a==0x7f?FLAG_PV:0) |\
             FLAG_N |\
             (a&(FLAG_3|FLAG_5));

#define M_ADD(b)  t.aword.low.aword = (u16)cpu->AF.abyte.high + (u16)b;\
      cpu->AF.abyte.low = SZTable[t.aword.low.abyte.low] |        /* S,Z */\
             ((t.aword.low.abyte.low^cpu->AF.abyte.high^b)&FLAG_H) |    /* H   */\
             ((~(cpu->AF.abyte.high^b)&(b^t.aword.low.abyte.low)&0x80)?FLAG_PV:0) | /* V   */\
             t.aword.low.abyte.high |         /* C   */\
             (t.aword.low.abyte.low & (FLAG_3|FLAG_5));     /* 3,5 */\
      cpu->AF.abyte.high = t.aword.low.abyte.low;

#define M_ADC(b)  dw.aword = (u16)b + (u16)(cpu->AF.abyte.low&FLAG_C);\
      t.aword.low.aword = (u16)cpu->AF.abyte.high + dw.aword;\
      cpu->AF.abyte.low = SZTable[t.aword.low.abyte.low] |        /* S,Z */\
             ((t.aword.low.abyte.low^cpu->AF.abyte.high^b)&FLAG_H) |    /* H   */\
             ((~(cpu->AF.abyte.high^b)&(b^t.aword.low.abyte.low)&0x80)?FLAG_PV:0) | /* V   */\
             t.aword.low.abyte.high |         /* C   */\
             (t.aword.low.abyte.low&(FLAG_3|FLAG_5));\
      cpu->AF.abyte.high = t.aword.low.abyte.low;

#define M_SUB(b)  t.aword.low.aword = (u16)cpu->AF.abyte.high - (u16)b;\
      cpu->AF.abyte.low = SZTable[t.aword.low.abyte.low] |        /* S,Z */\
             ((t.aword.low.abyte.low^cpu->AF.abyte.high^b)&FLAG_H) |    /* H   */\
             (((cpu->AF.abyte.high^b)&(cpu->AF.abyte.high^t.aword.low.abyte.low)&0x80)>>5) |  /* V   */\
             -t.aword.low.abyte.high |          /* C   */\
             FLAG_N |             /* N   */\
             (t.aword.low.abyte.low&(FLAG_3|FLAG_5));\
      cpu->AF.abyte.high = t.aword.low.abyte.low;

#define M_SBC(b)  dw.aword = (u16)b + (u16)(cpu->AF.abyte.low&FLAG_C);\
      t.aword.low.aword = (u16)cpu->AF.abyte.high - dw.aword;\
      cpu->AF.abyte.low = SZTable[t.aword.low.abyte.low] |        /* S,Z */\
             ((t.aword.low.abyte.low^cpu->AF.abyte.high^b)&FLAG_H) |    /* H   */\
             (((cpu->AF.abyte.high^b)&(cpu->AF.abyte.high^t.aword.low.abyte.low)&0x80)?FLAG_PV:0) | /* V   */\
             -t.aword.low.abyte.high |          /* C   */\
             FLAG_N |             /* N   */\
             (dw.abyte.low&(FLAG_3|FLAG_5));\
      cpu->AF.abyte.high = t.aword.low.abyte.low;

#define M_CP(b) t.aword.low.aword = (u16)cpu->AF.abyte.high - (u16)b;\
      cpu->AF.abyte.low = SZTable[t.aword.low.abyte.low] |        /* S,Z */\
             ((t.aword.low.abyte.low^cpu->AF.abyte.high^b)&FLAG_H) |    /* H   */\
             (((cpu->AF.abyte.high^b)&(cpu->AF.abyte.high^t.aword.low.abyte.low)&0x80)?FLAG_PV:0) | /* V   */\
             -t.aword.low.abyte.high |          /* C   */\
             FLAG_N |             /* N   */\
             (cpu->AF.abyte.high & (FLAG_3|FLAG_5));          /* 3,5 */


#define M_AND(b)  cpu->AF.abyte.high &= b;\
      cpu->AF.abyte.low = PSZTable[cpu->AF.abyte.high] |  /* P,S,Z */\
             FLAG_H;        /* H */

#define M_OR(b)   cpu->AF.abyte.high |= b;\
      cpu->AF.abyte.low = PSZTable[cpu->AF.abyte.high];   /* P,S,Z */

#define M_XOR(b)  cpu->AF.abyte.high ^= b;\
      cpu->AF.abyte.low = PSZTable[cpu->AF.abyte.high];   /* P,S,Z */


#define M_EX(a,b) t.aword.low.aword = cpu->a.aword;\
      cpu->a.aword = cpu->b.aword;\
      cpu->b.aword = t.aword.low.aword;


#define M_BIT(a,b)  cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) | PSZTable[a&b] | FLAG_H;

#define M_RLC(a)  t.aword.low.abyte.low = (a&0x80)>>7;\
      a = (a<<1) | t.aword.low.abyte.low;\
      cpu->AF.abyte.low = PSZTable[a] | t.aword.low.abyte.low;

#define M_RL(a)   t.aword.low.abyte.low = (a&0x80)>>7;\
      a = (a<<1) | (cpu->AF.abyte.low & FLAG_C);\
      cpu->AF.abyte.low = PSZTable[a] | t.aword.low.abyte.low;

#define M_RRC(a)  t.aword.low.abyte.low = a & FLAG_C;\
      a = (a>>1) | (t.aword.low.abyte.low<<7);\
      cpu->AF.abyte.low = PSZTable[a] | t.aword.low.abyte.low;

#define M_RR(a)   t.aword.low.abyte.low = a & FLAG_C;\
      a = (a>>1) | ((cpu->AF.abyte.low & FLAG_C)<<7);\
      cpu->AF.abyte.low = PSZTable[a] | t.aword.low.abyte.low;

#define M_SLA(a)  t.aword.low.abyte.low = (a&0x80)>>7;\
      a <<= 1;\
      cpu->AF.abyte.low = PSZTable[a] | t.aword.low.abyte.low;

#define M_SRA(a)  t.aword.low.abyte.low = a & FLAG_C;\
      a = (a>>1) | (a&0x80);\
      cpu->AF.abyte.low = PSZTable[a] | t.aword.low.abyte.low;

#define M_SRL(a)  t.aword.low.abyte.low = a & FLAG_C;\
      a >>= 1;\
      cpu->AF.abyte.low = PSZTable[a] | t.aword.low.abyte.low;


#define M_INI   dw.abyte.low = in(cpu->BC.abyte.low);\
      write(cpu->HL.aword,dw.abyte.low);\
      cpu->BC.abyte.high--;\
      cpu->HL.aword++;\
      cpu->AF.abyte.low = (cpu->BC.abyte.high==0?FLAG_Z:0)|\
             (cpu->BC.abyte.high&(FLAG_3|FLAG_5|FLAG_S))|\
             ((dw.abyte.low>>6)&FLAG_N)|\
             (((u16)(cpu->BC.abyte.low+1)+(u16)dw.abyte.low)&0xff00?(FLAG_C|FLAG_H):0);

#define M_IND   dw.abyte.low = in(cpu->BC.abyte.low);\
      write(cpu->HL.aword,dw.abyte.low);\
      cpu->BC.abyte.high--;\
      cpu->HL.aword--;\
      cpu->AF.abyte.low = (cpu->BC.abyte.high==0?FLAG_Z:0)|\
             (cpu->BC.abyte.high&(FLAG_3|FLAG_5|FLAG_S))|\
             ((dw.abyte.low>>6)&FLAG_N)|\
             (((u16)(cpu->BC.abyte.low-1)+(u16)dw.abyte.low)&0xff00?(FLAG_C|FLAG_H):0);

#define M_OUTI    dw.abyte.low = read(cpu->HL.aword);\
      out(cpu->BC.abyte.low, dw.abyte.low);\
      cpu->BC.abyte.high--;\
      cpu->HL.aword++;\
      cpu->AF.abyte.low = (cpu->BC.abyte.high==0?FLAG_Z:0)|\
             (cpu->BC.abyte.high&(FLAG_3|FLAG_5|FLAG_S))|\
             ((dw.abyte.low>>6)&FLAG_N)|\
             (((u16)(cpu->BC.abyte.low+1)+(u16)dw.abyte.low)&0xff00?(FLAG_C|FLAG_H):0);

#define M_OUTD    dw.abyte.low = read(cpu->HL.aword);\
      out(cpu->BC.abyte.low, dw.abyte.low);\
      cpu->BC.abyte.high--;\
      cpu->HL.aword--;\
      cpu->AF.abyte.low = (cpu->BC.abyte.high==0?FLAG_Z:0)|\
             (cpu->BC.abyte.high&(FLAG_3|FLAG_5|FLAG_S))|\
             ((dw.abyte.low>>6)&FLAG_N)|\
             (((u16)(cpu->BC.abyte.low-1)+(u16)dw.abyte.low)&0xff00?(FLAG_C|FLAG_H):0);

#define M_LDI   write(cpu->DE.aword,read(cpu->HL.aword));\
      cpu->DE.aword++;cpu->HL.aword++;cpu->BC.aword--;\
      cpu->AF.abyte.low = (cpu->AF.abyte.low & ~(FLAG_N|FLAG_H|FLAG_PV)) | (cpu->BC.aword==0?0:FLAG_PV);

#define M_LDD   write(cpu->DE.aword,read(cpu->HL.aword));\
      cpu->DE.aword--;cpu->HL.aword--;cpu->BC.aword--;\
      cpu->AF.abyte.low = (cpu->AF.abyte.low & ~(FLAG_N|FLAG_H|FLAG_PV)) | (cpu->BC.aword==0?0:FLAG_PV);

#define M_CPI   dw.abyte.low = read(cpu->HL.aword);\
      t.aword.low.aword = cpu->AF.abyte.high - dw.abyte.low;\
      cpu->HL.aword++;cpu->BC.aword--;\
      cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) |\
             SZTable[t.aword.low.abyte.low] |\
               ((t.aword.low.abyte.low^cpu->AF.abyte.high^dw.abyte.low)&FLAG_H) |\
             (cpu->BC.aword==0?0:FLAG_PV) |\
             FLAG_N;

#define M_CPD   dw.abyte.low = read(cpu->HL.aword);\
      t.aword.low.aword = cpu->AF.abyte.high - dw.abyte.low;\
      cpu->HL.aword--;cpu->BC.aword--;\
      cpu->AF.abyte.low = (cpu->AF.abyte.low & FLAG_C) |\
             SZTable[t.aword.low.abyte.low] |\
               ((t.aword.low.abyte.low^cpu->AF.abyte.high^dw.abyte.low)&FLAG_H) |\
             (cpu->BC.aword==0?0:FLAG_PV) |\
             FLAG_N;

#endif /* __Z80_MACROS_H__ */

