/* Z80.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Z80.h"
#include "inctab.h"
#include "dectab.h"
#include "cptab.h"
#include "addtab.h"
#include "Z80_macros.h"
#include "Z80_Disasm.h"
#include "../GGMS.h"
/* #include "Z80_instructions.h" */

extern const u8 DAATable[];
extern int debug;

#ifdef GP32
#include "gpfont.h"
#include "gpgraphic.h"
#include "gpstdlib.h"

extern GPDRAWSURFACE gpDraw[2];
#endif

static u8 **smspages;
static GGMS *machine;

#ifdef NON_OO
static union_word cpuAF,cpuBC,cpuDE,cpuHL,cpuSP,cpuPC,cpuIX,cpuIY;
static union_word cpuaf,cpubc,cpude,cpuhl;
static u8 cpuIFF;
static u8 cpudata[4];
#endif


#ifdef MASTERLATOR
void writem(u16 a, u8 d) {

  if (a >= 0xc000) { /* WRAM */

    machine->ram[a & 0x1fff] = d;

	  if (a >= 0xfffc) {

		  int i;
		  int p = a & 0x03;

			d &= 0x1f;

			if (machine->banks[p] != d) {

			  machine->banks[p] = d;

			  if (p >= 1)
  			  for (i = (p - 1) * 16; i < ((p - 1) * 16) + 16; i++)
  			    GGMS_UpdatePage(machine, i);
  			else
  			for (i = 32; i < 48; i++)
  			  GGMS_UpdatePage(machine, i);
      }
  	}
  } else
  if (a >= 0x8000) { /* SRAM */

    if ((int)(machine->banks[0] & 0x08) != 0) {

      machine->sram[(a & 0x1fff) | ((u16)machine->banks[0] & 0x04) << 12] = d;
      machine->sram_changed = TRUE;
    }
  }
}

u8 readm(u16 a) {

	return smspages[a >> 10][a & 0x3ff];
}
#endif

int Z80_IllegalInstruction(Z80 *cpu) {

    int i;
/*     printf ("Illegal opcode at %08x\n",GGMS_Addy2ROM((GGMS*)cpu_val, cpu_PC.aword)); */
		printf("Illegal instruction: 0x");

		for (i = 0; i < 4; i++) {

			printf("%02x", read(cpu_getPC + i));
		}
		printf("\n");
    cpu_setPC(cpu_getPC + 1);
    return 4;
  }

void Z80_Reset(Z80 *cpu) {

  cpu->I = 0;
  cpu->R = 0;
  cpu_setAF(0);
  cpu_setaf(0);
  cpu_setBC(0);
  cpu_setbc(0);
  cpu_setDE(0);
  cpu_setde(0);
  cpu_setHL(0);
  cpu_sethl(0);
  cpu_setSP(0);
  cpu_setPC(0);
  cpu_setIX(0);
  cpu_setIY(0);
  cpu_IFF = 0;
  cpu->cyclecount = 0;
  cpu->eicount = -1;
}

void Z80_Init(Z80 *cpu, u8 (*readMem)(Z80*, u16), void (*writeMem)(Z80*, u16, u8), u8 (*in)(Z80*, u8), void (*out)(Z80*, u8, u8)) {

  /*  Initialize registers */
  Z80_Reset(cpu);
  cpu->readMem = (u8(*)(void*,u16))readMem;
  cpu->writeMem = (void(*)(void*,u16,u8))writeMem;
  cpu->in = (u8(*)(void*,u8))in;
  cpu->out = (void(*)(void*,u8,u8))out;
  cpu->curInst = NULL;
  cpu->rdy = FALSE;
  cpu->vec = 0;
    
  smspages = ((GGMS*)cpu->val)->pages;
  machine = cpu->val;
}

void Z80_Terminate(Z80 *cpu) {

  Z80_Reset ( cpu );
}

/*  Decode instructions with the DDCB prefix */
/*
int Z80_Decode_DDCB ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,t2;
    union_word  dw;

#define CYCLES(a) return a;
#include "decode_ddcb.h"

  }
*/

/*  Decode instructions with the FDCB prefix */
/*
int Z80_Decode_FDCB ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,t2;
    union_word  dw;

#define CYCLES(a) return a;
#include "decode_fdcb.h"

  }
*/

/*  Decode instructions with the ED prefix */
/*
int Z80_Decode_ED ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,tt;
    union_word  dw;

#define CYCLES(a) return a;
#include "decode_ed.h"
  }
*/

/*  Decode instructions with the DD prefix */
/*
int Z80_Decode_DD ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,t2;
    union_word  dw;

#define CYCLES(a) return a;
#include "decode_dd.h"
  }
*/

/*  Decode instructions with the FD prefix */
/*
int Z80_Decode_FD ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,t2;
    union_word  dw;

#define CYCLES(a) return a;
#include "decode_fd.h"
  }
*/

/*  Decode instructions with the CB prefix */
/*
int Z80_Decode_CB ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t,t2;
    union_word  dw;

#define CYCLES(a) return a;
#include "decode_cb.h"
  }
*/

/*  Decode the next instruction at cpu_PC */
/*  and determine the corresponding function */
/*
int Z80_Decode ( Z80 * x )
  {
    Z80 * cpu = x;
    union_dword t, tt, t2;
    union_word  dw;

#define CYCLES(a) return a
#include "decode.h"
  }
*/


/*  Create an interrupt */
void Z80_Int( Z80 *cpu, u16 vector) {

  if (vector == Z80_NMI) {

    cpu_IFF |= 0x20;   /*  NMI */
  } else {/* if ( cpu_IFF&1 ) */

    cpu_IFF |= 0x10;   /*  INT */
  }
}


int Z80_ExecInt(Z80 *cpu) {

  /*  Any pending interrupts ? */
  if (cpu_IFF & 0x20) { /*  NMI */

    /*  Has the cpu been halted ? */
    if (cpu_IFF & 0x80) {

      cpu_IFF &= ~0x80;
      cpu_setPC(cpu_getPC + 1);
    }

    /*  ** Reset bit5 and bit1=bit0 */
    cpu_IFF = (cpu_IFF & 0xdc) | ((cpu_IFF & 0x01) << 1);
/*         cpu_IFF &= ~0x01; */
        /*  ** Now restart at 0x66 */
        /*  Push PC onto the stack */
    write(cpu_getSP - 1, cpu_getPCh);
    write(cpu_getSP - 2, cpu_getPCl);
    cpu_setSP(cpu_getSP - 2);
    /*  JMP to 0x0066 */
    cpu_setPC(0x66);

    cpu->cyclecount = 11;

    return 11;
  } else
  if ((cpu_IFF & 0x10) && (cpu_IFF & 1)) { /*  INT */

    /*  Has the cpu been halted ? */
    if (cpu_IFF & 0x80) {

      cpu_IFF &= ~0x80;
      cpu_setPC(cpu_getPC + 1);
    }

    cpu_IFF &= ~0x10;
    cpu_IFF &= ~0x03; /*  Disable interrupts */
    /*  ** Now restart at 0x38 */
    /*  Push PC onto the stack */
    write(cpu_getSP - 1, cpu_getPCh);
    write(cpu_getSP - 2, cpu_getPCl);
    cpu_setSP(cpu_getSP - 2);
    /*  JMP to 0x0038 */
    cpu_setPC(0x38);

    cpu->cyclecount = 11;

    return 11;
  }
  return 0;
}

int Z80_Disassemble(u8 *op, char *dest) {

  char *inst;
  unsigned int i, j, l, ofs = 0;

  switch (op[0]) {

    case 0xcb:
      inst = Z80_DisasmCB[op[1]];
      l = Z80_LengthCB[op[1]];
      break;

    case 0xdd:
      if (op[1] == 0xcb) {

        inst = Z80_DisasmDDCB[op[3]];
        l = Z80_LengthDDCB[op[3]];
      } else {

        inst = Z80_DisasmDD[op[1]];
        l = Z80_LengthDD[op[1]];

        if (!inst) {

          ofs = 1;
          inst = Z80_Disasm[op[1]];
          l = Z80_Length[op[1]]+1;
        }
      }
      break;

    case 0xed:
      inst = Z80_DisasmED[op[1]];
      l = Z80_LengthED[op[1]];
      break;

    case 0xfd:
      if (op[1] == 0xcb) {

        inst = Z80_DisasmFDCB[op[3]];
        l = Z80_LengthFDCB[op[3]];
      } else {

        inst = Z80_DisasmFD[op[1]];
        l = Z80_LengthFD[op[1]];

        if (!inst) {

          ofs = 1;
          inst = Z80_Disasm[op[1]];
          l = Z80_Length[op[1]]+1;
        }
      }
      break;

    default:
      inst = Z80_Disasm[op[0]];
      l = Z80_Length[op[0]];
      break;
  }

  if (!inst) {

    sprintf(dest, "DB %02Xh", op[0]);
    return 1;
  }

  for (i = j = 0; i < strlen(inst);) {

    if (((inst[i] == '%') || (inst[i] == '$')) &&
         (inst[i + 1] >= '0') && (inst[i + 1] <= '9')) {

      char t[8];
      u8 r;
      u16 v;

      v = inst[i+1]-'0';
      r = op[v+ofs];

      if (inst[i] == '$') {

        if (r & 0x80) {

          sprintf(t,"-%02X",(-r)&0xff);
        } else {
          
          sprintf(t,"+%02X",r);
        }
      } else {

        sprintf(t, "%02X", r);
      }

      strcpy(&dest[j], t);
      j += strlen(t);
      i += 2;
    } else {

      dest[j++] = inst[i++];
    }
  }

  dest[j] = '\0';
  return l;
}

/*  Execute a single instruction */
int Z80_Exec(Z80 *cpu) {

  cpu->R++;

  if (cpu_IFF & 0x40) {

    cpu_IFF &= ~0x40;
  }

  if (cpu->eicount > 0) {

    cpu->eicount--;
  } else
  if (cpu->eicount == 0) {

    cpu_IFF |= 0x03;
    cpu->eicount = -1;
  }

  /*  If the last instruction was an EI, */
  /*  set IFF1 and IFF2 to 1 */
  if (cpu_IFF & 0x80) {

    cpu->cyclecount = 4;
    return 4;
  }

  /*  Execute current instruction */
/*     cpu->cyclecount = Z80_Decode ( cpu ); */
/*     return cpu->cyclecount;*/

/*    cpu->cyclecount = Z80_Decode(cpu);*/

  return (cpu->cyclecount);
}


/*  One CLK tick */
void Z80_Cycle(Z80 *cpu) {

  cpu->rdy = FALSE;

  if (cpu->cyclecount == 0) {

    /*  It's the very first cycle */
    if (Z80_ExecInt(cpu) != -1) {
      
      return;
    }
    Z80_Exec(cpu);
  } else {

    cpu->cyclecount--;
    if (cpu->cyclecount == 0) {

      if (Z80_ExecInt(cpu) != -1) {
        
        return;
      }

      Z80_Exec(cpu);
    }
  }
}

int Z80_Run(Z80 *cpu, int c) { /*  Run at least c cycles */

  int i = 0, x = 0;

/*      x = Z80_ExecInt ( cpu );
      if (x!=-1)
        i += x;*/

  i += Z80_ExecInt(cpu);

  if ((cpu_IFF & 0x80) && (cpu->eicount <= 0)) {

    return c;
  }

  while (i < c) {

    /*  Any pending interrupts ? */
    cpu->R++;

    if (cpu->eicount > 0) {

      if (!--cpu->eicount) {

        cpu_IFF |= 0x03;
        cpu->eicount=-1;
        i += Z80_ExecInt(cpu);
      }
    }

    if (cpu_IFF & 0x80) {

      i +=  4;
    } else {

      union_dword t, tt, t2;
      union_word  dw;

#define CYCLES(a) i += a
#include "decode.h"
    }
  }
  return x;
}
