/* SN76489.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sn76489.h"
#include "sn76489noise.h"

static u16 SN76489_Volumes[16] = {

  65535,  /*  0 dB */
  52057,  /* -2 dB */
  41350,  /* -4 dB */
  32846,  /* -6 dB */
  26090,  /* -8 dB */
  20724,  /* -10dB */
  16462,  /* -12dB */
  13076,  /* -14dB */
  10387,  /* -16dB */
   8250,  /* -18dB */
   6554,  /* -20dB */
   5206,  /* -22dB */
   4135,  /* -24dB */
   3285,  /* -26dB */
   2609,  /* -28dB */
      0   /* Off */
};

void SN76489_Reset(SN76489 *snd) {

  snd->sqofs[0] = 0;
  snd->sqofs[1] = 0;
  snd->sqofs[2] = 0;
  snd->sqofs[3] = 0;
  snd->sqwidth[0] = 882 << 16;
  snd->sqwidth[1] = 882 << 16;
  snd->sqwidth[2] = 882 << 16;
  snd->sqwidth[3] = 882 << 16;
  snd->chanvol[0] = 15;
  snd->chanvol[1] = 15;
  snd->chanvol[2] = 15;
  snd->chanvol[3] = 15;
  snd->pos[0] = TRUE;
  snd->pos[1] = TRUE;
  snd->pos[2] = TRUE;
  snd->noiseofs = 0;
  snd->FB = FALSE;
}

void SN76489_Init(SN76489 *snd, u16 freq, u8 bits, u8 channels, u16 buflen) {

  int l;

  snd->freq = freq;
  snd->bits = bits;
  snd->channels = channels;
  snd->buflen = buflen;

  SN76489_Reset(snd);

  l = (bits / 8) * channels * buflen;
  snd->buf[0] = (void*)malloc(l);
  snd->buf[1] = (void*)malloc(l);
  snd->buf[2] = (void*)malloc(l);
  snd->buf[3] = (void*)malloc(l);

  snd->lastchan = 0;
}

void SN76489_Terminate(SN76489 *snd) {

  int i;

  for (i = 0; i < 4; i++) {

    if (snd->buf[i]) {

      free(snd->buf[i]);
			snd->buf[i] = NULL;
    }
  }
}

void SN76489_CalcVoice(SN76489 *snd, int voice, u16 *buf, BOOL overwrite) {

	s16 pos = (((snd->pos[voice] ? -32767 : 32767) * (s32)SN76489_Volumes[snd->chanvol[voice]]) >> 16) >> 2;
	u32 sqofs = snd->sqofs[voice];
	u32 sqwidth = snd->sqwidth[voice];
	int cLen = snd->buflen >> 4;
	int cRem = snd->buflen & ((1 << 4) - 1);
	int i;
  s16 pos2 = -pos;
	BOOL p = snd->pos[voice];

/*  pos2 ^= 0x8000;
  pos ^= 0x8000;*/

	if (overwrite) {

    for (i = 0; i < cLen; i++) {

      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
    }

	  for (i = 0; i < cRem; i++) {

      *buf++ = pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
 	  }
	} else {

    for (i = 0; i < cLen; i++) {

      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
    }

	  for (i = 0; i < cRem; i++) {

      *buf++ += pos; sqofs += 0x00010000; if (sqofs >= sqwidth) { s16 t; sqofs -= sqwidth; t = pos; pos = pos2; pos2 = t; p = !p; }
 	  }
  }

	snd->pos[voice] = p;
	snd->sqofs[voice] = sqofs;
}

void SN76489_Cycle(SN76489 *snd, u16 *dest) {

  int i,c;

	SN76489_CalcVoice(snd, 0, dest, TRUE);

  for (c = 1; c < 3; c++) {

		SN76489_CalcVoice(snd, c, dest, FALSE);
  }

  for (i = 0;i < snd->buflen ; i++) {

    u8 d;
    u32 w;

    if (!snd->FB) {

      w = snd->sqwidth[3] << 4;
      d = snd->noiseofs & 1;
    } else {

      w = snd->sqwidth[3];
      d = SN76489_Noise[snd->noiseofs];
    }

/*    ((s16*)snd->buf[3])[i] = ((d ? 32767 : -32768) *
                             (s32)SN76489_Volumes[snd->chanvol[c]]) >> 16;*/
    *dest++ += ((((d ? 32767 : -32767) * (s32)SN76489_Volumes[snd->chanvol[c]]) >> 16) >> 2);
    snd->sqofs[3] += 0x00010000;

    if (snd->sqofs[3] >= w) {

      snd->sqofs[3] -= w;
      snd->noiseofs++;

      if (snd->noiseofs > 32766) {

        snd->noiseofs = 0;
      }
    }
  }
}

void SN76489_Write(SN76489 *snd, u8 d) {

  if ((d & 0x90) == 0x90) {

    snd->chanvol[(d >> 5) & 3] = d & 0x0f;
  } else
  if ((d & 0x90) == 0x80) {

    u32 t;
    u8 c = (d >> 5) & 3;

    if (d == 3) {

    	return;
  	}

    snd->lastchan = c;

    if (c != 3) {

      snd->chanfreq[c] = (snd->chanfreq[c] & ~0x000f) | (d & 0x0f);
      t = (u32)snd->freq * (u32)snd->chanfreq[c];
      snd->sqwidth[c]  = (t / 125000) << 16;
      snd->sqwidth[c] |= (u32)((double)(t % 125000) * (double)(65535.0 / 125000.0));
      snd->sqwidth[c] >>= 1;

      if (snd->chan3f && (c == 2)) {

        snd->sqwidth[3] = snd->sqwidth[2];
      }
    }
  } else
  if ((d & 0x80) == 0x00) {

    u32 t;
    u8 c = snd->lastchan;

    if (c != 3) {

      snd->chanfreq[c] = (snd->chanfreq[c] & ~0xfff0) | ((d & 0x3f) << 4);
      t = (u32)snd->freq * (u32)snd->chanfreq[c];
      snd->sqwidth[c]  = (t / 125000) << 16;
      snd->sqwidth[c] |= (u32)((double)(t % 125000) * (double)(65535.0 / 125000.0));
      snd->sqwidth[c] >>= 1;

      if (snd->chan3f && (c == 2)) {

        snd->sqwidth[3] = snd->sqwidth[2];
      }
    }
  }

  if ((d & 0xf0) == 0xe0) {

    if (d & 0x04) {

      snd->FB = TRUE;
    } else {

      snd->FB = FALSE;
    }

    switch (d & 0x03) {

      case 0: /* 7800Hz */
        snd->chan3f = FALSE;
        snd->sqwidth[3] = (u32)((snd->freq / 7800) << 16) |
                          (u32)((double)(snd->freq % 7800) *
                                (double)(65535.0 / 7800.0));
        break;

      case 1: /* 3900Hz */
        snd->chan3f = FALSE;
        snd->sqwidth[3] = (u32)((snd->freq / 3900) << 16) |
                          (u32)((double)(snd->freq % 3900) *
                                (double)(65535.0 / 3900.0));
        break;

      case 2: /* 1950Hz */
        snd->chan3f = FALSE;
        snd->sqwidth[3] = (u32)((snd->freq / 1950) << 16) |
                          (u32)((double)(snd->freq % 1950) *
                                (double)(65535.0 / 1950.0));
        break;

      case 3: /* Channel 2 */
        snd->chan3f = TRUE;
        snd->sqwidth[3] = snd->sqwidth[2];
        break;
    }
  }
}
