/* main.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "getopts.h"
#include "sdl/sdl.h"
#include "config.h"
#include "emu.h"

int debug;

int loadEMUconfig(EMU *emu, int argc, char *argv[] ) {

  int bios = -1,
      fullscreen_stretch = -1,
      joystick = 1,
      fullscreen = -1,
      sound = -1,
      limit_speed = -1,
      frameskip = -1,
      jap = -1,
      key_up = -1,
      key_down = -1,
      key_left = -1,
      key_right = -1,
      key_a = -1,
      key_b = -1,
      key_start = -1;

  char *args;
  int i, l;
  Config cfg;
  char p[256];
  char romfile[256];

  struct options opts[] = {

    { 1,  "version", "Show version info", "V", 0 },
    { 2,  "bios", "Enable BIOS (on/off)", "b", 1 },
    { 3,  "stretch", "Stretch screen in fullscreen mode (on/off)", "s", 1 },
    { 4,  "joystick", "Enable Joystick/pad (on/off)", "j", 1 },
    { 5,  "fullscreen", "Enable fullscreen on startup (on/off)", "f", 1 },
    { 6,  "limitspeed", "Limit emulation speed to 60Hz (on/off)", "l", 1 },
    { 7,  "frameskip", "Set frameskipping (0..9)", "fs", 1 },
    { 8,  "key_up", "Set key for up button", "ku", 1 },
    { 9,  "key_down", "Set key for down button", "kd", 1 },
    { 10, "key_left", "Set key for left button", "kl", 1 },
    { 11, "key_right", "Set key for right button", "kr", 1 },
    { 12, "key_a", "Set key for A button", "ka", 1 },
    { 13, "key_b", "Set key for B button", "kb", 1 },
    { 14, "key_start", "Set key for Start button", "ks", 1 },
    { 15, "rom", "Load and start ROM file on startup", "r", 1 },
    { 16, "japanese", "Enable the Japanese language", "jp", 1 },
    { 0, NULL, NULL, NULL, 0 }
  };

  if (!emu) {

    return 1;
  }

  if ((l=strlen(argv[0])) == 0) {

    return 1;
  }

  romfile[0] = '\0';
  /* ** Parse the command line ** */
  args = (char *)calloc(1, 128);

  while ((i = getopts(argc, argv, opts, args)) != 0) {

    int res = 0, type = -1;
    char *message;

    switch (i) {

      case 1:   /* version */
        printf("Masterlator v0.07b Copyright (c)2001, 2002 by Christian Nowak <chnowak@web.de>\n");
        printf("Project website: http://chn.roarvgm.com/\n");
        break;

      case 2:   /* bios */
        type = 1;
        if ((res = bios = cfg.getContentsVal(args, type))<0) {

          bios = -1;
        }
        break;

      case 3:   /* stretch */
        type = 1;
        if ((res = fullscreen_stretch = cfg.getContentsVal(args, type))<0) {

          fullscreen_stretch = -1;
        }
        break;

      case 4:   /* joystick */
        type = 1;
        if ((res = joystick = cfg.getContentsVal(args, type))<0) {

          joystick = -1;
        }
        break;

      case 5:   /* fullscreen */
        type = 1;
        if ((res = fullscreen = cfg.getContentsVal(args, type))<0) {

          fullscreen = -1;
        }
        break;

      case 6:   /* limitspeed */
        type = 1;
        if ((res = limit_speed = cfg.getContentsVal(args, type))<0) {

          limit_speed = -1;
        }
        break;

      case 7:   /* frameskip */
        type = 0;
        if ((res = frameskip = cfg.getContentsVal(args, type))<0) {

          frameskip = -1;
        }
        break;

      case 8:   /* key_up */
        type = 2;
        if ((res = key_up = cfg.getContentsVal(args, type ))<0) {

          key_up = -1;
        }
        break;

      case 9:   /* key_down */
        type = 2;
        if ((res = key_down = cfg.getContentsVal(args, type))<0) {

          key_down = -1;
        }
        break;

      case 10:  /* key_left */
        type = 2;
        if ((res = key_left = cfg.getContentsVal( args, type))<0) {

          key_left = -1;
        }
        break;

      case 11:  /* key_right */
        type = 2;
        if ((res = key_right = cfg.getContentsVal(args, type))<0) {

          key_right = -1;
        }
        break;

      case 12:  /* key_a */
        type = 2;
        if ((res = key_a = cfg.getContentsVal(args, type))<0) {

          key_a = -1;
        }
        break;

      case 13:  /* key_b */
        type = 2;
        if ( (res = key_b = cfg.getContentsVal(args, type))<0) {

          key_b = -1;
        }
        break;

      case 14:  /* key_start */
        if ((res = key_start = cfg.getContentsVal(args, type))<0) {

          key_start = -1;
        }
        break;

      case 15:  /* rom */
        strcpy(romfile, args);
        break;

      case 16:  /* jap */
        type = 1;
        if ((res = jap = cfg.getContentsVal(args, type))<0) {

          jap = -1;
        }
        break;

      default:
        break;
    }

    if (type == 0) {

      message = "Integer value expected";
    } else
    if (type == 1) {

      message = "Boolean expected";
    } else
    if (type == 2) {

      message = "Key expected";
    } else {

      message = "Unknown error";
    }

    if (res < 0) {

      printf ("--%s : %s\n", opts[i-1].name, message);
    }

    memset(args, 0, 128);
  }

  /* ** Read the configuration file ** */
  for (i = l - 1; i >= 0; i--) {

    char c = argv[0][i];

    if ((c == ':') || (c == '\\') || (c == '/')) {

      i++;
      if (i >= l) {

        i--;
      }
      break;
    }
  }

  if (i > 0) {
	    
    memcpy(p, argv[0], i);
    p[i] = '\0';
  } else {
    
    p[0] = '\0';
  }

  strcat(p, "masterlator.cfg");

  if (cfg.load(p) == 0) {

    if (bios == -1) {

        bios = cfg.getValue("bios", 1);
    }

    if (fullscreen_stretch == -1) {

      fullscreen_stretch = cfg.getValue("fullscreen_stretch", 1);
    }

    if (joystick == -1) {

      joystick = cfg.getValue("joystick", 1);
    }

    if (fullscreen == -1) {

      fullscreen = cfg.getValue("fullscreen", 1);
    }

    if (sound == -1) {

      sound = cfg.getValue("sound", 1);
    }

    if (limit_speed == -1) {

      limit_speed = cfg.getValue("limit_speed", 1);
    }

    if (frameskip == -1) {

      frameskip = cfg.getValue("frameskip", 0);
    }

    if (key_up == -1) {

      key_up = cfg.getValue("key_up", 2);
    }

    if (key_down == -1) {

      key_down = cfg.getValue("key_down", 2);
    }

    if (key_left == -1) {

      key_left = cfg.getValue("key_left", 2);
    }

    if (key_right == -1) {

      key_right = cfg.getValue("key_right", 2);
    }

    if (key_a == -1) {

      key_a = cfg.getValue("key_a", 2);
    }

    if (key_b == -1) {

      key_b = cfg.getValue("key_b", 2);
    }

    if (key_start == -1) {

      key_start = cfg.getValue("key_start", 2);
    }

    if (jap == -1) {

      jap = cfg.getValue("japanese", 1);
    }

    cfg.destroy();
  } else {

    fprintf(stderr,"Couldn't load the configuration file at \"%s\" !\n", p);
  }

  if (sound == 1) {

    EMU_AudioEnable(emu);
    emu->audioen = TRUE;
  } else {

    EMU_AudioDisable(emu);
    emu->audioen = FALSE;
  }

  if ((frameskip < 0) || (frameskip > 9)) {

    frameskip = 0;
  }

  EMU_EnableBIOS(emu, bios < 0 ? FALSE : bios);
  EMU_FullStretchEnable(emu, fullscreen_stretch < 0 ? FALSE : fullscreen_stretch);
  EMU_SpeedLimitEnable(emu, limit_speed < 0 ? TRUE : limit_speed);
  EMU_JoystickEnable(emu, joystick < 0 ? FALSE : joystick);
  EMU_FullscreenEnable(emu, fullscreen < 0 ? FALSE : fullscreen);
  EMU_SetFrameSkip(emu, frameskip);

  emu->key_up = key_up < 0 ? SDLK_UP : key_up;
  emu->key_down = key_down < 0 ? SDLK_DOWN : key_down;
  emu->key_left = key_left < 0 ? SDLK_LEFT : key_left;
  emu->key_right = key_right < 0 ? SDLK_RIGHT : key_right;
  emu->key_a = key_a < 0 ? SDLK_z : key_a;
  emu->key_b = key_b < 0 ? SDLK_x : key_b;
  emu->key_start = key_start < 0 ? SDLK_c : key_start;
  emu->jap = jap < 0 ? FALSE : jap;

  if (romfile[0] != '\0') {

    EMU_LoadROM(emu, romfile);
  }

  return 0;
}

int main (int argc, char *argv[]) {

  EMU *emu = NULL;
  char *romfname = NULL;

  debug = 0;

  if (argc > 1) {

    if (argv[1][0] != '-') {

      romfname = argv[1];
    }
  }

  /* Initialize virtual machine */
  if ((emu = EMU_Create(romfname, argv)) == NULL) {

    fprintf(stderr,"Error initializing the virtual machine\n");
    return 1;
  }

  loadEMUconfig(emu, argc, argv);

  /* Main loop */
  while (1) {

    /* Do the main work */
    if (EMU_RunFrame(emu)) {
      
      break;
    }
    SDL_Delay(0);
  }

  EMU_Terminate (emu);
  return 0;
}
